unit UltraOptions;

// The contents of this file are subject to the Mozilla Public License
// Version 1.1 (the "License"); you maynot use this file except in compliance
// with the License. You may obtain a copy of the License at http://www.mozilla.org/MPL/
//
// Alternatively, you may redistribute this library, use and/or modify it under the terms of the
// GNU Lesser General Public License as published by the Free Software Foundation;
// either version 2.1 of the License, or (at your option) any later version.
// You may obtain a copy of the LGPL at http://www.gnu.org/copyleft/.
//
// Software distributed under the License is distributed on an "AS IS" basis,
// WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for the
// specific language governing rights and limitations under the License.

interface

uses
  Windows,
  Messages,
  SysUtils,
  Classes,
  Graphics,
  Controls,
  Forms,
  VirtualThumbnails,
  VirtualExplorerEasyListview,
  VirtualExplorerTree,
  MPShellUtilities,
  ActiveX,
  ShlObj,
  TntClasses,
  ATViewer,
  ATBinHex,
  ATxCodepages,
  EasyListview,
  TntForms,
  UnitEditGroupingCriteria,
  SpTBXItem;

const
  ULTRAOPTIONS_VERSION = 0;

type
  TUltraAttribs = class;

  TDefaultTabLocation = (
    dtlCurrent,
    dtlCustom
  );

  TDefaultStartupPath = (
    dspLast,
    dspCustom
  );

  TUltraButtonLayout = (
    ublGlyph,
    ublText,
    ublGlyphAndText
  );

  TUltraTabsetPosition = (
    utpTop,
    utpBottom
  );

  TPlugins = class(TCollection)
  end;

  TQueryModifyColorProc = function(var Color: TColor): Boolean of object;
  TQueryModifyFontProc = function(Font: TFont): Boolean of object;
  TSetAllWindowColorsProc = procedure(Color: TColor) of object;
  TSetAllWindowFontsProc = procedure(Font: TFont) of object;
  TLockAllColorsProc = function: Boolean of object;
  TLockAllFontsProc = function: Boolean of object;
  TIsDirtyProc = procedure(const Dirty: Boolean) of object;

  TStreamedBase = class(TComponent)
  protected
    procedure GetChildren(Proc: TGetChildProc; Root: TComponent); override;
    procedure SetParentComponent(Value: TComponent); override;
  end;

  TBaseAttribs = class(TStreamedBase)
  private
    FVersion: Integer;
  protected
    FColor: TColor;
    FFont: TFont;
    function GetColor: TColor;
    procedure SetColor(const Value: TColor);
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
  published
    property Color: TColor read GetColor write SetColor;
    property Font: TFont read FFont write FFont;
    property Version: Integer read FVersion write FVersion;
  end;

  TCmdLineAttribs = class(TBaseAttribs)
  private
    FMaxLineCount: Integer;
    FSyncWithFolder: Boolean;
  protected
    procedure SetParentComponent(Value: TComponent); override;
  public
    constructor Create(AOwner: TComponent); override;
  published
    property MaxLineCount: Integer read FMaxLineCount write FMaxLineCount default 8192;
    property SyncWithFolder: Boolean read FSyncWithFolder write FSyncWithFolder;
  end;

  TTreeviewAttribs = class(TBaseAttribs)
  private
    FAlwaysCollapseFirst: Boolean;
    FAutoHorzScroll: Boolean;
    FFullRowSelect: Boolean;
    FHilightCompressed: Boolean;
    FHotTrack: Boolean;
    FHoverExpand: Boolean;
    FShowBranchLines: Boolean;
    FShowTooltip: Boolean;
    FSingleClickExpand: Boolean;
    FHilightEncrypted: Boolean;
  protected
    procedure SetParentComponent(Value: TComponent); override;
  published
    property AlwaysCollapseFirst: Boolean read FAlwaysCollapseFirst write FAlwaysCollapseFirst;
    property AutoHorzScroll: Boolean read FAutoHorzScroll write FAutoHorzScroll;
    property FullRowSelect: Boolean read FFullRowSelect write FFullRowSelect;
    property HilightCompressed: Boolean read FHilightCompressed write FHilightCompressed;
    property HilightEncrypted: Boolean read FHilightEncrypted write FHilightEncrypted;
    property HotTrack: Boolean read FHotTrack write FHotTrack;
    property HoverExpand: Boolean read FHoverExpand write FHoverExpand;
    property ShowBranchLines: Boolean read FShowBranchLines write FShowBranchLines;
    property ShowTooltip: Boolean read FShowTooltip write FShowTooltip;
    property SingleClickExpand: Boolean read FSingleClickExpand write FSingleClickExpand;
  end;

  TGeneralAttribs = class(TBaseAttribs)
  private
    FAutoCheckForUpdates: Boolean;
    FDefaultPath: TDefaultStartupPath;
    FDefaultStartupPath: PItemIDList;
    FDefaultTab: TDefaultTabLocation;
    FDefaultTabPIDL: PItemIDList;
    FDualPane: Boolean;
    FDualPaneLink: Boolean;
    FFullPathInTabMenus: Boolean;
    FIncrementalDelayTime: Integer;
    FLanguage: Word;
    FLockWindowColors: Boolean;
    FLockWindowFonts: Boolean;
    FPathInCaption: Boolean;
    FProxyHost: string;
    FProxyPort: string;
    FSaveMappedDriveTabs: Boolean;
    FSaveNetworkPathTabs: Boolean;     // if False then Tabs to network paths are not saved to be restored
    FSaveRemovableDriveTabs: Boolean;  // if False then Tabs to removable drives are not saved to be restored
    FShowCloseButtons: Boolean;
    FShowFullPathInAddressbar: Boolean;
    FShowHints: Boolean;
    FShowTabPathHints: Boolean;
    FSingleInstance: Boolean;
    FSuppressNoDiskMessages: Boolean;
    FTrackSelectedSize: Boolean;
    FTrayIcon: Boolean;
    FUseOnByDefaultFlag: Boolean;   // On By Default Column Handler Flag
  protected
    procedure DefineProperties(Filer: TFiler); override;
    procedure LoadCustomStartupPath(Stream: TStream);
    procedure LoadCustomTabPath(Stream: TStream);
    procedure SaveCustomStartupPath(Stream: TStream);
    procedure SaveCustomTabPath(Stream: TStream);
    procedure SetParentComponent(Value: TComponent); override;
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    property DefaultStartupPath: PItemIDList read FDefaultStartupPath write FDefaultStartupPath;
    property DefaultTabPIDL: PItemIDList read FDefaultTabPIDL write FDefaultTabPIDL;
  published
    property AutoCheckForUpdates: Boolean read FAutoCheckForUpdates write FAutoCheckForUpdates;
    property DefaultPath: TDefaultStartupPath read FDefaultPath write FDefaultPath;
    property DefaultTab: TDefaultTabLocation read FDefaultTab write FDefaultTab;
    property DualPane: Boolean read FDualPane write FDualPane;
    property DualPaneLink: Boolean read FDualPaneLink write FDualPaneLink;
    property FullPathInTabMenus: Boolean read FFullPathInTabMenus write FFullPathInTabMenus;
    property IncrementalDelayTime: Integer read FIncrementalDelayTime write FIncrementalDelayTime;
    property Language: Word read FLanguage write FLanguage;
    property LockWindowColors: Boolean read FLockWindowColors write FLockWindowColors;
    property LockWindowFonts: Boolean read FLockWindowFonts write FLockWindowFonts;
    property PathInCaption: Boolean read FPathInCaption write FPathInCaption;
    property ProxyHost: string read FProxyHost write FProxyHost;
    property ProxyPort: string read FProxyPort write FProxyPort;
    property SaveMappedDriveTabs: Boolean read FSaveMappedDriveTabs write FSaveMappedDriveTabs;
    property SaveNetworkPathTabs: Boolean read FSaveNetworkPathTabs write FSaveNetworkPathTabs;
    property SaveRemovableDriveTabs: Boolean read FSaveRemovableDriveTabs write FSaveRemovableDriveTabs;
    property ShowCloseButtons: Boolean read FShowCloseButtons write FShowCloseButtons;
    property ShowFullPathInAddressbar: Boolean read FShowFullPathInAddressbar write FShowFullPathInAddressbar;
    property ShowHints: Boolean read FShowHints write FShowHints;
    property ShowTabPathHints: Boolean read FShowTabPathHints write FShowTabPathHints;
    property SingleInstance: Boolean read FSingleInstance write FSingleInstance;
    property SuppressNoDiskMessages: Boolean read FSuppressNoDiskMessages write FSuppressNoDiskMessages;
    property TrackSelectedSize: Boolean read FTrackSelectedSize write FTrackSelectedSize;
    property TrayIcon: Boolean read FTrayIcon write FTrayIcon;
    property UseOnByDefaultFlag: Boolean read FUseOnByDefaultFlag write FUseOnByDefaultFlag;
  end;

  TTabAttribs = class(TBaseAttribs)
  private
    FMaxWidth: Integer;
    FPosition: TUltraTabsetPosition;
  protected
    procedure SetParentComponent(Value: TComponent); override;
  public
    constructor Create(AOwner: TComponent); override;
  published
    property MaxWidth: Integer read FMaxWidth write FMaxWidth;
    property Position: TUltraTabsetPosition read FPosition write FPosition;
  end;

  TListviewAttribs = class(TBaseAttribs)
  private
    FAlwaysShowHeader: Boolean;
    FAspectRatio: extended;
    FAutoOrientation: Boolean;
    FAutoSizeCells: Boolean;
    FColumnHilightBandColor: TColor;
    FColumnHilightBanding: Boolean;
    FColumnsShownInHeaderMenu: Integer;
    FCopyPrevView: Boolean;
    FDefaultColumn: Integer;  // Column used for default sorting
    FDefaultColumnModeWidth: Integer;
    FDefaultGroup: Integer;   // Column used for default grouping
    FDefaultView: Integer;    // 0 = Icon, 1 = SmallIcon, 2 = List etc.
    FExtensionColorCode: Boolean;
    FExtensionColorCodeList: TExtensionColorCodeList;
    FFileSizeStyle: Integer;
    FFocusRect: Boolean;
    FFoldersAlwaysFirst: Boolean;
    FFolderSizeDisplay: TBtyeSize;
    FFolderSizeShow: Boolean;
    FFullRowSelect: Boolean;
    FGradientSelection: Boolean;
    FGridLines: Boolean;
    FHideCaptions: Boolean;
    FHilightCompressed: Boolean;
    FHilightEncrypted: Boolean;
    FHotTrack: Boolean;
    FQueryInfoHintTimout: LongWord;
    FShowTitleBar: Boolean;
    FFileObject: TFileObjects;
    FGroupedByDefault: Boolean;
    FHideTabsetWhenOneTab: Boolean;
    FHilightColumn: Boolean;
    FHilightColumnColor: TColor;
    FQueryInfoHints: Boolean;
    FSavePerFolder: Boolean;
    FSelectFirstItem: Boolean;
    FShellThumbs: Boolean;
    FSimpleChangeNotify: Boolean; // depreciated not that I can change just the PIDL and not recreate the object.  Minimal speed hit
    FSmoothScroll: Integer;
    FSortDirection: TEasySortDirection;
    FThumbsCache: Boolean;
    FThumbsCacheFile: WideString;
    FThumbsCompress: Boolean;
    FThumbsHeight: Integer;
    FThumbsLoadAll: Boolean;
    FThumbsSingleFile: Boolean;
    FThumbsWidth: Integer;
    FUpLevelOnDblClk: Boolean;
    FUseOnByDefaultFlag: Boolean;
  protected
    procedure SetParentComponent(Value: TComponent); override;
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    property ExtensionColorCodeList: TExtensionColorCodeList read FExtensionColorCodeList write FExtensionColorCodeList;
  published
    property AlwaysShowHeader: Boolean read FAlwaysShowHeader write FAlwaysShowHeader;
    property AspectRatio: extended read FAspectRatio write FAspectRatio;
    property AutoOrientation: Boolean read FAutoOrientation write FAutoOrientation;
    property AutoSizeCells: Boolean read FAutoSizeCells write FAutoSizeCells;
    property ColumnHilightBandColor: TColor read FColumnHilightBandColor write FColumnHilightBandColor default $00F7F7F7;
    property ColumnHilightBanding: Boolean read FColumnHilightBanding write FColumnHilightBanding default True;
    property ColumnsShownInHeaderMenu: Integer read FColumnsShownInHeaderMenu write FColumnsShownInHeaderMenu;
    property CopyPrevView: Boolean read FCopyPrevView write FCopyPrevView;
    property DefaultColumn: Integer read FDefaultColumn write FDefaultColumn;
    property DefaultColumnModeWidth: Integer read FDefaultColumnModeWidth write FDefaultColumnModeWidth;
    property DefaultGroup: Integer read FDefaultGroup write FDefaultGroup;
    property DefaultView: Integer read FDefaultView write FDefaultView;
    property ExtensionColorCode: Boolean read FExtensionColorCode write FExtensionColorCode;
    property FileObjects: TFileObjects read FFileObject write FFileObject;
    property FileSizeStyle: Integer read FFileSizeStyle write FFileSizeStyle;
    property FocusRect: Boolean read FFocusRect write FFocusRect;
    property FoldersAlwaysFirst: Boolean read FFoldersAlwaysFirst write FFoldersAlwaysFirst;
    property FolderSizeDisplay: TBtyeSize read FFolderSizeDisplay write FFolderSizeDisplay;
    property FolderSizeShow: Boolean read FFolderSizeShow write FFolderSizeShow;
    property FullRowSelect: Boolean read FFullRowSelect write FFullRowSelect;
    property GradientSelection: Boolean read FGradientSelection write FGradientSelection;
    property GridLines: Boolean read FGridLines write FGridLines;
    property GroupedByDefault: Boolean read FGroupedByDefault write FGroupedByDefault;
    property HideCaptions: Boolean read FHideCaptions write FHideCaptions;
    property HideTabsetWhenOneTab: Boolean read FHideTabsetWhenOneTab write FHideTabsetWhenOneTab;
    property HilightColumn: Boolean read FHilightColumn write FHilightColumn;
    property HilightColumnColor: TColor read FHilightColumnColor write FHilightColumnColor;
    property HilightCompressed: Boolean read FHilightCompressed write FHilightCompressed;
    property HilightEncrypted: Boolean read FHilightEncrypted write FHilightEncrypted;
    property HotTrack: Boolean read FHotTrack write FHotTrack;
    property QueryInfoHints: Boolean read FQueryInfoHints write FQueryInfoHints;
    property QueryInfoHintTimout: LongWord read FQueryInfoHintTimout write FQueryInfoHintTimout;
    property SavePerFolder: Boolean read FSavePerFolder write FSavePerFolder;
    property SelectFirstItem: Boolean read FSelectFirstItem write FSelectFirstItem;
    property ShellThumbs: Boolean read FShellThumbs write FShellThumbs;
    property ShowTitleBar: Boolean read FShowTitleBar write FShowTitleBar;
    property SimpleChangeNotify: Boolean read FSimpleChangeNotify write FSimpleChangeNotify;
    property SmoothScroll: Integer read FSmoothScroll write FSmoothScroll default 20;
    property SortDirection: TEasySortDirection read FSortDirection write FSortDirection;
    property ThumbsCache: Boolean read FThumbsCache write FThumbsCache;
    property ThumbsCompress: Boolean read FThumbsCompress write FThumbsCompress;
    property ThumbsLoadAll: Boolean read FThumbsLoadAll write FThumbsLoadAll;
    property ThumbsSingleFile: Boolean read FThumbsSingleFile write FThumbsSingleFile;
    property ThumbsWidth: Integer read FThumbsWidth write FThumbsWidth default 125;
    property ThumbsHeight: Integer read FThumbsHeight write FThumbsHeight default 143;
    property ThumbsCacheFile: WideString read FThumbsCacheFile write FThumbsCacheFile;
    property UpLevelOnDblClk: Boolean read FUpLevelOnDblClk write FUpLevelOnDblClk;
    property UseOnByDefaultFlag: Boolean read FUseOnByDefaultFlag write FUseOnByDefaultFlag;
  end;

  THistoryAttribs = class(TBaseAttribs)
  private
    FLevels: Integer;
    FSaveMappedPaths: Boolean;
    FSaveNetworkPaths: Boolean;
    FSavePerTab: Boolean;
    FSaveRemovableDrivePaths: Boolean;
    FWarnBeforeContextAction: Boolean;
    FWarnBeforeDropAction: Boolean;
  protected
    procedure SetParentComponent(Value: TComponent); override;
  public
    constructor Create(AOwner: TComponent); override;
  published
    property Levels: Integer read FLevels write FLevels;
    property SaveMappedPaths: Boolean read FSaveMappedPaths write FSaveMappedPaths;
    property SaveNetworkPaths: Boolean read FSaveNetworkPaths write FSaveNetworkPaths;
    property SavePerTab: Boolean read FSavePerTab write FSavePerTab;
    property SaveRemovableDrivePaths: Boolean read FSaveRemovableDrivePaths write FSaveRemovableDrivePaths;
    property WarnBeforeContextAction: Boolean read FWarnBeforeContextAction write FWarnBeforeContextAction default True;
    property WarnBeforeDropAction: Boolean read FWarnBeforeDropAction write FWarnBeforeDropAction default True;
  end;

  TFavoritesAttribs = class(TBaseAttribs)
  private
    FFullRowSelect: Boolean;
    FOpenFoldersInNewTab: Boolean;
    FShowPathHint: Boolean;
    FWarnBeforeContextAction: Boolean;
    FWarnBeforeFileAction: Boolean;
  protected
    procedure SetParentComponent(Value: TComponent); override;
  public
    constructor Create(AOwner: TComponent); override;
  published
    property FullRowSelect: Boolean read FFullRowSelect write FFullRowSelect;
    property OpenFoldersInNewTab: Boolean read FOpenFoldersInNewTab write FOpenFoldersInNewTab;
    property ShowPathHint: Boolean read FShowPathHint write FShowPathHint;
    property WarnBeforeContextAction: Boolean read FWarnBeforeContextAction write FWarnBeforeContextAction default True;
    property WarnBeforeDropAction: Boolean read FWarnBeforeFileAction write FWarnBeforeFileAction default True;
  end;

  TDropStackAttribs = class(TBaseAttribs)
  private
    FAutoRemove: TVirtualDropStackAutoRemove;
    FContentsHint: Boolean;
    FDepth: Integer;
    FHintCount: Integer;
    FWarningFirstRun: Boolean;
  protected
    procedure SetParentComponent(Value: TComponent); override;
  public
    constructor Create(AOwner: TComponent); override;
  published
    property AutoRemove: TVirtualDropStackAutoRemove read FAutoRemove write FAutoRemove;
    property ContentsHint: Boolean read FContentsHint write FContentsHint;
    property Depth: Integer read FDepth write FDepth default 20;
    property HintCount: Integer read FHintCount write FHintCount default 50;
    property WarningFirstRun: Boolean read FWarningFirstRun write FWarningFirstRun;
  end;

  TFilterAttribs = class(TBaseAttribs)
  private
    FFilterOut: Boolean;
    FQuickFilterAutoWildcard: Boolean;
    FQuickFilterClearOnBrowse: Boolean;
    FQuickFilterSavePerTab: Boolean;
  protected
    procedure SetParentComponent(Value: TComponent); override;
  published
    property FilterOut: Boolean read FFilterOut write FFilterOut;
    property QuickFilterAutoWildcard: Boolean read FQuickFilterAutoWildcard write FQuickFilterAutoWildcard;
    property QuickFilterClearOnBrowse: Boolean read FQuickFilterClearOnBrowse write FQuickFilterClearOnBrowse;
    property QuickFilterSavePerTab: Boolean read FQuickFilterSavePerTab write FQuickFilterSavePerTab;
  end;

  TMouseAttribs = class(TBaseAttribs)
  private
    FBackStroke: WideChar;
    FFolderUpStroke: WideChar;
    FForwardStroke: Widechar;
    FNewTabStroke: WideChar;
    FRightButtonAssignment: Integer;
  protected
    procedure SetParentComponent(Value: TComponent); override;
  public
    constructor Create(AOwner: TComponent); override;
  published
    property BackStroke: WideChar read FBackStroke write FBackStroke;
    property FolderUpStroke: WideChar read FFolderUpStroke write FFolderUpStroke;
    property ForwardStroke: Widechar read FForwardStroke write FForwardStroke;
    property NewTabStroke: WideChar read FNewTabStroke write FNewTabStroke;
    property RightButtonAssignment: Integer read FRightButtonAssignment write FRightButtonAssignment;
  end;

  TToolbarAttribs = class(TBaseAttribs)
  private
    FAddressBarWidth: Integer;
    FAutoOpenBreadcrumb: Boolean;
    FBreadCrumbHoverTime: Integer;
    FBreadCrumbWrap: Integer;
    FButtonLayout: TUltraButtonLayout;
    FDriveFavoriteCrumbWrap: Integer;
    FDriveShowControlPanel: Boolean;
    FDriveShowDesktop: Boolean;
    FDriveShowHints: Boolean;
    FDriveShowMyComputer: Boolean;
    FDriveShowMyDocuments: Boolean;
    FDriveShowNetwork: Boolean;
    FDriveShowRecycleBin: boolean;
    FFavoriteCrumbWrap: Integer;
    FLock: Boolean;
    FShowBrowseableInBreadcrumbMenus: Boolean;
  protected
    procedure SetParentComponent(Value: TComponent); override;
  public
    constructor Create(AOwner: TComponent); override;
  published
    property AddressBarWidth: Integer read FAddressBarWidth write FAddressBarWidth default 370;
    property AutoOpenBreadcrumb: Boolean read FAutoOpenBreadcrumb write FAutoOpenBreadcrumb;
    property BreadCrumbHoverTime: Integer read FBreadCrumbHoverTime write FBreadCrumbHoverTime;
    property BreadCrumbWrap: Integer read FBreadCrumbWrap write FBreadCrumbWrap;
    property ButtonLayout: TUltraButtonLayout read FButtonLayout write FButtonLayout;
    property DriveFavoriteCrumbWrap: Integer read FDriveFavoriteCrumbWrap write FDriveFavoriteCrumbWrap;
    property DriveShowControlPanel: Boolean read FDriveShowControlPanel write FDriveShowControlPanel;
    property DriveShowDesktop: Boolean read FDriveShowDesktop write FDriveShowDesktop;
    property DriveShowHints: Boolean read FDriveShowHints write FDriveShowHints;
    property DriveShowMyComputer: Boolean read FDriveShowMyComputer write FDriveShowMyComputer;
    property DriveShowMyDocuments: Boolean read FDriveShowMyDocuments write FDriveShowMyDocuments;
    property DriveShowNetwork: Boolean read FDriveShowNetwork write FDriveShowNetwork;
    property DriveShowRecycleBin: boolean read FDriveShowRecycleBin write FDriveShowRecycleBin;
    property FavoriteCrumbWrap: Integer read FFavoriteCrumbWrap write FFavoriteCrumbWrap;
    property Lock: Boolean read FLock write FLock;
    property ShowBrowseableInBreadcrumbMenus: Boolean read FShowBrowseableInBreadcrumbMenus write FShowBrowseableInBreadcrumbMenus;
  end;

  TViewerAttribs = class(TBaseAttribs)
  private
    FAutoModeDetect: Boolean;
    FAutoPlayOnLoad: Boolean;
    FHex1Color: TColor;
    FHex2Color: TColor;
    FHexBkGndColor: TColor;
    FImageExtensions: TTntStringList;
    FInternetExtensions: TTntStringList;
    FMediaExtensions: TTntStringList;
    FMediaFit: Boolean;
    FMode: TATViewerMode;
    FOEMFont: TFont;
    FPlugIns: TPlugins;
    FRTFExtensions: TTntStringList;
    FTextBkGndColor: TColor;
    FTextEncoding: TATEncoding;
    FTextExtensions: TTntStringList;
    FTextWrap: Boolean;
    FWebOffline: Boolean;
  protected
    procedure SetParentComponent(Value: TComponent); override;
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
  published
    property AutoModeDetect: Boolean read FAutoModeDetect write FAutoModeDetect;
    property AutoPlayOnLoad: Boolean read FAutoPlayOnLoad write FAutoPlayOnLoad;
    property Hex1Color: TColor read FHex1Color write FHex1Color;
    property Hex2Color: TColor read FHex2Color write FHex2Color;
    property HexBkGndColor: TColor read FHexBkGndColor write FHexBkGndColor;
    property ImageExtensions: TTntStringList read FImageExtensions write FImageExtensions;
    property InternetExtensions: TTntStringList read FInternetExtensions write FInternetExtensions;
    property MediaExtensions: TTntStringList read FMediaExtensions write FMediaExtensions;
    property MediaFit: Boolean read FMediaFit write FMediaFit;
    property Mode: TATViewerMode read FMode write FMode;
    property OEMFont: TFont read FOEMFont write FOEMFont;
    property PlugIns: TPlugins read FPlugIns write FPlugIns;
    property RTFExtensions: TTntStringList read FRTFExtensions write FRTFExtensions;
    property TextBkGndColor: TColor read FTextBkGndColor write FTextBkGndColor;
    property TextEncoding: TATEncoding read FTextEncoding write FTextEncoding;
    property TextExtensions: TTntStringList read FTextExtensions write FTextExtensions;
    property TextWrap: Boolean read FTextWrap write FTextWrap;
    property WebOffline: Boolean read FWebOffline write FWebOffline;
  end;

  TLoadAttribsProc = function(Attribs: TUltraAttribs): Boolean of object;
  TSaveAttribsProc = function(Attribs: TUltraAttribs): Boolean of object;

  TUltraAttribs = class(TStreamedBase)
  private
    FApplyCallback: TNotifyEvent;
    FChangeNotifierEnabled: Boolean;
    FCmdLine: TCmdLineAttribs;
    FContextMenuHandlerInstalled: Boolean;
    FDropStack: TDropStackAttribs;
    FFavorites: TFavoritesAttribs;
    FFilter: TFilterAttribs;
    FGeneral: TGeneralAttribs;
    FHistory: THistoryAttribs;
    FListview: TListviewAttribs;
    FLoadCallback: TLoadAttribsProc;
    FMouse: TMouseAttribs;
    FSaveCallback: TSaveAttribsProc;
    FShowPopupHints: Boolean;
    FTabs: TTabAttribs;
    FToolbars: TToolbarAttribs;
    FTreeview: TTreeviewAttribs;
    FVersion: Integer;
    FViewer: TViewerAttribs;
    procedure SetCmdLine(const Value: TCmdLineAttribs);
    procedure SetDropStack(const Value: TDropStackAttribs);
    procedure SetFavorites(const Value: TFavoritesAttribs);
    procedure SetFilter(const Value: TFilterAttribs);
    procedure SetGeneral(const Value: TGeneralAttribs);
    procedure SetHistory(const Value: THistoryAttribs);
    procedure SetListview(const Value: TListviewAttribs);
    procedure SetToolbars(const Value: TToolbarAttribs);
    procedure SetTreeview(const Value: TTreeviewAttribs);
    procedure SetViewer(const Value: TViewerAttribs);
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;

    procedure ReadComponents(S: TStream);
    procedure LoadFromFileText(FileName: WideString);
    procedure SaveToFileText(FileName: WideString);
    procedure WriteComponents(S: TStream);
    property ApplyCallback: TNotifyEvent read FApplyCallback write FApplyCallback;
    property LoadCallback: TLoadAttribsProc read FLoadCallback write FLoadCallback;
    property SaveCallback: TSaveAttribsProc read FSaveCallback write FSaveCallback;
  published
    property ChangeNotifierEnabled: Boolean read FChangeNotifierEnabled write FChangeNotifierEnabled;
    property CmdLine: TCmdLineAttribs read FCmdLine write SetCmdLine;
    property ContextMenuHandlerInstalled: Boolean read FContextMenuHandlerInstalled write FContextMenuHandlerInstalled;
    property DropStack: TDropStackAttribs read FDropStack write SetDropStack;
    property Favorites: TFavoritesAttribs read FFavorites write SetFavorites;
    property Filter: TFilterAttribs read FFilter write SetFilter;
    property General: TGeneralAttribs read FGeneral write SetGeneral;
    property History: THistoryAttribs read FHistory write SetHistory;
    property Listview: TListviewAttribs read FListview write SetListview;
    property Mouse: TMouseAttribs read FMouse write FMouse;
    property ShowPopupHints: Boolean read FShowPopupHints write FShowPopupHints;
    property Tabs: TTabAttribs read FTabs write FTabs;
    property Toolbars: TToolbarAttribs read FToolbars write SetToolbars;
    property Treeview: TTreeviewAttribs read FTreeview write SetTreeview;
    property Version: Integer read FVersion write FVersion;
    property Viewer: TViewerAttribs read FViewer write SetViewer;
  end;

  TPlugin = class(TCollectionItem)
  private
    FEnabled: Boolean;
    FExtensions: WideString;
    FPath: WideString;
  published
    property Enabled: Boolean read FEnabled write FEnabled;
    property Extensions: WideString read FExtensions write FExtensions;
    property Path: WideString read FPath write FPath;
  end;


  function EditGroupingCriteria(Owner: TTnTForm; SpTBXDockHeader: TSpTBXDock; SpTBXToolbar: TSpTBXToolbar): Boolean;



var
  Attribs: TUltraAttribs;
  GlobalStorage: TRootNodeStorage;
  GroupingModifiedArray: TGroupingModifiedArray;
  GroupingFileSizeArray: TGroupingFileSizeArray;

implementation

function EditGroupingCriteria(Owner: TTnTForm; SpTBXDockHeader: TSpTBXDock; SpTBXToolbar: TSpTBXToolbar): Boolean;
var
  Form: TTntFormGroupingCriteria;
  i: Integer;
begin
  Result := False;
  Form := TTntFormGroupingCriteria.CreateEx(Owner);
  try
    Form.SpTBXDockHeader := SpTBXDockHeader;
    Form.SpTBXToolbar := SpTBXToolbar;
    SetLength(Form.FGroupingModifiedArray, Length(GroupingModifiedArray));
    for i := 0 to Length(GroupingModifiedArray) - 1 do
      Form.GroupingModifiedArray[i] := GroupingModifiedArray[i];

    SetLength(Form.FGroupingFileSizeArray, Length(GroupingFileSizeArray));
    for i := 0 to Length(GroupingFileSizeArray) - 1 do
      Form.GroupingFileSizeArray[i] := GroupingFileSizeArray[i];

    if Form.ShowModal = mrOK then
    begin
      SetLength(GroupingModifiedArray, Length(Form.GroupingModifiedArray));
      for i := 0 to Length(GroupingModifiedArray) - 1 do
        GroupingModifiedArray[i] := Form.GroupingModifiedArray[i];

      SetLength(GroupingFileSizeArray, Length(Form.GroupingFileSizeArray));
      for i := 0 to Length(GroupingFileSizeArray) - 1 do
        GroupingFileSizeArray[i] := Form.GroupingFileSizeArray[i];

      Result := True
    end
  finally
    Form.Release
  end
end;

{ TBaseAttribs}
constructor TBaseAttribs.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  FFont := TFont.Create;
  FColor := clWindow;
  FFont.Name := 'MS Shell Dlg 2';
  FFont.Size := 8;
  FVersion := ULTRAOPTIONS_VERSION;
end;

destructor TBaseAttribs.Destroy;
begin
  FreeAndNil(FFont);
  inherited Destroy;
end;

function TBaseAttribs.GetColor: TColor;
begin
  Result := FColor;
end;

procedure TBaseAttribs.SetColor(const Value: TColor);
begin
  FColor := Value;
end;

{ TUltraAttribs}
constructor TUltraAttribs.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  Version := ULTRAOPTIONS_VERSION;
  CmdLine := TCmdLineAttribs.Create(Self);
  Favorites := TFavoritesAttribs.Create(Self);
  Filter := TFilterAttribs.Create(Self);
  History := THistoryAttribs.Create(Self);
  Listview := TListviewAttribs.Create(Self);
  Treeview := TTreeviewAttribs.Create(Self);
  DropStack := TDropStackAttribs.Create(Self);
  General := TGeneralAttribs.Create(Self);
  Viewer := TViewerAttribs.Create(Self);
  Toolbars := TToolbarAttribs.Create(Self);
  Tabs := TTabAttribs.Create(Self);
  Mouse := TMouseAttribs.Create(Self);
  ChangeNotifierEnabled := True;
end;

destructor TUltraAttribs.Destroy;
begin
  FreeAndNil(FCmdLine);
  FreeAndNil(FFavorites);
  FreeAndNil(FFilter);
  FreeAndNil(FHistory);
  FreeAndNil(FListview);
  FreeAndNil(FTreeview);
  FreeAndNil(FDropStack);
  FreeAndNil(FGeneral);
  FreeAndNil(FViewer);
  FreeAndNil(FTabs);
  FreeAndNil(FToolbars);
  FreeAndNil(FMouse);
  inherited Destroy;
end;

procedure TUltraAttribs.ReadComponents(S: TStream);

  procedure RunRead(Component: TComponent; S: TStream);
  var
    i: Integer;
  begin
    for i := 0 to Component.ComponentCount - 1 do
    begin
      RunRead(Component.Components[i], S);
      S.ReadComponent(Component.Components[i])
    end
  end;

begin
  RunRead(Self, S);
end;

procedure TUltraAttribs.LoadFromFileText(FileName: WideString);
var
  FileStream: TTntFileStream;
  BinaryStream: TMemoryStream;
begin
  BinaryStream := TMemoryStream.Create;
  FileStream := TTntFileStream.Create(FileName, fmOpenRead or fmShareExclusive);
  try
    ObjectTextToBinary(FileStream, BinaryStream);
    BinaryStream.Seek(0, soFromBeginning);
    BinaryStream.ReadComponent(Self);
  finally
    FileStream.Free;
    BinaryStream.Free
  end
end;

procedure TUltraAttribs.SaveToFileText(FileName: WideString);
var
  FileStream: TTntFileStream;
  BinaryStream: TMemoryStream;
begin
  BinaryStream := TMemoryStream.Create;
  FileStream := TTntFileStream.Create(FileName, fmCreate or fmShareExclusive);
  try
    BinaryStream.WriteComponent(Self);
    BinaryStream.Seek(0, soFromBeginning);
    FileStream.Seek(0, soFromBeginning);
    ObjectBinaryToText(BinaryStream, FileStream);
  finally
    FileStream.Free;
    BinaryStream.Free
  end
end;

procedure TUltraAttribs.SetCmdLine(const Value: TCmdLineAttribs);
begin
  if FCmdLine <> Value then
  begin
    FreeAndNil(FCmdLine);
    FCmdLine := Value;
  end
end;

procedure TUltraAttribs.SetDropStack(const Value: TDropStackAttribs);
begin
  if FDropStack <> Value then
  begin
    FreeAndNil(FDropStack);
    FDropStack := Value;
  end
end;

procedure TUltraAttribs.SetFavorites(const Value: TFavoritesAttribs);
begin
  if FFavorites <> Value then
  begin
    FreeAndNil(FFavorites);
    FFavorites := Value;
  end
end;

procedure TUltraAttribs.SetFilter(const Value: TFilterAttribs);
begin
  if FFilter <> Value then
  begin
    FreeAndNil(FFilter);
    FFilter := Value;
  end
end;

procedure TUltraAttribs.SetGeneral(const Value: TGeneralAttribs);
begin
  if FGeneral <> Value then
  begin
    FreeAndNil(FGeneral);
    FGeneral := Value;
  end
end;

procedure TUltraAttribs.SetHistory(const Value: THistoryAttribs);
begin
  if FHistory <> Value then
  begin
    FreeAndNil(FHistory);
    FHistory := Value;
  end
end;

procedure TUltraAttribs.SetListview(const Value: TListviewAttribs);
begin
  if FListview <> Value then
  begin
    FreeAndNil(FListview);
    FListview := Value;
  end
end;

procedure TUltraAttribs.SetToolbars(const Value: TToolbarAttribs);
begin
  if FToolbars <> Value then
  begin
    FreeAndNil(FToolbars);
    FToolbars := Value;
  end
end;

procedure TUltraAttribs.SetTreeview(const Value: TTreeviewAttribs);
begin
  if FTreeview <> Value then
  begin
    FreeAndNil(FTreeview);
    FTreeview := Value;
  end
end;

procedure TUltraAttribs.SetViewer(const Value: TViewerAttribs);
begin
  if FViewer <> Value then
  begin
    FreeAndNil(FViewer);
    FViewer := Value;
  end
end;

procedure TUltraAttribs.WriteComponents(S: TStream);

  procedure RunWrite(Component: TComponent; S: TStream);
  var
    i: Integer;
  begin
    for i := 0 to Component.ComponentCount - 1 do
    begin
      RunWrite(Component.Components[i], S);
      S.WriteComponent(Component.Components[i])
    end
  end;

begin
  RunWrite(Self, S)
end;

{ THistoryAttribs }
constructor THistoryAttribs.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  WarnBeforeContextAction := True;
  WarnBeforeDropAction := True;
  SavePerTab := True;
  Levels := 20
end;

procedure THistoryAttribs.SetParentComponent(Value: TComponent);
begin
  (Value as TUltraAttribs).History := Self;
end;

{ TFavoritesAttribs}
constructor TFavoritesAttribs.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  WarnBeforeDropAction := True;
  WarnBeforeContextAction := True;
  ShowPathHint := True;
end;

procedure TFavoritesAttribs.SetParentComponent(Value: TComponent);
begin
  (Value as TUltraAttribs).Favorites := Self;
end;

{ TDropStackAttribs }
constructor TDropStackAttribs.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  HintCount := 50;
  Depth := 20;
  ContentsHint := True
end;

procedure TDropStackAttribs.SetParentComponent(Value: TComponent);
begin
  (Value as TUltraAttribs).DropStack := Self;
end;

{ TCmdLineAttribs }
constructor TCmdLineAttribs.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  Font.Color := clWhite;
  MaxLineCount := 8192
end;

procedure TCmdLineAttribs.SetParentComponent(Value: TComponent);
begin
  (Value as TUltraAttribs).CmdLine := Self;
end;

{ TListviewAttribs }
constructor TListviewAttribs.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  ColumnsShownInHeaderMenu := 5;
  FileObjects := [foFolders, foNonFolders];
  HilightColumnColor := $00F7F7F7;
  SmoothScroll := 20;
  SortDirection := esdAscending;
  DefaultColumnModeWidth := 200;
  AspectRatio := 1.114;
  ColumnHilightBandColor := $00F7F7F7;
  ColumnHilightBanding := True;
  ThumbsWidth := 125;
  ThumbsHeight := 143;
  UpLevelOnDblClk := True;
  QueryInfoHintTimout := 2500; // ms
  AutoOrientation := True;
  FExtensionColorCodeList := TExtensionColorCodeList.Create;
end;

destructor TListviewAttribs.Destroy;
begin
  FreeAndNil(FExtensionColorCodeList);
  inherited Destroy;
end;

procedure TListviewAttribs.SetParentComponent(Value: TComponent);
begin
 (Value as TUltraAttribs).Listview := Self;
end;

{ TGeneralAttribs }
constructor TGeneralAttribs.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  TrackSelectedSize := True;
  ShowCloseButtons := True;
  LockWindowColors := True;
  LockWindowFonts := True;
  FIncrementalDelayTime := 250;
end;

destructor TGeneralAttribs.Destroy;
begin
  PIDLMgr.FreeAndNilPIDL(FDefaultTabPIDL);
  PIDLMgr.FreeAndNilPIDL(FDefaultStartupPath);
  inherited Destroy;
end;

procedure TGeneralAttribs.DefineProperties(Filer: TFiler);
begin
  inherited DefineProperties(Filer);
  Filer.DefineBinaryProperty('CustomTabPath', LoadCustomTabPath, SaveCustomTabPath, True);
  Filer.DefineBinaryProperty('CustomStartupPath', LoadCustomStartupPath, SaveCustomStartupPath, True)
end;

procedure TGeneralAttribs.LoadCustomStartupPath(Stream: TStream);
begin
  PIDLMgr.FreeAndNilPIDL(FDefaultStartupPath);
  DefaultStartupPath := PIDLMgr.LoadFromStream(Stream)
end;

procedure TGeneralAttribs.LoadCustomTabPath(Stream: TStream);
begin
  PIDLMgr.FreeAndNilPIDL(FDefaultTabPIDL);
  DefaultTabPIDL := PIDLMgr.LoadFromStream(Stream)
end;

procedure TGeneralAttribs.SaveCustomStartupPath(Stream: TStream);
begin
  PIDLMgr.SaveToStream(Stream, DefaultStartupPath)
end;

procedure TGeneralAttribs.SaveCustomTabPath(Stream: TStream);
begin
  PIDLMgr.SaveToStream(Stream, DefaultTabPIDL)
end;

procedure TGeneralAttribs.SetParentComponent(Value: TComponent);
begin
  (Value as TUltraAttribs).General := Self;
end;

{ TViewerAttribs }
constructor TViewerAttribs.Create(AOwner: TComponent);
begin
  inherited;
  OEMFont := TFont.Create;
  TextExtensions := TTntStringList.Create;
  ImageExtensions := TTntStringList.Create;
  InternetExtensions := TTntStringList.Create;
  MediaExtensions := TTntStringList.Create;
  RTFExtensions := TTntStringList.Create;
  PlugIns := TPlugins.Create(TPlugin);
  HexBkGndColor := clLtGray
end;

destructor TViewerAttribs.Destroy;
begin
  FreeAndNil(FOEMFont);
  FreeAndNil(FImageExtensions);
  FreeAndNil(FInternetExtensions);
  FreeAndNil(FMediaExtensions);
  FreeAndNil(FRTFExtensions);
  FreeAndNil(FTextExtensions);
  FreeAndNil(FPlugIns);
  inherited Destroy;
end;

procedure TViewerAttribs.SetParentComponent(Value: TComponent);
begin
  (Value as TUltraAttribs).Viewer := Self;
end;

{ TStreamedBase }
procedure TStreamedBase.GetChildren(Proc: TGetChildProc; Root: TComponent);
var
  I: Integer;
  Control: TComponent;
begin
  for I := 0 to ComponentCount - 1 do
  begin
    Control := Components[I];
    if Control.Owner = Root then
      Proc(Control);
  end;
end;

procedure TStreamedBase.SetParentComponent(Value: TComponent);
begin

end;

{ TFilterAttribs }
procedure TFilterAttribs.SetParentComponent(Value: TComponent);
begin
  (Value as TUltraAttribs).Filter := Self;
end;

{ TTreeviewAttribs }
procedure TTreeviewAttribs.SetParentComponent(Value: TComponent);
begin
  (Value as TUltraAttribs).Treeview := Self;
end;

{ TToolbarAttribs }
constructor TToolbarAttribs.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  BreadCrumbWrap := 1;
  FavoriteCrumbWrap := 1;
  DriveShowHints := True;
  AddressBarWidth := 370;
  BreadCrumbHoverTime := 500; // ms
end;

procedure TToolbarAttribs.SetParentComponent(Value: TComponent);
begin
  (Value as TUltraAttribs).Toolbars := Self;
end;

{ TTabAttribs }
constructor TTabAttribs.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  MaxWidth := -1;
end;

procedure TTabAttribs.SetParentComponent(Value: TComponent);
begin
  (Value as TUltraAttribs).Tabs := Self;
end;

{ TMouseAttribs }
constructor TMouseAttribs.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  BackStroke := 'L';
  FolderUpStroke := 'U';
  ForwardStroke := 'R';
  NewTabStroke := 'D'
end;

procedure TMouseAttribs.SetParentComponent(Value: TComponent);
begin
  (Value as TUltraAttribs).Mouse := Self;
end;

initialization
  Attribs := TUltraAttribs.Create(nil);
  GlobalStorage := TRootNodeStorage.Create;
  RegisterClass(TUltraAttribs);
  RegisterClass(TCmdLineAttribs);
  RegisterClass(TDropStackAttribs);
  RegisterClass(TFavoritesAttribs);
  RegisterClass(TFilterAttribs);
  RegisterClass(TGeneralAttribs);
  RegisterClass(THistoryAttribs);
  RegisterClass(TListviewAttribs);
  RegisterClass(TToolbarAttribs);
  RegisterClass(TTreeviewAttribs);
  RegisterClass(TTabAttribs);
  RegisterClass(TViewerAttribs);
  RegisterClass(TMouseAttribs);
  LoadDefaultGroupingModifiedArray(GroupingModifiedArray, False);
  LoadDefaultGroupingFileSizeArray(GroupingFileSizeArray, False);
  
finalization
  FreeAndNil(Attribs);
  FreeAndNil(GlobalStorage);

end.
