unit UnitEditGroupingCriteria;

// The contents of this file are subject to the Mozilla Public License
// Version 1.1 (the "License"); you maynot use this file except in compliance
// with the License. You may obtain a copy of the License at http://www.mozilla.org/MPL/
//
// Alternatively, you may redistribute this library, use and/or modify it under the terms of the
// GNU Lesser General Public License as published by the Free Software Foundation;
// either version 2.1 of the License, or (at your option) any later version.
// You may obtain a copy of the LGPL at http://www.gnu.org/copyleft/.
//
// Software distributed under the License is distributed on an "AS IS" basis,
// WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for the
// specific language governing rights and limitations under the License.

interface

uses
  Windows,
  Messages,
  SysUtils,
  Variants,
  Classes,
  Graphics,
  Controls,
  Forms,
  Dialogs,
  TntForms,
  DKLang,
  MPCommonObjects,
  EasyListview,
  SpTBXControls,
  SpTBXItem,
  UltraUtilities,
  VirtualExplorerEasyListview,
  UltraUserStrings,
  TB2Item,
  Menus,
  TntMenus,
  ActnList,
  TntActnList,
  MPCommonUtilities,
  Math;

type
  TCriteria = (
    cFileSize,
    cModifyDate
  );

  TTntFormGroupingCriteria = class(TTntForm)
    ActionAdd: TTntAction;
    ActionDefaults: TTntAction;
    ActionDelete: TTntAction;
    ActionSave: TTntAction;
    DKLanguageController1: TDKLanguageController;
    ELV: TEasyListview;
    SpTBXButton1: TSpTBXButton;
    SpTBXButtonAdd: TSpTBXButton;
    SpTBXButtonCancel: TSpTBXButton;
    SpTBXButtonDefaults: TSpTBXButton;
    SpTBXButtonDelete: TSpTBXButton;
    SpTBXButtonOk: TSpTBXButton;
    SpTBXItem1: TSpTBXItem;
    SpTBXItem2: TSpTBXItem;
    SpTBXItem3: TSpTBXItem;
    SpTBXPanel1: TSpTBXPanel;
    SpTBXPopupMenuELV: TSpTBXPopupMenu;
    SpTBXRadioGroupType: TSpTBXRadioGroup;
    SpTBXSeparatorItem1: TSpTBXSeparatorItem;
    TntActionList: TTntActionList;
    SpTBXButton2: TSpTBXButton;
    ActionEdit: TTntAction;

    procedure TntFormShow(Sender: TObject);
    procedure SpTBXRadioGroupTypeClick(Sender: TObject);
    procedure ActionAddExecute(Sender: TObject);
    procedure ActionDeleteExecute(Sender: TObject);
    procedure ActionDefaultsExecute(Sender: TObject);
    procedure ActionSaveExecute(Sender: TObject);
    procedure ELVItemSelectionsChanged(Sender: TCustomEasyListview);
    procedure SpTBXButtonCancelClick(Sender: TObject);
    procedure SpTBXButtonOkClick(Sender: TObject);
    procedure ELVKeyAction(Sender: TCustomEasyListview; var CharCode: Word; var Shift: TShiftState; var DoDefault: Boolean);
    procedure ELVItemCheckChange(Sender: TCustomEasyListview; Item: TEasyItem);
    procedure ActionEditExecute(Sender: TObject);
    function ELVItemCompare(Sender: TCustomEasyListview; Column: TEasyColumn; Group: TEasyGroup; Item1, Item2: TEasyItem): Integer;
    procedure ELVItemDblClick(Sender: TCustomEasyListview; Button: TCommonMouseButton; MousePos: TPoint; HitInfo: TEasyHitInfoItem);
    procedure TntFormKeyDown(Sender: TObject; var Key: Word; Shift: TShiftState);
    procedure TntFormCreate(Sender: TObject);
  private
    FCurrentCriteria: TCriteria;
    FIsDirty: Boolean;
    FIsSpecialFolder: Boolean;
    FSpTBXDockHeader: TSpTBXDock;
    FSpTBXToolbar: TSpTBXToolbar;
    FWndParent: HWnd;
    { Private declarations }
    procedure SetIsDirty(const Value: Boolean);
  protected
    procedure CreateParams(var Params: TCreateParams); override;
    function EditCriteria: Boolean;
    procedure LoadListviewWithFileSizeInfo(GroupingArray: TGroupingFileSizeArray);
    procedure LoadListviewWithModifiedDateInfo(ModifiedArray: TGroupingModifiedArray);
    property WndParent: HWnd read FWndParent write FWndParent;
  public
    FGroupingFileSizeArray: TGroupingFileSizeArray;
    FGroupingModifiedArray: TGroupingModifiedArray;
    { Public declarations }
    constructor CreateEx(AnOwner: TCustomForm);

    function IsEditModifiedDate: Boolean;
    function IsEditSize: Boolean;
    procedure CheckForSave;
    procedure SyncActions;
    property CurrentCriteria: TCriteria read FCurrentCriteria write FCurrentCriteria;
    property GroupingFileSizeArray: TGroupingFileSizeArray read FGroupingFileSizeArray write FGroupingFileSizeArray;
    property GroupingModifiedArray: TGroupingModifiedArray read FGroupingModifiedArray write FGroupingModifiedArray;
    property IsDirty: Boolean read FIsDirty write SetIsDirty;
    property IsSpecialFolder: Boolean read FIsSpecialFolder write FIsSpecialFolder;
    property SpTBXDockHeader: TSpTBXDock read FSpTBXDockHeader write FSpTBXDockHeader;
    property SpTBXToolbar: TSpTBXToolbar read FSpTBXToolbar write FSpTBXToolbar;
  published
  end;


implementation

uses
  UnitEditGroupingCriteriaEdit;

{$R *.DFM}

function TTntFormGroupingCriteria.EditCriteria: Boolean;
var
  Form: TTntFormGroupingCriteriaEdit;
  Item: TEasyItem;
begin
  Result := False;
  if ELV.Selection.First <> nil then
  begin
    Form := TTntFormGroupingCriteriaEdit.CreateEx(Self);
    try
      case CurrentCriteria of
        cFileSize: Form.NumericType := ntInt64;
        cModifyDate: Form.NumericType := ntSingle;
      end;
      Item := ELV.Selection.First;
      Form.SpTBXEditCaption.Text := Item.Captions[0];
      Form.SpTBXEditValue.Text := Item.Captions[1];
      if Form.ShowModal = mrOk then
      begin
        Item.Captions[0] := Form.SpTBXEditCaption.Text;
        case CurrentCriteria of
          cFileSize: Item.Captions[1] := Form.SpTBXEditValue.Text;
          cModifyDate: Item.Captions[1] := FloatToStrF(StrToFloat(Form.SpTBXEditValue.Text), ffFixed, 4, 4);
        end;
        ELV.Sort.SortAll;
        IsDirty := True;
        Result := True
      end
    finally
      Form.Release
    end;
  end
end;

function TTntFormGroupingCriteria.IsEditModifiedDate: Boolean;
begin
  Result := SpTBXRadioGroupType.ItemIndex = 1
end;

function TTntFormGroupingCriteria.IsEditSize: Boolean;
begin
  Result := SpTBXRadioGroupType.ItemIndex = 0
end;

procedure TTntFormGroupingCriteria.CheckForSave;
var
  Item: TEasyItem;
  i: Integer;
begin
  if IsDirty then
  begin
    case CurrentCriteria of
      cFileSize:
        begin
          SetLength(FGroupingFileSizeArray, ELV.Items.Count);
          for i := 0 to Length(GroupingFileSizeArray) - 1 do
          begin
            Item := ELV.Items[i];
            GroupingFileSizeArray[i].Caption := Item.Caption;
            GroupingFileSizeArray[i].FileSize := StrToInt64(Item.Captions[1]);
            GroupingFileSizeArray[i].Enabled := Item.Checked;
            GroupingFileSizeArray[i].SpecialFolder := Item.OwnerGroup.Index > 0
          end;
        end;
      cModifyDate:
        begin
          SetLength(FGroupingModifiedArray, ELV.Items.Count);
          for i := 0 to Length(GroupingModifiedArray) - 1 do
          begin
            Item := ELV.Items[i];
            GroupingModifiedArray[i].Caption := Item.Caption;
            GroupingModifiedArray[i].Days := StrToFloat(Item.Captions[1]);
            GroupingModifiedArray[i].Enabled := Item.Checked;      
          end;
        end;
      end;
    IsDirty := False
  end
end;

procedure TTntFormGroupingCriteria.LoadListviewWithFileSizeInfo(GroupingArray: TGroupingFileSizeArray);
var
  i: Integer;
  Item: TEasyItem;
  FileGroup, SpecialGroup: TEasyGroup;
begin
  if Assigned(GroupingArray) then
  begin
    ELV.BeginUpdate;
    try
      ELV.Groups.Clear;
      ELV.ShowGroupMargins := True;
      FileGroup := ELV.Groups.Add;
      FileGroup.Caption := STR_GROUPINGFOLDEROBJECTS;
      SpecialGroup := ELV.Groups.Add;
      SpecialGroup.Caption := STR_GROUPINGSPECIALFOLDEROBJECTS;
      ELV.GroupFont.Style := ELV.GroupFont.Style + [fsBold];
      for i := 0 to Length(GroupingArray) - 1 do
      begin
        if GroupingArray[i].SpecialFolder then
        begin
          Item := SpecialGroup.Items.Add();
          Item.Enabled := False
        end else
          Item := FileGroup.Items.Add();
        Item.Caption := GroupingArray[i].Caption;
        Item.Captions[1] := IntToStr(GroupingArray[i].FileSize);
        Item.Checked := GroupingArray[i].Enabled
      end
    finally
      ELV.EndUpdate();
    end
  end;
end;

procedure TTntFormGroupingCriteria.LoadListviewWithModifiedDateInfo(ModifiedArray: TGroupingModifiedArray);
var
  i: Integer;
  Item: TEasyItem;
begin
  if Assigned(ModifiedArray) then
  begin
    ELV.BeginUpdate;
    try
      ELV.Items.Clear;
      ELV.ShowGroupMargins := False;
      for i := 0 to Length(ModifiedArray) - 1 do
      begin
        Item := ELV.Items.Add();
        Item.Caption := ModifiedArray[i].Caption;
        Item.Captions[1] := FloatToStrF(ModifiedArray[i].Days, ffFixed, 4, 4);
        Item.Checked := ModifiedArray[i].Enabled
      end
    finally
      ELV.EndUpdate();
    end
  end;
end;

procedure TTntFormGroupingCriteria.SetIsDirty(const Value: Boolean);
begin
  FIsDirty := Value;
  SyncActions
end;

procedure TTntFormGroupingCriteria.SyncActions;
begin
  ActionAdd.Enabled := not IsSpecialFolder;
  ActionDefaults.Enabled := not IsSpecialFolder;
  ActionDelete.Enabled := (ELV.Selection.Count > 0) and not IsSpecialFolder;
  ActionSave.Enabled := IsDirty;
  ActionEdit.Enabled := (ELV.Selection.Count = 1) and not IsSpecialFolder;
end;

procedure TTntFormGroupingCriteria.TntFormShow(Sender: TObject);
begin
  FixFormFont(Font);
  SpTBXRadioGroupType.Items.Clear;
  SpTBXRadioGroupType.Items.Add(STR_FILESIZE);
  SpTBXRadioGroupType.Items.Add(STR_MODIFYDATE);
  ELV.Header.Columns[0].Caption := STR_CAPTION;
  SpTBXRadioGroupType.ItemIndex := 0;
  IsDirty := False;
  ClientWidth := SpTBXPanel1.Width + 8;
  ClientHeight := SpTBXPanel1.Height + 8;
  if SpTBXButtonDefaults.Top + SpTBXButtonDefaults.Height > ClientHeight then
   ClientHeight := SpTBXButtonDefaults.Top + SpTBXButtonDefaults.Height + 8
end;

procedure TTntFormGroupingCriteria.SpTBXRadioGroupTypeClick(Sender: TObject);
begin
  CheckForSave;
  if IsEditSize then
  begin
    CurrentCriteria := cFileSize;
    ELV.Header.Columns[1].Caption := STR_GROUPINGEDITFILESIZE;
    LoadListviewWithFileSizeInfo(GroupingFileSizeArray);
  end else
  begin
    CurrentCriteria := cModifyDate;
    ELV.Header.Columns[1].Caption := STR_GROUPINGEDITMODIFIEDDATE;
    LoadListviewWithModifiedDateInfo(GroupingModifiedArray);
  end;
  IsDirty := False
end;

procedure TTntFormGroupingCriteria.ActionAddExecute(Sender: TObject);
var
  Item: TEasyItem;
begin
  ELV.Selection.ClearAll;
  Item := ELV.Groups[0].Items.Add();
  Item.Caption := STR_GROUPINGEDITMODIFIEDNEW;
  Item.Captions[1] := FloatToStr(0.0);
  Item.Checked := True;
  Item.MakeVisible(emvBottom);
  Item.Focused := True;
  Item.Selected := True;
  if EditCriteria then
  begin
    ELV.Sort.SortAll;
    IsDirty := True
  end else
    ELV.Selection.DeleteSelected
end;

procedure TTntFormGroupingCriteria.ActionDeleteExecute(Sender: TObject);
var
  Item: TEasyItem;
  i: Integer;
begin
  Item := ELV.Selection.First;
  if Assigned(Item) then
  begin
    i := Item.Index;
    ELV.Selection.DeleteSelected;
    if (i >= ELV.Items.Count) then
      i := ELV.Items.Count - 1;
    if i > -1 then
      ELV.Items[i].Selected := True;
    IsDirty := True;
  end
end;

procedure TTntFormGroupingCriteria.ActionDefaultsExecute(Sender: TObject);
var
  TempModifyDateArray: TGroupingModifiedArray;
  TempFileSizeArray: TGroupingFileSizeArray;
begin
  ELV.BeginUpdate;
  try
    ELV.Items.Clear;
    if IsEditSize then
    begin
      SetLength(TempFileSizeArray, 0);
      LoadDefaultGroupingFileSizeArray(TempFileSizeArray, False);
      LoadListviewWithFileSizeInfo(TempFileSizeArray);
    end else
    if IsEditModifiedDate then
    begin
      SetLength(TempModifyDateArray, 0);
      LoadDefaultGroupingModifiedArray(TempModifyDateArray, False);
      LoadListviewWithModifiedDateInfo(TempModifyDateArray);
    end;
  finally
    ELV.EndUpdate();
  end;
  IsDirty := True;
end;

procedure TTntFormGroupingCriteria.ActionSaveExecute(Sender: TObject);
begin
  CheckForSave;
end;

procedure TTntFormGroupingCriteria.ELVItemSelectionsChanged(Sender: TCustomEasyListview);
begin
  SyncActions;
end;

procedure TTntFormGroupingCriteria.SpTBXButtonCancelClick(Sender: TObject);
begin
  CheckForSave
end;

procedure TTntFormGroupingCriteria.SpTBXButtonOkClick(Sender: TObject);
begin
  ActionSave.Execute
end;

procedure TTntFormGroupingCriteria.ELVKeyAction(
  Sender: TCustomEasyListview; var CharCode: Word; var Shift: TShiftState;
  var DoDefault: Boolean);
begin
  case CharCode of
    VK_DELETE: ActionDelete.Execute;
    VK_INSERT: ActionAdd.Execute;
  end
end;

procedure TTntFormGroupingCriteria.ELVItemCheckChange(Sender: TCustomEasyListview; Item: TEasyItem);
begin
  IsDirty := True;
end;

procedure TTntFormGroupingCriteria.ActionEditExecute(Sender: TObject);
begin
  EditCriteria;
end;

function TTntFormGroupingCriteria.ELVItemCompare(
  Sender: TCustomEasyListview; Column: TEasyColumn; Group: TEasyGroup;
  Item1, Item2: TEasyItem): Integer;
var
  Diff: extended;
begin
  Diff := StrToFloat(Item1.Captions[1]) - StrToFloat(Item2.Captions[1]);
  if Diff > 0 then
    Result := 1
  else
  if Diff < 0 then
    Result := -1
  else
    Result := 0
end;

procedure TTntFormGroupingCriteria.ELVItemDblClick(
  Sender: TCustomEasyListview; Button: TCommonMouseButton;
  MousePos: TPoint; HitInfo: TEasyHitInfoItem);
begin
  ELV.Selection.ClearAll;
  HitInfo.Item.Selected := True;
  EditCriteria
end;

procedure TTntFormGroupingCriteria.TntFormKeyDown(Sender: TObject;
  var Key: Word; Shift: TShiftState);
begin
  if Key = VK_ESCAPE then
    Close
end;

procedure TTntFormGroupingCriteria.TntFormCreate(Sender: TObject);
begin
  BorderStyle := bsSizeable;
end;

constructor TTntFormGroupingCriteria.CreateEx(AnOwner: TCustomForm);
begin
  if Assigned(AnOwner) then
     WndParent := AnOwner.Handle
   else
     WndParent := 0;
   inherited Create(nil);
end;

procedure TTntFormGroupingCriteria.CreateParams(var Params: TCreateParams);
begin
  inherited;
  if WndParent <> 0 then
    Params.WndParent := WndParent;
end;

end.
