unit UnitFlattenFolderPluginForm;

interface

uses
  Windows,
  Messages,
  SysUtils,
  Variants,
  Classes,
  Graphics,
  Controls,
  Forms,
  TntForms,
  ShlObj,
  Dialogs,
  ImgList,
  StdCtrls,
  EasyListview,
  MPCommonObjects,
  MPCommonUtilities,
  MPShellUtilities,
  MPThreadManager,
  SpTBXControls,
  SpTBXEditors,
  SpTBXItem,
  TntClasses,
  TntStdCtrls,
  UltraFavTree,
  UltraUserStrings,
  UltraUtilities,
  UnitBasePlugInForm,
  UnitInPlaceWndInf,
  VirtualExplorerEasyListview;


const
  WM_ENUMDONE = WM_APP + 100;
  ID_FILTER_BKGND = 1001;          // Resource ID

  FLATTEN_FOLDER_STREAM_VERSION = 0;
  
type
  TFlatFolderEnumThread = class(TCommonThread)
  private
    FItemList: TList;
    FOwnerWnd: TWinControl;
    FPathList: TList;
  protected
    procedure Execute; override;
    property ItemList: TList read FItemList write FItemList;
    property PathList: TList read FPathList write FPathList;
  public
    function EnumCallback(MessageWnd: HWnd; APIDL: PItemIDList; AParent: TNamespace; Data: Pointer; var Terminate: Boolean): Boolean;
    property OwnerWnd: TWinControl read FOwnerWnd write FOwnerWnd;
  end;

  TFormFlatFolderPlugin = class(TBasePluginForm)
    FlatFolderELV: TVirtualMultiPathExplorerEasyListview;
    ImageList: TImageList;
    procedure FlatFolderELVItemSelectionsChanged(Sender: TCustomEasyListview);
  private
    { Private declarations }
    FEnumThread: TFlatFolderEnumThread;
    FItemList: TList;
    FNamespaceList: TList;
    FShownOnce: boolean;
    FSpTBXDockHeader: TSpTBXDock;
    FSpTBXToolbar: TSpTBXToolbar;
  protected
    procedure WMEnumDone(var Msg: TMessage); message WM_ENUMDONE;
    property EnumThread: TFlatFolderEnumThread read FEnumThread write FEnumThread;
    property ItemList: TList read FItemList write FItemList;
    property ShownOnce: boolean read FShownOnce write FShownOnce;
  public
    { Public declarations }
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    function ActionOperation(Action: TInPlaceWindowAction): Boolean; override;
    function GetArrange(var Arrange: TInPlaceWindowArrange): Boolean; override;
    function GetGrouped(var Grouped: Boolean): Boolean; override;
    function GetView(var WindowView: TInPlaceWindowView): Boolean; override;
    function IsActionEnabled(var InPlaceWindowAction: TInPlaceWindowActions): Boolean; override;
    function IsQuickFiltered: Boolean; override;
    function LoadConfig: Boolean; override;
    function SaveConfig: Boolean; override;
    function SetArrange(Arrange: TInPlaceWindowArrange): Boolean; override;
    function SetGrouped(Grouped: Boolean): Boolean; override;
    function SetQuickFilter(Enable: Boolean; FilterMask: WideString): Boolean; override;
    function SetTheme(ThemeName: PChar): Boolean; override;
    function SetView(View: TInPlaceWindowView): Boolean; override;
    function ShowWindow(Show: Boolean): Boolean; override;
    function StatusBarInfo(var ItemCount: Cardinal; var SelectedItemCount: Cardinal; var SelectedItemSize: Int64): Boolean; override;
    function TabImage(var AnImageList: TCustomImageList; var AnImageIndex: Integer): Boolean; override;
    property NamespaceList: TList read FNamespaceList write FNamespaceList;

    property SpTBXDockHeader: TSpTBXDock read FSpTBXDockHeader write FSpTBXDockHeader;
    property SpTBXToolbar: TSpTBXToolbar read FSpTBXToolbar write FSpTBXToolbar;
  end;


implementation

{$R *.dfm}

function FlattenFolderSettingsFile: WideString;
begin
  Result := AppFilesPath + '\' + STR_FLATTENFOLDER_FILE
end;

{ TFlatFolderEnumThread}

function TFlatFolderEnumThread.EnumCallback(MessageWnd: HWnd; APIDL: PItemIDList; AParent: TNamespace; Data: Pointer; var Terminate: Boolean): Boolean;
var
  NS: TNamespace;
begin
  NS := TNamespace.Create( PIDLMgr.AppendPIDL(AParent.AbsolutePIDL, APIDL), nil);
  if Assigned(NS) then
  begin
    if NS.Folder and NS.FileSystem then
    begin
      NS.EnumerateFolder(0, True, True, False, EnumCallback, Data);
      FreeAndNil(NS)
    end else
    begin
      if NS.FileSystem then
        TList( Data).Add(NS)
      else
        FreeAndNil(NS)
    end
  end;
  Terminate := Self.Terminated;
  Result := True;
end;

procedure TFlatFolderEnumThread.Execute;
var
  i: Integer;
  NS: TNamespace;
begin
  i := 0;
  while not Terminated and (i < PathList.Count) do
  begin
    NS := TNamespace(PathList[i]);
    NS.EnumerateFolder(0, True, True, False, EnumCallback, ItemList);
    Inc(i)
  end;
  if not Terminated and OwnerWnd.HandleAllocated then
    PostMessage(OwnerWnd.Handle, WM_ENUMDONE, 0, 0);
end;

{ TFormFlatFolderPlugin }
constructor TFormFlatFolderPlugin.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
end;

destructor TFormFlatFolderPlugin.Destroy;
var
  i: Integer;
begin
  EnumThread.Terminate;
  while not EnumThread.Finished do
    Sleep(100);
  FreeAndNil(FEnumThread);

  if Assigned(NamespaceList) then
  begin
    try
      for i := 0 to NamespaceList.Count - 1 do
        TObject(NamespaceList[i]).Free;
    finally
      NamespaceList.Count := 0;
      FreeAndNil(FNamespaceList);
    end
  end;
  if Assigned(ItemList) then
  begin
    try
      for i := 0 to ItemList.Count - 1 do
        TObject(ItemList[i]).Free;
    finally
      ItemList.Count := 0;
      FreeAndNil(FItemList);
    end
  end;

  inherited Destroy;
end;

function TFormFlatFolderPlugin.ActionOperation(Action: TInPlaceWindowAction): Boolean;
begin
  case Action of
    pwaCopy: DesktopFolder.Copy(FlatFolderELV.SelectedToNamespaceArray);
    pwaCut: DesktopFolder.Cut(FlatFolderELV.SelectedToNamespaceArray);
    pwaPaste: begin end; // do nothing, not supported
    pwaPasteShortCut: begin end; // do nothing, not supported
    pwaDelete: DesktopFolder.Delete(FlatFolderELV.SelectedToNamespaceArray);
    pwaSelectAll: FlatFolderELV.Selection.SelectAll;
    pwaInvertSelection: FlatFolderELV.Selection.Invert;
    pwaProperites: DesktopFolder.ShowPropertySheetMulti(FlatFolderELV.SelectedToNamespaceArray, True, True);
  end;
  Result := True;
end;

function TFormFlatFolderPlugin.GetArrange(var Arrange: TInPlaceWindowArrange): Boolean;
begin
  Result:= inherited GetArrange(Arrange);
end;

function TFormFlatFolderPlugin.GetGrouped(var Grouped: Boolean): Boolean;
begin
  Result:= True;
  Grouped := FlatFolderELV.Grouped
end;

function TFormFlatFolderPlugin.GetView(var WindowView: TInPlaceWindowView): Boolean;
begin
  Result := True;
  case FlatFolderELV.View of
    elsIcon: WindowView := pwvIcon;
    elsSmallIcon: WindowView := pwvSmallIcon;
    elsList: WindowView := pwvList;
    elsReport: WindowView := pwvReport;
    elsThumbnail: WindowView := pwvThumbnail;
    elsTile: WindowView := pwvTile;
  else
    Result := False
  end
end;

function TFormFlatFolderPlugin.IsActionEnabled(var InPlaceWindowAction: TInPlaceWindowActions): Boolean;
var
  NSA: TNamespaceArray;
  i: Integer;
begin
  NSA := FlatFolderELV.SelectedToNamespaceArray;
  if pwaCopy in InPlaceWindowAction then
  begin
    i := 0;
    if Length(NSA) = 0 then
      Exclude(InPlaceWindowAction, pwaCopy);
    while (pwaCopy in InPlaceWindowAction) and (i < Length(NSA)) do
    begin
      if not NSA[i].CanCopy then
        Exclude(InPlaceWindowAction, pwaCopy);
      Inc(i)
    end;
  end;
  if pwaCut in InPlaceWindowAction then
  begin
    i := 0;
    if Length(NSA) = 0 then
      Exclude(InPlaceWindowAction, pwaCut);
    while (pwaCut in InPlaceWindowAction) and (i < Length(NSA)) do
    begin
      if not NSA[i].CanMove then
        Exclude(InPlaceWindowAction, pwaCut);
      Inc(i)
    end;
  end;
  if pwaPaste in InPlaceWindowAction then
  begin
    // don't support it
    Exclude(InPlaceWindowAction, pwaPaste);
  end;
  if pwaPasteShortCut in InPlaceWindowAction then
  begin
    // don't support it
    Exclude(InPlaceWindowAction, pwaPasteShortCut);
  end;
  if pwaDelete in InPlaceWindowAction then
  begin
    i := 0;
    if Length(NSA) = 0 then
      Exclude(InPlaceWindowAction, pwaDelete);
    while (pwaDelete in InPlaceWindowAction) and (i < Length(NSA)) do
    begin
      if not NSA[i].CanDelete then
        Exclude(InPlaceWindowAction, pwaDelete);
      Inc(i)
    end;
  end;
  if pwaSelectAll in InPlaceWindowAction then
  begin
    Include(InPlaceWindowAction, pwaSelectAll);
  end;
  if pwaInvertSelection in InPlaceWindowAction then
  begin
    Include(InPlaceWindowAction, pwaInvertSelection);
  end;
  if pwaProperites in InPlaceWindowAction then
  begin
    if FlatFolderELV.Selection.Count = 0 then
      Exclude(InPlaceWindowAction, pwaProperites);
  end;
  Result := True
end;

function TFormFlatFolderPlugin.IsQuickFiltered: Boolean;
begin
  Result := FlatFolderELV.QuickFiltered;
end;

function TFormFlatFolderPlugin.LoadConfig: Boolean;
var
  F: TTntFileStream;
begin
  Result := True;
  try
    if WideFileExists(FlattenFolderSettingsFile) then
    begin
      F := TTntFileStream.Create(FlattenFolderSettingsFile, fmOpenRead or fmShareDenyWrite);
      try
        FlatFolderELV.Header.StreamColumns := True;
        FlatFolderELV.Header.LoadFromStream(F, FLATTEN_FOLDER_STREAM_VERSION)
      finally
        F.Free
      end
    end
  except
    Result := False
  end
end;

function TFormFlatFolderPlugin.SaveConfig: Boolean;
var
  F: TTntFileStream;
begin
  Result := True;
  try
    F := TTntFileStream.Create(FlattenFolderSettingsFile, fmCreate or fmShareDenyWrite);
    try
      FlatFolderELV.Header.StreamColumns := True;
      FlatFolderELV.Header.SaveToStream(F, FLATTEN_FOLDER_STREAM_VERSION)
    finally
      F.Free
    end
  except
    Result := False
  end
end;

function TFormFlatFolderPlugin.SetArrange(Arrange: TInPlaceWindowArrange): Boolean;
begin
  Result:= inherited SetArrange(Arrange);
end;

function TFormFlatFolderPlugin.SetGrouped(Grouped: Boolean): Boolean;
begin
  Result:= True;
  FlatFolderELV.Grouped := Grouped
end;

function TFormFlatFolderPlugin.SetQuickFilter(Enable: Boolean; FilterMask: WideString): Boolean;
begin
  if Enabled then
    FlatFolderELV.QuickFilterMask := FilterMask;
    
  FlatFolderELV.QuickFiltered := Enable;
  FlatFolderELV.BackGround.Enabled := Enable;
  Result := True;
end;

function TFormFlatFolderPlugin.SetTheme(ThemeName: PChar): Boolean;
begin
  Result:= inherited SetTheme(ThemeName);
  FlatFolderELV.Invalidate;
  Invalidate
end;

function TFormFlatFolderPlugin.SetView(View: TInPlaceWindowView): Boolean;
begin
  Result := True;
  case View of
    pwvIcon: FlatFolderELV.View := elsIcon;
    pwvSmallIcon: FlatFolderELV.View := elsSmallIcon;
    pwvList: FlatFolderELV.View := elsList;
    pwvReport: FlatFolderELV.View := elsReport;
    pwvThumbnail: FlatFolderELV.View := elsThumbnail;
    pwvTile: FlatFolderELV.View := elsTile;
  else
    Result := False
  end
end;

function TFormFlatFolderPlugin.ShowWindow(Show: Boolean): Boolean;
begin
  Result:= inherited ShowWindow(Show);
  FixFormFont(Font);
  if Show and not ShownOnce then
  begin
    ItemList := TList.Create;
    EnumThread := TFlatFolderEnumThread.Create(True);
    EnumThread.PathList := NamespaceList;
    EnumThread.ItemList := ItemList;
    EnumThread.OwnerWnd := Self;
    EnumThread.Resume;
    FlatFolderELV.BackGround.Image.LoadFromResourceID(hInstance, ID_FILTER_BKGND);
    FlatFolderELV.BackGround.Transparent := True;
    FlatFolderELV.BackGround.OffsetTrack := True;
    FlatFolderELV.BackGround.AlphaBlend := True;
    FlatFolderELV.BackGround.Tile := False;
    FlatFolderELV.BackGround.BlendAlpha := 192;
    FlatFolderELV.BackGround.OffsetX := ClientWidth - 120;
    FlatFolderELV.BackGround.OffsetY := ClientHeight - 70;
    ShownOnce := True;
  end
end;

function TFormFlatFolderPlugin.StatusBarInfo(var ItemCount: Cardinal; var SelectedItemCount: Cardinal; var SelectedItemSize: Int64): Boolean;
var
  Item: TExplorerItem;
begin
  Result := True;
  ItemCount := FlatFolderELV.ItemCount;
  SelectedItemCount := FlatFolderELV.Selection.Count;
  if SelectedItemSize = -1 then
  begin
    SelectedItemSize := 0;
    if Assigned(FlatFolderELV) then
    begin
      Item := FlatFolderELV.Selection.First as TExplorerItem;
      while Assigned(Item) do
      begin
        Inc(SelectedItemSize, Item.Namespace.SizeOfFileInt64);
        Item := FlatFolderELV.Selection.Next(Item) as TExplorerItem;
      end;
    end;
  end else
    SelectedItemSize := 0;
end;

function TFormFlatFolderPlugin.TabImage(var AnImageList: TCustomImageList; var AnImageIndex: Integer): Boolean;
begin
  AnImageList := ImageList;
  AnImageIndex := 0;
  Result := True;
end;

procedure TFormFlatFolderPlugin.WMEnumDone(var Msg: TMessage);
var
  i: Integer;
begin
  try
    FlatFolderELV.BeginUpdate;
    try
      for i := 0 to ItemList.Count - 1 do
        FlatFolderELV.AddCustomItem(nil, TNamespace( ItemList[i]), True);
    finally
      FlatFolderELV.EndUpdate();
      Notifier.UpdateUI(nuiStatusBar);
    end
  finally
    ItemList.Count := 0
  end
end;

procedure TFormFlatFolderPlugin.FlatFolderELVItemSelectionsChanged(
  Sender: TCustomEasyListview);
begin
  Notifier.UpdateUI(nuiStatusBar);
  Notifier.UpdateUI(nuiActionbar);
end;

end.
