unit UnitInPlaceWndInf;

// The contents of this file are subject to the Mozilla Public License
// Version 1.1 (the "License"); you maynot use this file except in compliance
// with the License. You may obtain a copy of the License at http://www.mozilla.org/MPL/
//
// Alternatively, you may redistribute this library, use and/or modify it under the terms of the
// GNU Lesser General Public License as published by the Free Software Foundation;
// either version 2.1 of the License, or (at your option) any later version.
// You may obtain a copy of the LGPL at http://www.gnu.org/copyleft/.
//
// Software distributed under the License is distributed on an "AS IS" basis,
// WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for the
// specific language governing rights and limitations under the License.

interface

uses
  Windows,
  Classes,
  Controls,
  ActiveX,
  Forms,
  TntForms,
  ImgList,
  VirtualExplorerTree,
  VirtualExplorerEasyListview,
  TB2Item,
  ShlObj,
  TntClasses,
  SpTBXTabs;

type
  TUltraNotifierUIUpdate = (
    nuiStatusbar,   // The main UI should update the Statusbar
    nuiActionbar,   // The main UI should update the Listview Action bar (copy/paste/cut etc)
    nuiTabItem      // Update the passed TabItem with the new Caption/Hint/Image information
  );

  TViewClipboardAction = (
    vcaCopy,
    vcaCut,
    vcaPaste,
    vcaPasteAsShortcut,
    vcaCopyP
  );

  TViewSelectionAction = (
    vsaSelectedDelete,       // Instructs view to delete the selected files
    vsaSelectedInvert,       // Instructs view to invert its selection
    vsaSelectAll,            // Instructs view to select all its items
    vsaSelectAllFiles,       // Instructs view to select all its file items
    vsaSelectAllFolders,     // Instructs view to select all its folder items
    vsaSelectMatchText,      // Instructs view to select all its items matching a text string
    vsaUnSelectFiles,        // Instructs view to unselect all its file items
    vsaUnSelectFolders,      // Instructs view to unselect all its folder items
    vsaUnSelectMatchText     // Instructs view to unselect all its items that match a text string
  );

  TViewSelectRecAction = record
    Action: TViewSelectionAction;      // Action to perform
    Combine: Boolean;                  // Combine with currently selected or clear first
    Text: WideString;                  // For match text calls
    SetFocusToView: Boolean;           // Sets the focus to the view after executing the command
  end;
  PViewSelectRecAction = ^TViewSelectRecAction;    

  TViewQuickFilterRecAction = record
    Enable: Boolean;
    Mask: WideString;
    ShowImage: Boolean;
  end;
  PViewQuickFilterRecAction = ^TViewQuickFilterRecAction;

  TViewCopyPathAction = record
    DosPath,               // convert to 8:3 file/folder names
    FileNamesOnly: Boolean  // only copy the file names not the complete path
  end;
  PViewCopyPathAction = ^TViewCopyPathAction;

  TViewExecuteAction = (
    veaCreateNewFolder,      // Instructs view to create a new folder and place in edit mode
    veaClipboard,            // Instructs view to perform a clipboard action of type TViewClipboardAction
    veaCalcualteFolderSizes, // Instructs view to calculate and display the size of the displayed folders  LoWord(Data) = *Boolean* Calcualate Selected Only;  HiWord(Data) = *Boolean* Force Calculation even if user settings say don't calculate
    veaFolderSizesClear,     // Instructs view to clear the folder sizes it may have displayed
    veaInvalidateAndRedraw,  // Instructs view to invalidate and redraw the window
    veaSelect,               // Instructs view to perform a selection action (Data = PViewSelectRecAction)
    veaQuickFilter,          // Instructs view to set/clear its Quick Filter  (Data = PViewQuickFilterRecAction)
    vaeRebuild,              // Instructs view to rebuild it contents
    vaeCopyPath              // Instructs view to copy file names/paths to the clipboard (Data = PViewCopyPathAction)
  );


  IUltraNotifier = interface(IUnknown)
  ['{9713099B-9D1B-4C2F-95FA-F84ECB6BC00B}']
    procedure FocusChanged(NewFocus: TWinControl);
    procedure UpdateUI(UpdateType: TUltraNotifierUIUpdate; Data: Integer);
    procedure ViewClipboardChanged;
  end;


  IUltraInPlaceWindow = interface(IUnknown)
  ['{BCAAE619-7EC3-4E96-AE58-E2CE86E1DEC0}']
    function ActiveViewUI(DoActiveate: Boolean): Boolean;
    procedure ConnectEventsAndBkGndMenu(DoConnect, DoConnectBkGndMenu: Boolean);
    function DestroyWindow: Boolean;
    function GetHandle(var WindowHandle: hWnd): Boolean;
    function GetObject(var APlugin: TTntFrame): Boolean;
    function HandleMessage(var Msg: TMsg; var Handled: Boolean): Boolean;
    function HintMsg(var AHint: WideString): Boolean;
    function LoadConfig(StreamVer: Integer; ConfigStream: TTntFileStream): Boolean;
    function Persistent: Boolean;       // If True then the window can store its state to disk and be recreated later
    function RegisterNotifier(ANotifier: IUltraNotifier): Boolean;
    function SaveConfig(StreamVer: Integer; ConfigStream: TTntFileStream): Boolean;
    procedure SetOwnerItem(ATabItem: TSpTBXTabItem);
    function SetParentWnd(AParent: TWinControl): Boolean;
    function SetTheme(ThemeName: PChar): Boolean;
    function SetWindowBounds(AWidth, AHeight: Integer): Boolean;
    function ShowWindow(Show: Boolean): Boolean;
    function TabCaption(var ACaption: WideString): Boolean;
    function TabImage(var AnImageList: TCustomImageList; var AnImageIndex: Integer): Boolean;
    // Returns information about the View itself
    procedure ViewBeginUpdate;
    procedure ViewEndUpdate;
    function ViewItemCount: Integer;
    function ViewItemFirst: TExplorerItem;
    function ViewItemNext(Item: TExplorerItem): TExplorerItem;
    function ViewRootPIDL: PItemIDList;     // Caution this returns the actual PIDL, NOT a COPY!!!!
    function ViewSelectionCount: Integer;
    function ViewSelectionFirst: TExplorerItem;
    function ViewExecuteAction(Action: TViewExecuteAction; Data: Integer): Boolean;
    procedure ViewSelectionChangeLock(DoLock: Boolean);
    function ViewSelectionNext(Item: TExplorerItem): TExplorerItem;
    procedure ViewSetRootPIDL(PIDL: PItemIDList);  // Expects the caller to free the PIDL, make a copy if needed for later
    function ViewShellFolder: IShellFolder;
    function ViewTopItem: TExplorerItem;
  end;


(*  TUltraInplaceNotifyTabState = (
    ntsActivating,
    ntsDeActivating,
    ntsDestroying,
    ntsCreating
  );

  IUltraInplaceNotifications = interface(IUnknown)
  ['{5010F3C6-34C7-4CE5-B719-26237E9D6E79}']
    procedure TabChanging(TabState: TUltraInplaceNotifyTabState);
    procedure RootFolderChanged(Root: PItemIDList);
  end;    *)

implementation

end.
