unit UnitListerPluginsForm;

// The contents of this file are subject to the Mozilla Public License
// Version 1.1 (the "License"); you maynot use this file except in compliance
// with the License. You may obtain a copy of the License at http://www.mozilla.org/MPL/
//
// Alternatively, you may redistribute this library, use and/or modify it under the terms of the
// GNU Lesser General Public License as published by the Free Software Foundation;
// either version 2.1 of the License, or (at your option) any later version.
// You may obtain a copy of the LGPL at http://www.gnu.org/copyleft/.
//
// Software distributed under the License is distributed on an "AS IS" basis,
// WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for the
// specific language governing rights and limitations under the License.

interface

uses
  Windows,
  Messages,
  SysUtils,
  Variants,
  Classes,
  Graphics,
  Controls,
  Forms,
  Dialogs,
  TntForms,
  TntDialogs,
  SpTBXControls,
  SpTBXDkPanels,
  StdCtrls,
  CheckLst,
  TntCheckLst,
  SpTBXEditors,
  MPCommonObjects,
  EasyListview,
  MPCommonUtilities,
  UltraOptions,
  TntSysUtils,
  WLXProc,
  StrUtils,
  DKLang,
  SpTBXItem,
  UltraUtilities,
  Types,
  UltraUserStrings;

type
  TTntFormPlugins = class(TTntForm)
    SpTBXPanel1: TSpTBXPanel;
    SpTBXLabel1: TSpTBXLabel;
    SpTBXButtonAdd: TSpTBXButton;
    SpTBXButtonDelete: TSpTBXButton;
    SpTBXButtonCancel: TSpTBXButton;
    SpTBXButtonOK: TSpTBXButton;
    EasyListviewPlugin: TEasyListview;
    SpTBXButtonEditExtensions: TSpTBXButton;
    SpTBXButtonMoveDown: TSpTBXButton;
    SpTBXButtonMoveUp: TSpTBXButton;
    TntOpenDialog: TTntOpenDialog;
    DKLanguageControllerPlugins: TDKLanguageController;
    procedure SpTBXButtonAddClick(Sender: TObject);
    procedure SpTBXButtonDeleteClick(Sender: TObject);
    procedure EasyListviewPluginItemSelectionsChanged(Sender: TCustomEasyListview);
    procedure SpTBXButtonOKClick(Sender: TObject);
    procedure TntFormShow(Sender: TObject);
    procedure SpTBXButtonEditExtensionsClick(Sender: TObject);
    procedure SpTBXButtonMoveUpClick(Sender: TObject);
    procedure SpTBXButtonMoveDownClick(Sender: TObject);
    procedure TntFormKeyDown(Sender: TObject; var Key: Word;
      Shift: TShiftState);
    procedure TntFormCreate(Sender: TObject);
  private
    FSpTBXDockHeader: TSpTBXDock;  // For stealing its painting routines
    { Private declarations }
    FSpTBXToolbar: TSpTBXToolbar;
    FWndParent: HWnd;
  protected
    function SelectionCanMoveDown: Boolean;
    function SelectionCanMoveUp: Boolean;
    procedure AddPlugin(Plugin: TPlugIn);
    procedure LoadPluginListview;
    procedure CreateParams(var Params: TCreateParams); override;
    property WndParent: HWnd read FWndParent write FWndParent;
    procedure SyncButtonState;
  public
    { Public declarations }
    constructor CreateEx(AnOwner: TCustomForm);
    destructor Destroy; override;

    property SpTBXDockHeader: TSpTBXDock read FSpTBXDockHeader write FSpTBXDockHeader;
    property SpTBXToolbar: TSpTBXToolbar read FSpTBXToolbar write FSpTBXToolbar;
  end;

implementation

{$R *.DFM}

uses
  UnitListerPluginsExtensionsForm;

constructor TTntFormPlugins.CreateEx(AnOwner: TCustomForm);
begin
  if Assigned(AnOwner) then
     WndParent := AnOwner.Handle
   else
     WndParent := 0;
   inherited Create(nil);
end;

procedure TTntFormPlugins.CreateParams(var Params: TCreateParams);
begin
  inherited;
  if WndParent <> 0 then
    Params.WndParent := WndParent;
end;

destructor TTntFormPlugins.Destroy;
begin
  inherited Destroy;
end;

function TTntFormPlugins.SelectionCanMoveDown: Boolean;
begin
  Result := False;
  if EasyListviewPlugin.Items.Count > 0 then
    Result := (EasyListviewPlugin.Selection.Count > 0) and not(EasyListviewPlugin.Items[EasyListviewPlugin.Items.Count-1].Selected);
end;

function TTntFormPlugins.SelectionCanMoveUp: Boolean;
begin
  Result := False;
  if EasyListviewPlugin.Items.Count > 0 then
    Result := (EasyListviewPlugin.Selection.Count > 0) and not(EasyListviewPlugin.Items[0].Selected);
end;

procedure TTntFormPlugins.AddPlugin(Plugin: TPlugIn);
var
  Item: TEasyItem;
begin
  Item := EasyListviewPlugin.Items.Add();
  Item.Caption := WideExtractFileName(Plugin.Path);
  Item.Checked := Plugin.Enabled;
  Item.Captions[1] := Plugin.Extensions;
  Item.Captions[2] := Plugin.Path;
end;

procedure TTntFormPlugins.LoadPluginListview;
var
  i: Integer;
begin
  EasyListviewPlugin.Header.Columns[0].Caption := STR_PLUGIN_FILENAME;
  EasyListviewPlugin.Header.Columns[1].Caption := STR_PLUGIN_EXTENSION;
  EasyListviewPlugin.Header.Columns[2].Caption := STR_PLUGIN_PATH;
  EasyListviewPlugin.Items.Clear;
  for i := 0 to Attribs.Viewer.PlugIns.Count - 1 do
    AddPlugin(TPlugIn(Attribs.Viewer.PlugIns.Items[i]));
end;

procedure TTntFormPlugins.SpTBXButtonAddClick(Sender: TObject);

    function IsDuplicate(Path: WideString): Boolean;
    var
      i: Integer;
    begin
      i := 0;
      Result := False;
      Path := WideUpperCase(Path);
      while (i < EasyListviewPlugin.Items.Count) and not Result do
      begin
        Result := WideCompareText(Path, WideUpperCase( EasyListviewPlugin.Items[i].Caption)) = 0;
        Inc(i)
      end
    end;

var
  i: Integer;
  Plugin: TPlugin;
begin
  TntOpenDialog.Filter := 'WLX Plugins (*.wlx)|*.WLX';
  TntOpenDialog.Options := TntOpenDialog.Options + [ofAllowMultiSelect];
  if TntOpenDialog.Execute then
  begin
    Plugin := TPlugin.Create(nil);
    for i := 0 to TntOpenDialog.Files.Count - 1 do
    begin
      if not IsDuplicate(TntOpenDialog.Files[i]) then
      begin
        Plugin.Path := TntOpenDialog.Files[i];
        Plugin.Enabled := True;
        Plugin.Extensions := WlxGetDetectString(Plugin.Path);;
        AddPlugin(PlugIn)
      end
    end;
    Plugin.Free;
  end
end;

procedure TTntFormPlugins.SpTBXButtonDeleteClick(Sender: TObject);
begin
  EasyListviewPlugin.Selection.DeleteSelected;
end;

procedure TTntFormPlugins.EasyListviewPluginItemSelectionsChanged(
  Sender: TCustomEasyListview);
begin
  SyncButtonState;
end;

procedure TTntFormPlugins.SpTBXButtonOKClick(Sender: TObject);
var
  Plugin: TPlugin;
  i: Integer;
begin
  Attribs.Viewer.PlugIns.Clear;
  for i := 0 to EasyListviewPlugin.Items.Count - 1 do
  begin
    Plugin := Attribs.Viewer.PlugIns.Add as TPlugin;
    Plugin.Path := EasyListviewPlugin.Items[i].Captions[2];
    Plugin.Enabled := EasyListviewPlugin.Items[i].Checked;
    Plugin.Extensions := EasyListviewPlugin.Items[i].Captions[1];
  end;
  Attribs.SaveCallback(Attribs)
end;

procedure TTntFormPlugins.SyncButtonState;
begin
  SpTBXButtonDelete.Enabled := EasyListviewPlugin.Selection.Count > 0;
  SpTBXButtonEditExtensions.Enabled := EasyListviewPlugin.Selection.Count = 1;
  SpTBXButtonMoveUp.Enabled := SelectionCanMoveUp;
  SpTBXButtonMoveDown.Enabled := SelectionCanMoveDown;
end;

procedure TTntFormPlugins.TntFormShow(Sender: TObject);
begin
  FixFormFont(Font);
  LoadPluginListview;
  ClientWidth := SpTBXPanel1.Width + 8;
  ClientHeight := SpTBXPanel1.Height + 8;
  if SpTBXButtonCancel.Top + SpTBXButtonCancel.Height > ClientHeight then
   ClientHeight := SpTBXButtonCancel.Top + SpTBXButtonCancel.Height + 8
end;

procedure TTntFormPlugins.SpTBXButtonEditExtensionsClick(Sender: TObject);
var
  TntFormExtEdit: TTntFormExtEdit;
  Item: TEasyItem;
begin
  TntFormExtEdit := TTntFormExtEdit.CreateEx(nil);
  try
    Item := EasyListviewPlugin.Selection.First;
    if Assigned(Item) then
    begin
      TntFormExtEdit.PluginPath := Item.Captions[2];
      TntFormExtEdit.SpTBXEditExt.Text := Item.Captions[1];
      if TntFormExtEdit.ShowModal = mrOK then
        Item.Captions[1] := TntFormExtEdit.SpTBXEditExt.Text
    end
  finally
   TntFormExtEdit.Release
  end
end;

procedure TTntFormPlugins.SpTBXButtonMoveUpClick(Sender: TObject);
var
  i: Integer;
  Selected: TEasyItemArray;
  IndexArray: TIntegerDynArray;
begin
  Selected := EasyListviewPlugin.Selection.SelectedToArray;
  SetLength(IndexArray, Length(Selected));
  for i := 0 to Length(Selected) - 1 do
    IndexArray[i] := Selected[i].Index;

  for i := 0 to Length(Selected) - 1 do
    EasyListviewPlugin.Items.Exchange(IndexArray[i] - 1, IndexArray[i]);
  SyncButtonState
end;

procedure TTntFormPlugins.SpTBXButtonMoveDownClick(Sender: TObject);
var
  i: Integer;
  Selected: TEasyItemArray;
  IndexArray: TIntegerDynArray;
begin
  Selected := EasyListviewPlugin.Selection.SelectedToArray;
  SetLength(IndexArray, Length(Selected));
  for i := 0 to Length(Selected) - 1 do
    IndexArray[i] := Selected[i].Index;

  for i := Length(Selected) - 1 downto 0  do
    EasyListviewPlugin.Items.Exchange(IndexArray[i] + 1, IndexArray[i]);
  SyncButtonState
end;

procedure TTntFormPlugins.TntFormKeyDown(Sender: TObject; var Key: Word;
  Shift: TShiftState);
begin
  if Key = VK_ESCAPE then
    Close
end;

procedure TTntFormPlugins.TntFormCreate(Sender: TObject);
begin
  BorderStyle := bsSizeable;
end;

end.
