unit UnitOptionsFrame;

// The contents of this file are subject to the Mozilla Public License
// Version 1.1 (the "License"); you maynot use this file except in compliance
// with the License. You may obtain a copy of the License at http://www.mozilla.org/MPL/
//
// Alternatively, you may redistribute this library, use and/or modify it under the terms of the
// GNU Lesser General Public License as published by the Free Software Foundation;
// either version 2.1 of the License, or (at your option) any later version.
// You may obtain a copy of the LGPL at http://www.gnu.org/copyleft/.
//
// Software distributed under the License is distributed on an "AS IS" basis,
// WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for the
// specific language governing rights and limitations under the License.

interface

uses
  Windows,
  Messages,
  SysUtils,
  Variants,
  Classes,
  Graphics,
  Controls,
  Forms,
  Dialogs,
  TntForms,
  ComCtrls,
  DKLang,
  EasyListview,
  MPShellUtilities,
  SpTBXItem,
  UltraOptions,
  UltraUserStrings,
  UnitOptionsTreeview,
  UnitOptionsListview,
  UnitOptionsListviewColoring,
  UnitOptionsListviewColumns,
  UnitOptionsListviewThumbnails,
  UnitOptionsListviewGrouping,
  UnitOptionsFilter,
  UnitOptionsCmdLine,
  UnitOptionsToolbar,
  UnitOptionsToolbarDrives,
  UnitOptionsDropStack,
  UnitOptionsViewer,
  UnitOptionsFolderSizes,
  UnitOptionsTabs,
  UnitOptionsHistory,
  UnitOptionsFavorites,
  UnitOptionsGeneral,
  UnitOptionsMouse,
  UnitOkDialog,
  SpTBXControls,
  SpTBXDkPanels,
  ExtCtrls,
  TntDialogs,
  UnitOptionsBase,
  TntComCtrls,
  ActnList,
  TntActnList,
  ImgList,
  MPCommonUtilities,
  VirtualExplorerEasyListview;

type
  TTntOptionsBaseFormClass = class of TTntOptionsBaseForm;
  TOptionFrames = array[0..MAX_OPTIONFRAMES-1] of TTntOptionsBaseForm;

const
  FRAME_FORMS: array[0..MAX_OPTIONFRAMES-1] of TTntOptionsBaseFormClass = (
    TTntFormOptionsTreeview,
    TTntFormOptionsListview,
    TTntFormOptionsListviewColoring,
    TTntFormOptionsListviewColumns,
    TTntFormOptionsListviewGrouping,
    TTntFormOptionsListviewThumbnails,
    TTntFormOptionsHistory,
    TTntFormOptionsFavorites,
    TTntFormOptionsFilter,
    TTntFormOptionsCmdLine,
    TTntFormOptionsDropStack,
    TTntFormOptionsViewer,
    TTntFormOptionsToolbar,
    TTntFormOptionsToolbarDrives,
    TTntFormOptionsFolderSizes,
    TTntFormOptionsTabs,
    TTntFormOptionsMouse,
    TTntFormOptionsGeneral
  );

  ID_OPTION_TREEVIEW = 0;
  ID_OPTION_LISTVIEW = 1;
  ID_OPTION_LISTVIEWCOLORING = 2;
  ID_OPTION_LISTVIEWCOLUMNS = 3;
  ID_OPTION_LISTVIEWGROUPING = 4;
  ID_OPTION_LISTVIEWTHUMBNAILS = 5;
  ID_OPTION_HISTORY = 6;
  ID_OPTION_FAVORITES = 7;
  ID_OPTION_FILTER = 8;
  ID_OPTION_CMDLINE = 9;
  ID_OPTION_DROPSTACK = 10;
  ID_OPTION_VIEWER = 11;
  ID_OPTION_TOOLBAR = 12;
  ID_OPTION_TOOLBARDRIVES = 13;
  ID_OPTION_FOLDERSIZE = 14;
  ID_OPTION_TABS = 15;
  ID_OPTION_MOUSE = 16;
  ID_OPTION_GENERAL = 17;
  ID_OPTION_LAST = ID_OPTION_GENERAL;


type
  TTntFormOptionsFrame = class(TTntForm)
    DKLanguageController1: TDKLanguageController;
    TntTreeView: TTntTreeView;
    FontDialog: TFontDialog;
    ColorDialog: TColorDialog;
    TntOpenDialog: TTntOpenDialog;
    SpTBXPanelButtons: TSpTBXPanel;
    PanelFrame: TPanel;
    SpTBXButtonOk: TSpTBXButton;
    SpTBXButtonCancel: TSpTBXButton;
    SpTBXButtonApply: TSpTBXButton;
    SpTBXButtonHelp: TSpTBXButton;
    SpTBXCheckBoxShowPopupHints: TSpTBXCheckBox;
    TntActionListOptions: TTntActionList;
    ActionOptionsOk: TTntAction;
    ActionOptionsCancel: TTntAction;
    ActionOptionsApply: TTntAction;
    ActionOptionsHelp: TTntAction;
    ImageList: TImageList;
    procedure TreeViewChange(Sender: TObject; Node: TTreeNode);
    procedure TntFormShow(Sender: TObject);
    procedure SpTBXButtonOkClick(Sender: TObject);
    procedure SpTBXButtonCancelClick(Sender: TObject);
    procedure SpTBXButtonApplyClick(Sender: TObject);
    procedure SpTBXButtonHelpClick(Sender: TObject);
    procedure TntFormCreate(Sender: TObject);
    procedure TntFormDestroy(Sender: TObject);
    procedure ActionOptionsOkExecute(Sender: TObject);
    procedure ActionOptionsCancelExecute(Sender: TObject);
    procedure ActionOptionsApplyExecute(Sender: TObject);
    procedure ActionOptionsHelpExecute(Sender: TObject);
    procedure TntFormKeyDown(Sender: TObject; var Key: Word; Shift: TShiftState);
    procedure SpTBXCheckBoxShowPopupHintsClick(Sender: TObject);
    procedure DKLanguageController1LanguageChanged(Sender: TObject);
    procedure TntTreeViewCollapsing(Sender: TObject; Node: TTreeNode; var AllowCollapse: Boolean);
    procedure TntFormCloseQuery(Sender: TObject; var CanClose: Boolean);
  private
    FCurrentFrame: TTntOptionsBaseForm;
    FInitialOptionsFrame: Integer;
    FIsDirty: Boolean;
    FLockTreeChange: Boolean;
    FTempColor: TColor;
    FTempFont: TFont;
    FSpTBXDockHeader: TSpTBXDock;
    FSpTBXToolbar: TSpTBXToolbar;
    FAttribs: TUltraAttribs;
    FWndParent: HWnd;
    function GetCmdLine: TTntFormOptionsCmdLine;
    function GetDropStack: TTntFormOptionsDropStack;
    function GetFavorites: TTntFormOptionsFavorites;
    function GetFilter: TTntFormOptionsFilter;
    function GetFolderSize: TTntFormOptionsFolderSizes;
    function GetGeneral: TTntFormOptionsGeneral;
    function GetHistory: TTntFormOptionsHistory;
    function GetListview: TTntFormOptionsListview;
    function GetListviewColoring: TTntFormOptionsListviewColoring;
    function GetListviewColumns: TTntFormOptionsListviewColumns;
    function GetListviewGrouping: TTntFormOptionsListviewGrouping;
    function GetListviewThumbnail: TTntFormOptionsListviewThumbnails;
    function GetMouse: TTntFormOptionsMouse;
    function GetTabs: TTntFormOptionsTabs;
    function GetToolbar: TTntFormOptionsToolbar;
    function GetToolbarDrives: TTntFormOptionsToolbarDrives;
    function GetTreeview: TTntFormOptionsTreeview;
    function GetViewer: TTntFormOptionsViewer;
    procedure SetIsDirty(const Value: Boolean);
    { Private declarations }
  protected
    procedure CreateParams(var Params: TCreateParams); override;
    property WndParent: HWnd read FWndParent write FWndParent;
    function MapIndexToStroke(StrokeIndex: Integer): WideChar;
    function MapStrokeToIndex(Stroke: WideChar): Integer;
    function StrokesValidated: Boolean;
    procedure BuildTreeview;
  public
    { Public declarations }
    constructor CreateEx(AnOwner: TCustomForm);
    function IsColorsLocked: Boolean;
    function IsFontsLocked: Boolean;
    function QueryModifyColor(var Color: TColor): Boolean;
    function QueryModifyFont(Font: TFont): Boolean;
    procedure ApplyChanges;
    procedure SelectFrame(FrameID: Integer);
    procedure SetAllWindowColors(Color: TColor);
    procedure SetAllWindowFonts(Font: TFont);
    procedure SyncControlsWithAttribs;
    property Attribs: TUltraAttribs read FAttribs write FAttribs;
    property CmdLine: TTntFormOptionsCmdLine read GetCmdLine;
    property CurrentFrame: TTntOptionsBaseForm read FCurrentFrame write FCurrentFrame;
    property DropStack: TTntFormOptionsDropStack read GetDropStack;
    property Favorites: TTntFormOptionsFavorites read GetFavorites;
    property Filter: TTntFormOptionsFilter read GetFilter;
    property FolderSize: TTntFormOptionsFolderSizes read GetFolderSize;
    property General: TTntFormOptionsGeneral read GetGeneral;
    property History: TTntFormOptionsHistory read GetHistory;
    property InitialOptionsFrame: Integer read FInitialOptionsFrame write FInitialOptionsFrame;
    property IsDirty: Boolean read FIsDirty write SetIsDirty;
    property Listview: TTntFormOptionsListview read GetListview;
    property ListviewColoring: TTntFormOptionsListviewColoring read GetListviewColoring;
    property ListviewColumns: TTntFormOptionsListviewColumns read GetListviewColumns;
    property ListviewGrouping: TTntFormOptionsListviewGrouping read GetListviewGrouping;
    property ListviewThumbnail: TTntFormOptionsListviewThumbnails read GetListviewThumbnail;
    property LockTreeChange: Boolean read FLockTreeChange write FLockTreeChange;
    property Mouse: TTntFormOptionsMouse read GetMouse;
    property SpTBXDockHeader: TSpTBXDock read FSpTBXDockHeader write FSpTBXDockHeader;
    property SpTBXToolbar: TSpTBXToolbar read FSpTBXToolbar write FSpTBXToolbar;
    property Tabs: TTntFormOptionsTabs read GetTabs;
    property TempColor: TColor read FTempColor write FTempColor;
    property TempFont: TFont read FTempFont write FTempFont;
    property Toolbar: TTntFormOptionsToolbar read GetToolbar;
    property ToolbarDrives: TTntFormOptionsToolbarDrives read GetToolbarDrives;
    property Treeview: TTntFormOptionsTreeview read GetTreeview;
    property Viewer: TTntFormOptionsViewer read GetViewer;
  published
  end;

var
  OptionFrames: TOptionFrames;


implementation

uses SpTBXEditors;

{$R *.DFM}

constructor TTntFormOptionsFrame.CreateEx(AnOwner: TCustomForm);
begin
  if Assigned(AnOwner) then
     WndParent := AnOwner.Handle
   else
     WndParent := 0;
   inherited Create(nil);
end;

procedure TTntFormOptionsFrame.CreateParams(var Params: TCreateParams);
begin
  inherited;
  if WndParent <> 0 then
    Params.WndParent := WndParent;
end;

function TTntFormOptionsFrame.GetCmdLine: TTntFormOptionsCmdLine;
begin
  Result := TTntFormOptionsCmdLine( OptionFrames[ID_OPTION_CMDLINE]);
end;

function TTntFormOptionsFrame.GetDropStack: TTntFormOptionsDropStack;
begin
  Result := TTntFormOptionsDropStack( OptionFrames[ID_OPTION_DROPSTACK]);
end;

function TTntFormOptionsFrame.GetFavorites: TTntFormOptionsFavorites;
begin
  Result := TTntFormOptionsFavorites( OptionFrames[ID_OPTION_FAVORITES]);
end;

function TTntFormOptionsFrame.GetFilter: TTntFormOptionsFilter;
begin
  Result := TTntFormOptionsFilter( OptionFrames[ID_OPTION_FILTER]);
end;

function TTntFormOptionsFrame.GetFolderSize: TTntFormOptionsFolderSizes;
begin
  Result := TTntFormOptionsFolderSizes( OptionFrames[ID_OPTION_FOLDERSIZE]);
end;

function TTntFormOptionsFrame.GetGeneral: TTntFormOptionsGeneral;
begin
  Result := TTntFormOptionsGeneral( OptionFrames[ID_OPTION_GENERAL]);
end;

function TTntFormOptionsFrame.GetHistory: TTntFormOptionsHistory;
begin
  Result := TTntFormOptionsHistory( OptionFrames[ID_OPTION_HISTORY]);
end;

function TTntFormOptionsFrame.GetListview: TTntFormOptionsListview;
begin
  Result := TTntFormOptionsListview( OptionFrames[ID_OPTION_LISTVIEW])
end;

function TTntFormOptionsFrame.GetListviewColoring: TTntFormOptionsListviewColoring;
begin
  Result := TTntFormOptionsListviewColoring( OptionFrames[ID_OPTION_LISTVIEWCOLORING]);
end;

function TTntFormOptionsFrame.GetListviewColumns: TTntFormOptionsListviewColumns;
begin
  Result := TTntFormOptionsListviewColumns( OptionFrames[ID_OPTION_LISTVIEWCOLUMNS]);
end;

function TTntFormOptionsFrame.GetListviewGrouping: TTntFormOptionsListviewGrouping;
begin
  Result := TTntFormOptionsListviewGrouping( OptionFrames[ID_OPTION_LISTVIEWGROUPING]);
end;

function TTntFormOptionsFrame.GetListviewThumbnail: TTntFormOptionsListviewThumbnails;
begin
  Result :=  TTntFormOptionsListviewThumbnails( OptionFrames[ID_OPTION_LISTVIEWTHUMBNAILS]);
end;

function TTntFormOptionsFrame.GetMouse: TTntFormOptionsMouse;
begin
  Result := TTntFormOptionsMouse( OptionFrames[ID_OPTION_MOUSE]);
end;

function TTntFormOptionsFrame.GetTabs: TTntFormOptionsTabs;
begin
  Result := TTntFormOptionsTabs( OptionFrames[ID_OPTION_TABS]);
end;

function TTntFormOptionsFrame.GetToolbar: TTntFormOptionsToolbar;
begin
  Result := TTntFormOptionsToolbar( OptionFrames[ID_OPTION_TOOLBAR]);
end;

function TTntFormOptionsFrame.GetToolbarDrives: TTntFormOptionsToolbarDrives;
begin
  Result := TTntFormOptionsToolbarDrives( OptionFrames[ID_OPTION_TOOLBARDRIVES]);
end;

function TTntFormOptionsFrame.GetTreeview: TTntFormOptionsTreeview;
begin
  Result := TTntFormOptionsTreeview( OptionFrames[ID_OPTION_TREEVIEW]);
end;

function TTntFormOptionsFrame.GetViewer: TTntFormOptionsViewer;
begin
  Result := TTntFormOptionsViewer( OptionFrames[ID_OPTION_VIEWER]);
end;

function TTntFormOptionsFrame.IsColorsLocked: Boolean;
begin
  Result := General.SpTBXCheckBoxGeneralLockColors.Checked
end;

function TTntFormOptionsFrame.IsFontsLocked: Boolean;
begin
  Result := General.SpTBXCheckBoxGeneralLockFonts.Checked
end;

function TTntFormOptionsFrame.MapIndexToStroke(StrokeIndex: Integer): WideChar;
begin
  Result := 'U';
  case StrokeIndex of
    0: Result := 'L';
    1: Result := 'R';
    2: Result := 'U';
    3: Result := 'D';
  end
end;

function TTntFormOptionsFrame.MapStrokeToIndex(Stroke: WideChar): Integer;
begin
  Result := 0;
  case Stroke of
    'L': Result := 0;
    'R': Result := 1;
    'U': Result := 2;
    'D': Result := 3;
  end
end;

function TTntFormOptionsFrame.QueryModifyColor(var Color: TColor): Boolean;
begin
  Result := False;
  ColorDialog.Color := Color;
  if ColorDialog.Execute then
  begin
    Color := ColorDialog.Color;
    Result := True
  end
end;

function TTntFormOptionsFrame.QueryModifyFont(Font: TFont): Boolean;
begin
  Result := False;
  FontDialog.Font.Assign(Font);
  if FontDialog.Execute then
  begin
    Font.Assign(FontDialog.Font);
    Result := True
  end
end;

function TTntFormOptionsFrame.StrokesValidated: Boolean;
var
  F: TTntFormOkDialog;
begin
  Result := True;
  if ((Mouse.SpTBXComboBoxGestureBack.ItemIndex = Mouse.SpTBXComboBoxGestureUp.ItemIndex)) or
     ((Mouse.SpTBXComboBoxGestureBack.ItemIndex = Mouse.SpTBXComboBoxGestureForward.ItemIndex)) or
     ((Mouse.SpTBXComboBoxGestureBack.ItemIndex = Mouse.SpTBXComboBoxGestureNewTab.ItemIndex)) or
     ((Mouse.SpTBXComboBoxGestureUp.ItemIndex = Mouse.SpTBXComboBoxGestureBack.ItemIndex)) or
     ((Mouse.SpTBXComboBoxGestureUp.ItemIndex = Mouse.SpTBXComboBoxGestureForward.ItemIndex)) or
     ((Mouse.SpTBXComboBoxGestureUp.ItemIndex = Mouse.SpTBXComboBoxGestureNewTab.ItemIndex)) or
     ((Mouse.SpTBXComboBoxGestureForward.ItemIndex = Mouse.SpTBXComboBoxGestureUp.ItemIndex)) or
     ((Mouse.SpTBXComboBoxGestureForward.ItemIndex = Mouse.SpTBXComboBoxGestureBack.ItemIndex)) or
     ((Mouse.SpTBXComboBoxGestureForward.ItemIndex = Mouse.SpTBXComboBoxGestureNewTab.ItemIndex)) or
     ((Mouse.SpTBXComboBoxGestureNewTab.ItemIndex = Mouse.SpTBXComboBoxGestureUp.ItemIndex)) or
     ((Mouse.SpTBXComboBoxGestureNewTab.ItemIndex = Mouse.SpTBXComboBoxGestureForward.ItemIndex)) or
     ((Mouse.SpTBXComboBoxGestureNewTab.ItemIndex = Mouse.SpTBXComboBoxGestureBack.ItemIndex)) then
  begin
    F := TTntFormOkDialog.CreateEx(nil);
    try
      F.SpTBXLabel.Caption := STR_DUPLICATE_GESTURE;
      F.ShowModal;
    finally
      Result := False;
      F.Release
    end
  end
end;

procedure TTntFormOptionsFrame.BuildTreeview;
var
  Item: TTntTreeNode;
  Root: TTntTreeNode;
begin
  TntTreeView.Items.BeginUpdate;
  try
    LockTreeChange := True;
    TntTreeView.Items.Clear;
    LoadOptionsTreeviewStrings;
    // Treeview
    Root := TntTreeView.Items.Add(nil, FrameCaptions[ID_OPTION_TREEVIEW]);
    Root.Data := Pointer(ID_OPTION_TREEVIEW);
    Root.ImageIndex := 0;
    Root.SelectedIndex := 0;
    // Listview
    Root := TntTreeView.Items.Add(nil, FrameCaptions[ID_OPTION_LISTVIEW]);
    Root.Data := Pointer(ID_OPTION_LISTVIEW);
    Root.ImageIndex := 1;
    Root.SelectedIndex := 1;
    Item := TntTreeView.Items.AddChild(Root, FrameCaptions[ID_OPTION_LISTVIEWCOLORING]);
    Item.Data := Pointer(ID_OPTION_LISTVIEWCOLORING);
    Item.ImageIndex := 17;
    Item.SelectedIndex := 17;
    Item := TntTreeView.Items.AddChild(Root, FrameCaptions[ID_OPTION_LISTVIEWCOLUMNS]);
    Item.Data := Pointer(ID_OPTION_LISTVIEWCOLUMNS);
    Item.ImageIndex := 12;
    Item.SelectedIndex := 12;
    Item := TntTreeView.Items.AddChild(Root, FrameCaptions[ID_OPTION_LISTVIEWGROUPING]);
    Item.Data := Pointer(ID_OPTION_LISTVIEWGROUPING);
    Item.ImageIndex := 13;
    Item.SelectedIndex := 13;
    Item := TntTreeView.Items.AddChild(Root, FrameCaptions[ID_OPTION_LISTVIEWTHUMBNAILS]);
    Item.Data := Pointer(ID_OPTION_LISTVIEWTHUMBNAILS);
    Item.ImageIndex := 14;
    Item.SelectedIndex := 14;
    // History
    Root := TntTreeView.Items.Add(nil, FrameCaptions[ID_OPTION_HISTORY]);
    Root.Data := Pointer(ID_OPTION_HISTORY);
    Root.ImageIndex := 2;
    Root.SelectedIndex := 2;
    // Favorites
    Root := TntTreeView.Items.Add(nil, FrameCaptions[ID_OPTION_FAVORITES]);
    Root.Data := Pointer(ID_OPTION_FAVORITES);
    Root.ImageIndex := 3;
    Root.SelectedIndex := 3;
    // Filter
    Root := TntTreeView.Items.Add(nil, FrameCaptions[ID_OPTION_FILTER]);
    Root.Data := Pointer(ID_OPTION_FILTER);
    Root.ImageIndex := 4;
    Root.SelectedIndex := 4;
    // Command Line
    Root := TntTreeView.Items.Add(nil, FrameCaptions[ID_OPTION_CMDLINE]);
    Root.Data := Pointer(ID_OPTION_CMDLINE);
    Root.ImageIndex := 5;
    Root.SelectedIndex := 5;
    // Dropstack
    Root := TntTreeView.Items.Add(nil, FrameCaptions[ID_OPTION_DROPSTACK]);
    Root.Data := Pointer(ID_OPTION_DROPSTACK);
    Root.ImageIndex := 6;
    Root.SelectedIndex := 6;
    // Viewer
    Root := TntTreeView.Items.Add(nil, FrameCaptions[ID_OPTION_VIEWER]);
    Root.Data := Pointer(ID_OPTION_VIEWER);
    Root.ImageIndex := 7;
    Root.SelectedIndex := 7;
    // Toolbar
    Root := TntTreeView.Items.Add(nil, FrameCaptions[ID_OPTION_TOOLBAR]);
    Root.Data := Pointer(ID_OPTION_TOOLBAR);
    Root.ImageIndex := 8;
    Root.SelectedIndex := 8;
    Item := TntTreeView.Items.AddChild(Root, FrameCaptions[ID_OPTION_TOOLBARDRIVES]);
    Item.Data := Pointer(ID_OPTION_TOOLBARDRIVES);
    Item.ImageIndex := 15;
    Item.SelectedIndex := 15;
    // Folder Sizes
    Root := TntTreeView.Items.Add(nil, FrameCaptions[ID_OPTION_FOLDERSIZE]);
    Root.Data := Pointer(ID_OPTION_FOLDERSIZE);
    Root.ImageIndex := 9;
    Root.SelectedIndex := 9;
    // Tabs
    Root := TntTreeView.Items.Add(nil, FrameCaptions[ID_OPTION_TABS]);
    Root.Data := Pointer(ID_OPTION_TABS);
    Root.ImageIndex := 10;
    Root.SelectedIndex := 10;
    // Mouse
    Root := TntTreeView.Items.Add(nil, FrameCaptions[ID_OPTION_MOUSE]);
    Root.Data := Pointer(ID_OPTION_MOUSE);
    Root.ImageIndex := 16;
    Root.SelectedIndex := 16;
    // General
    Root := TntTreeView.Items.Add(nil, FrameCaptions[ID_OPTION_GENERAL]);
    Root.Data := Pointer(ID_OPTION_GENERAL);
    Root.ImageIndex := 11;
    Root.SelectedIndex := 11;
    TntTreeView.FullExpand;
  finally
    TntTreeView.Items.EndUpdate;
    LockTreeChange := False;
  end
end;

procedure TTntFormOptionsFrame.SelectFrame(FrameID: Integer);

  function RunFrames(Item: TTntTreeNode): Boolean;
  var
    Sibling: TTntTreeNode;
  begin
    Result := False;
    if Assigned(Item) then
    begin
      if Item.getFirstChild <> nil then
        RunFrames(Item.getFirstChild);
      if Integer( Item.Data) = FrameID then
      begin
        Item.Selected := True;
        Result := True
      end;

      Sibling := Item.getNextSibling;
      while assigned(Sibling) and not Result do
      begin
        if Sibling.getFirstChild <> nil then
          RunFrames(Sibling.getFirstChild);
        if Integer( Sibling.Data) = FrameID then
        begin
          Sibling.Selected := True;
          Result := True
        end;
        Sibling := Sibling.getNextSibling
      end
    end
  end;

begin
  if (FrameID >= 0) and (FrameID <= ID_OPTION_LAST) then
    RunFrames(TntTreeView.Items.GetFirstNode)
end;

procedure TTntFormOptionsFrame.SetAllWindowColors(Color: TColor);
begin
  Treeview.SpTBXEditTreeviewSample.Color := Color;
  Listview.SpTBXEditListviewSample.Color := Color;
  Filter.SpTBXEditFilterSample.Color := Color;
  DropStack.SpTBXEditDropStackSample.Color := Color;
  History.SpTBXEditHistorySample.Color := Color;
  Favorites.SpTBXEditFavoritesSample.Color := Color;
end;

procedure TTntFormOptionsFrame.SetAllWindowFonts(Font: TFont);
begin
  Treeview.SpTBXEditTreeviewSample.Font.Assign(Font);
  Listview.SpTBXEditListviewSample.Font.Assign(Font);
  Filter.SpTBXEditFilterSample.Font.Assign(Font);
  DropStack.SpTBXEditDropStackSample.Font.Assign(Font);
  History.SpTBXEditHistorySample.Font.Assign(Font);
  Favorites.SpTBXEditFavoritesSample.Font.Assign(Font);
end;

procedure TTntFormOptionsFrame.SyncControlsWithAttribs;
var
  i: Integer;
  ExtColorItem: TExtColorCodeItem;
begin
  SpTBXCheckBoxShowPopupHints.Checked := Attribs.ShowPopupHints;

  // DropStack Frame
  DropStack.SpTBXEditDropStackSample.Color := Attribs.DropStack.Color;
  DropStack.SpTBXEditDropStackSample.Font.Assign(Attribs.DropStack.Font);
  DropStack.SpTBXEditDropStackDepth.Text := IntToStr(Attribs.DropStack.Depth);
  DropStack.SpTBXEditDropStackHintFiles.Text := IntToStr(Attribs.DropStack.HintCount);
  DropStack.SpTBXRadioGroupDropStackAutoRemove.ItemIndex := Integer(Attribs.DropStack.AutoRemove);
  DropStack.SpTBXCheckBoxDropStackContentsHint.Checked := Attribs.DropStack.ContentsHint;

  // CmdLine Frame
  CmdLine.SpTBXEditCmdLineSample.Color := Attribs.CmdLine.Color;
  CmdLine.SpTBXEditCmdLineSample.Font.Assign(Attribs.CmdLine.Font);
  CmdLine.SpTBXEditOptionsCmdLineDepth.Text := IntToStr(Attribs.CmdLine.MaxLineCount);
  CmdLine.SpTBXCheckBoxCmdLineSync.Checked := Attribs.CmdLine.SyncWithFolder;

  // History Frame
  History.SpTBXEditHistorySample.Color := Attribs.History.Color;
  History.SpTBXEditHistorySample.Font.Assign(Attribs.History.Font);
  History.SpTBXCheckBoxHistorySaveRemovableDrives.Checked := Attribs.History.SaveRemovableDrivePaths;
  History.SpTBXCheckBoxHistorySaveNetworkPaths.Checked := Attribs.History.SaveNetworkPaths;
  History.SpTBXCheckBoxHistorySaveMappedPaths.Checked := Attribs.History.SaveMappedPaths;
  History.SpTBXCheckBoxOptionsHistoryContextWarning.Checked := Attribs.History.WarnBeforeContextAction;
  History.SpTBXCheckBoxOptionsHistoryDropWarning.Checked := Attribs.History.WarnBeforeDropAction;
  History.SpTBXEditHistoryStackDepth.Text := IntToStr(Attribs.History.Levels);
  History.SpTBXCheckBoxSavePerTab.Checked := Attribs.History.SavePerTab;

  // Favorites Frame
  Favorites.SpTBXEditFavoritesSample.Color := Attribs.Favorites.Color;
  Favorites.SpTBXEditFavoritesSample.Font.Assign(Attribs.Favorites.Font);
  Favorites.SpTBXCheckBoxFavoritesShowPathHint.Checked := Attribs.Favorites.ShowPathHint;
  Favorites.SpTBXCheckBoxOptionsFavoritesDropWarning.Checked := Attribs.Favorites.WarnBeforeDropAction;
  Favorites.SpTBXCheckBoxOptionsFavoritesContextWarning.Checked := Attribs.Favorites.WarnBeforeContextAction;
  Favorites.SpTBXCheckBoxFavoritesFullRowSelect.Checked := Attribs.Favorites.FullRowSelect;
  Favorites.SpTBXCheckBoxFavoritesOpenInNewTab.Checked := Attribs.Favorites.OpenFoldersInNewTab;

  // Filter Frame
  Filter.SpTBXEditFilterSample.Color := Attribs.Filter.Color;
  Filter.SpTBXEditFilterSample.Font.Assign(Attribs.Filter.Font);
  Filter.SpTBXCheckBoxQuickFilterClearOnBrowse.Checked := Attribs.Filter.QuickFilterClearOnBrowse;
  Filter.SpTBXCheckBoxQuickFilterSavePerTab.Checked := Attribs.Filter.QuickFilterSavePerTab;

  // Tabs Frame
  Tabs.CustomNewTabLocation := TNamespace.Create(PIDLMgr.CopyPIDL(Attribs.General.DefaultTabPIDL), nil);
  Tabs.CustomStartupPath := TNamespace.Create(PIDLMgr.CopyPIDL(Attribs.General.DefaultStartupPath), nil);
  Tabs.SpTBXButtonEditDefaultNewTabLocation.Text := Tabs.CustomNewTabLocation.NameAddressBar;
  Tabs.SpTBXButtonEditStartupCustomFolder.Text := Tabs.CustomStartupPath.NameAddressBar;
  Tabs.SpTBXRadioGroupDefaultNewTab.ItemIndex := Integer(Attribs.General.DefaultTab);
  Tabs.SpTBXRadioGroupGeneralStartup.ItemIndex := Integer(Attribs.General.DefaultPath);
  Tabs.SpTBXCheckBoxHideTabsetWithOne.Checked := Attribs.Listview.HideTabsetWhenOneTab;
  Tabs.SpTBXCheckBoxRemoveableDrivesTab.Checked := Attribs.General.SaveRemovableDriveTabs;
  Tabs.SpTBXCheckBoxNetworkDrivesTab.Checked := Attribs.General.SaveNetworkPathTabs;
  Tabs.SpTBXCheckBoxGeneralTabsShowHints.Checked := Attribs.General.ShowTabPathHints;
  Tabs.SpTBXCheckBoxGeneralSaveMappedDrives.Checked := Attribs.General.SaveMappedDriveTabs;
  Tabs.SpTBXCheckBoxTabsetShowFullPaths.Checked := Attribs.General.FullPathInTabMenus;
  Tabs.SpTBXRadioGroupTabSetPosition.ItemIndex := Integer(Attribs.Tabs.Position);
  Tabs.SpTBXSpinEditTabWidth.Value := Attribs.Tabs.MaxWidth;
  Tabs.SpTBXCheckBoxTabWidth.Checked := Attribs.Tabs.MaxWidth > 0;

  // General Frame
  General.SpTBXCheckBoxSingleInstance.Checked := Attribs.General.SingleInstance;
  General.SpTBXCheckBoxPathInCaption.Checked := Attribs.General.PathInCaption;
  General.SpTBXCheckBoxShowHints.Checked := Attribs.General.ShowHints;
  General.SpTBXCheckBoxFullPathInAddress.Checked := Attribs.General.ShowFullPathInAddressbar;
  General.SpTBXCheckBoxGeneralTrackFileSize.Checked := Attribs.General.TrackSelectedSize;
  General.SpTBXCheckBoxGeneralUseOnByDefaultFlag.Checked := Attribs.General.UseOnByDefaultFlag;
  General.SpTBXCheckBoxGeneralAutoCheckForUpdates.Checked := Attribs.General.AutoCheckForUpdates;
  General.SpTBXCheckBoxGeneralLockFonts.Checked := Attribs.General.LockWindowFonts;
  General.SpTBXCheckBoxGeneralLockColors.Checked := Attribs.General.LockWindowColors;
  General.SpTBXCheckBoxGeneralTray.Checked := Attribs.General.TrayIcon;
  General.SpTBXCheckBoxSuppressNoDiskMessages.Checked := Attribs.General.SuppressNoDiskMessages;
  General.SpTBXCheckBoxFolders.Checked := foFolders in Attribs.Listview.FileObjects;
  General.SpTBXCheckBoxNonFolders.Checked := foNonFolders in Attribs.Listview.FileObjects;
  General.SpTBXCheckBoxHidden.Checked := foHidden in Attribs.Listview.FileObjects;
  General.SpTBXSpinEditIncrementalDelayTime.SpinOptions.ValueAsInteger := Attribs.General.IncrementalDelayTime;

  // Toolbars Frame
  Toolbar.SpTBXRadioGroupToolbarBreadCrumb.ItemIndex := Attribs.Toolbars.BreadCrumbWrap;
  Toolbar.SpTBXRadioGroupToolbarsFavoriteCrumb.ItemIndex := Attribs.Toolbars.FavoriteCrumbWrap;
  Toolbar.SpTBXCheckBoxGeneralShowCloseButtons.Checked := Attribs.General.ShowCloseButtons; // Moved to Toolbars Tab
  Toolbar.SpTBXCheckBoxToolbarsLock.Checked := Attribs.Toolbars.Lock;
  Toolbar.SpTBXEditAddressComboboxWidth.Text := IntToStr(Attribs.Toolbars.AddressBarWidth);
  Toolbar.SpTBXCheckBoxAutoOpenBreadcrumb.Checked := Attribs.Toolbars.AutoOpenBreadcrumb;
  Toolbar.SpTBXEditBreadCrumbHoverTime.Text := IntToStr( Attribs.Toolbars.BreadCrumbHoverTime);
  Toolbar.SpTBXCheckBoxShowBrowseable.Checked := Attribs.Toolbars.ShowBrowseableInBreadcrumbMenus;

  // Toolbars-Drives Frame
  ToolbarDrives.SpTBXRadioGroupToolbarsButtonLayout.ItemIndex := Integer( Attribs.Toolbars.ButtonLayout);
  ToolbarDrives.SpTBXCheckBoxToolbarsDrivesShowNetwork.Checked := Attribs.Toolbars.DriveShowNetwork;
  ToolbarDrives.SpTBXCheckBoxToolbarsDrivesShowControlPanel.Checked := Attribs.Toolbars.DriveShowControlPanel;
  ToolbarDrives.SpTBXCheckBoxToolbarsDrivesShowRecycleBin.Checked := Attribs.Toolbars.DriveShowRecycleBin;
  ToolbarDrives.SpTBXCheckBoxToolbarsDrivesShowMyComputer.Checked := Attribs.Toolbars.DriveShowMyComputer;
  ToolbarDrives.SpTBXCheckBoxToolbarsDrivesShowMyDocuments.Checked := Attribs.Toolbars.DriveShowMyDocuments;
  ToolbarDrives.SpTBXCheckBoxToolbarDrivesShowHints.Checked := Attribs.Toolbars.DriveShowHints;
  ToolbarDrives.SpTBXCheckBoxToolbarsDrivesShowDesktop.Checked := Attribs.Toolbars.DriveShowDesktop;

  // Viewer Frame
  Viewer.SpTBXEditViewerTextExt.Text := Attribs.Viewer.TextExtensions.Text;
  Viewer.SpTBXEditViewerImageExt.Text := Attribs.Viewer.ImageExtensions.Text;
  Viewer.SpTBXEditViewerMediaExt.Text := Attribs.Viewer.MediaExtensions.Text;
  Viewer.SpTBXEditViewerInternetExt.Text := Attribs.Viewer.InternetExtensions.Text;
  Viewer.SpTBXEditViewerRTFExt.Text :=  Attribs.Viewer.RTFExtensions.Text;
  Viewer.SpTBXPanelTextWindowBkgnd.Color := Attribs.Viewer.TextBkGndColor;
  Viewer.SpTBXPanelTextWindowHex1.Color := Attribs.Viewer.Hex1Color;
  Viewer.SpTBXPanelTextWindowHex2.Color := Attribs.Viewer.Hex2Color;
  Viewer.SpTBXPanelTextWindowHexBkGnd.Color := Attribs.Viewer.HexBkGndColor;
  Viewer.OEMFont.Assign(Attribs.Viewer.OEMFont);
  Viewer.AnsiFont.Assign(Attribs.Viewer.Font);
  Viewer.SpTBXCheckBoxAutoPlayOnLoad.Checked := Attribs.Viewer.AutoPlayOnLoad;

  // Listview Frame     
  Listview.SpTBXCheckBoxShowTitleBar.Checked := Attribs.Listview.ShowTitleBar;
  Listview.SpTBXCheckBoxSelectFirst.Checked := Attribs.Listview.SelectFirstItem;
  Listview.SpTBXCheckBoxCopyPrevFolder.Checked := Attribs.Listview.CopyPrevView;
  Listview.SpTBXCheckBoxPerFolderSave.Checked := Attribs.Listview.SavePerFolder;
  Listview.SpTBXCheckBoxQueryInfoHints.Checked := Attribs.Listview.QueryInfoHints;
  Listview.SpTBXComboBoxView.ItemIndex := Attribs.Listview.DefaultView;
  Listview.SpTBXEditListviewSample.Color := Attribs.Listview.Color;
  Listview.SpTBXEditListviewSample.Font.Assign(Attribs.Listview.Font);
  Listview.SpTBXCheckBoxSelectionFocusRect.Checked := Attribs.Listview.FocusRect;
  Listview.SpTBXCheckBoxSelectionGradient.Checked := Attribs.Listview.GradientSelection;
  Listview.SpTBXCheckBoxSelectionFullRow.Checked := Attribs.Listview.FullRowSelect;
  Listview.SpTBXCheckBoxListviewHilightCompressed.Checked := Attribs.Listview.HilightCompressed;
  Listview.SpTBXCheckBoxHilightEncryptedFiles.Checked := Attribs.Listview.HilightEncrypted;
  Listview.SpTBXCheckBoxListviewAlwaysShowHeader.Checked := Attribs.Listview.AlwaysShowHeader;
  Listview.SpTBXCheckBoxListviewGridLines.Checked := Attribs.Listview.GridLines;
  Listview.SpTBXCheckBoxDblClickBkBrowse.Checked := Attribs.Listview.UpLevelOnDblClk;
  Listview.SpTBXCheckBoxAutoSizeItemCaptions.Checked := Attribs.Listview.AutoSizeCells;
  Listview.SpTBXSpinEditPopupHintTimeout.SpinOptions.ValueAsInteger := Attribs.Listview.QueryInfoHintTimout;
  Listview.SpTBXCheckBoxListviewHotTrack.Checked := Attribs.Listview.HotTrack;

  // Listview Columns Frame
  ListviewColumns.SpTBXComboBoxDefaultColumn.ItemIndex := Attribs.Listview.DefaultColumn;
  ListviewColumns.SpTBXCheckBoxListviewHighlightColumn.Checked := Attribs.Listview.HilightColumn;
  ListviewColumns.SpTBXPanelColumnColor.Color := Attribs.Listview.HilightColumnColor;
  ListviewColumns.SpTBXEditColumnInHeaderMenu.Text := IntToStr(Attribs.Listview.ColumnsShownInHeaderMenu);
  ListviewColumns.SpTBXSpinEditListviewSmoothScroll.Value := Attribs.Listview.SmoothScroll;
  ListviewColumns.SpTBXRadioColumnSortDirection.ItemIndex := Integer(Attribs.Listview.SortDirection) - 1;
  ListviewColumns.SpTBXEditDefaultColumnWidth.Text := IntToStr(Attribs.Listview.DefaultColumnModeWidth);
  ListviewColumns.SpTBXCheckBoxFoldersFirst.Checked := Attribs.Listview.FoldersAlwaysFirst;
  ListviewColumns.SpTBXCheckBoxHilightBanding.Checked := Attribs.Listview.ColumnHilightBanding;
  ListviewColumns.SpTBXPanelHilightBandColor.Color := Attribs.Listview.ColumnHilightBandColor;
  ListviewColumns.SpTBXRadioGroupFileSizeStyle.ItemIndex := Attribs.Listview.FileSizeStyle;

  // Listview Thumbnails Frame
  ListviewThumbnail.SpTBXCheckBoxListviewThumbsLoadAll.Checked := Attribs.Listview.ThumbsLoadAll;
  ListviewThumbnail.SpTBXCheckBoxCacheSingleFile.Checked := Attribs.Listview.ThumbsSingleFile;
  ListviewThumbnail.SpTBXSpinEditListviewThumbsWidth.Value := Attribs.Listview.ThumbsWidth;
  ListviewThumbnail.SpTBXSpinEditListviewThumbsHeight.Value := Attribs.Listview.ThumbsHeight;
  ListviewThumbnail.SpTBXButtonEditListviewThumbsCacheFile.Text := Attribs.Listview.ThumbsCacheFile;
  ListviewThumbnail.SpTBXCheckBoxThumbsCache.Checked := Attribs.Listview.ThumbsCache;
  ListviewThumbnail.SpTBXCheckBoxListviewThumbsCompress.Checked := Attribs.Listview.ThumbsCompress;
  ListviewThumbnail.SpTBXCheckBoxHideCaption.Checked := Attribs.Listview.HideCaptions;
  ListviewThumbnail.SpTBXEditAspectRatio.Text := FloatToStrF(Attribs.Listview.AspectRatio, ffFixed, 2, 4);
  ListviewThumbnail.SpTBXCheckBoxAutoOrient.Checked := Attribs.Listview.AutoOrientation;


  // Listview Grouping Frame
  ListviewGrouping.SpTBXCheckBoxGroupedByDefault.Checked := Attribs.Listview.GroupedByDefault;
  ListviewGrouping.SpTBXComboBoxDefaultGrouping.ItemIndex := Attribs.Listview.DefaultGroup;


  // Listview Coloring Frame
  ListviewColoring.EasyListviewExtColorGrouping.Items.Clear;
  for i := 0 to Attribs.Listview.ExtensionColorCodeList.Count - 1 do
  begin
    ExtColorItem := ListviewColoring.EasyListviewExtColorGrouping.Items.AddCustom(TExtColorCodeItem) as TExtColorCodeItem;
    ExtColorItem.LoadItems(Attribs.Listview.ExtensionColorCodeList[i].ExtensionMask,
                           Attribs.Listview.ExtensionColorCodeList[i].Color,
                           Attribs.Listview.ExtensionColorCodeList[i].Bold,
                           Attribs.Listview.ExtensionColorCodeList[i].Italic,
                           Attribs.Listview.ExtensionColorCodeList[i].UnderLine,
                           Attribs.Listview.ExtensionColorCodeList[i].Enabled)
  end;
  ListviewColoring.SpTBXCheckBoxColorGroupingEnable.Checked := Attribs.Listview.ExtensionColorCode;

  // Folder Size Frame
  FolderSize.SpTBXRadioGroupFolderSizeDisplayLayout.ItemIndex := Integer( Attribs.Listview.FolderSizeDisplay);
  FolderSize.SpTBXCheckBoxFolderSizeShowColumn.Checked := Attribs.Listview.FolderSizeShow;

  // Treeview
  Treeview.SpTBXEditTreeviewSample.Color := Attribs.Treeview.Color;
  Treeview.SpTBXEditTreeviewSample.Font.Assign(Attribs.Treeview.Font);
  Treeview.SpTBXCheckBoxTreeviewHoverAutoExpand.Checked := Attribs.Treeview.HoverExpand;
  Treeview.SpTBXCheckBoxTreeviewAutoHorzScroll.Checked := Attribs.Treeview.AutoHorzScroll;
  Treeview.SpTBXCheckBoxTreeviewBranchLines.Checked := Attribs.Treeview.ShowBranchLines;
  Treeview.SpTBXCheckBoxTreeviewHotTrack.Checked := Attribs.Treeview.HotTrack;
  Treeview.SpTBXCheckBoxTreeviewFullRowSelect.Checked := Attribs.Treeview.FullRowSelect;
  Treeview.SpTBXCheckBoxTreeviewSingleClickExpand.Checked := Attribs.Treeview.SingleClickExpand;
  Treeview.SpTBXCheckBoxTreeviewShowToolTip.Checked := Attribs.Treeview.ShowToolTip;
  Treeview.SpTBXCheckBoxHilightCompressedFolders.Checked := Attribs.Treeview.HilightCompressed;
  Treeview.SpTBXCheckBoxHilightEncryptedFiles.Checked := Attribs.Treeview.HilightEncrypted;
  Treeview.SpTBXCheckBoxAlwaysCollapseFirst.Checked := Attribs.Treeview.AlwaysCollapseFirst;

  // Mouse
  Mouse.SpTBXRadioButtonDragSelect.Checked := Attribs.Mouse.RightButtonAssignment = 0;
  Mouse.SpTBXRadioButtonGesture.Checked := Attribs.Mouse.RightButtonAssignment = 1;
  Mouse.SpTBXComboBoxGestureUp.ItemIndex := MapStrokeToIndex(Attribs.Mouse.FolderUpStroke);
  Mouse.SpTBXComboBoxGestureBack.ItemIndex := MapStrokeToIndex(Attribs.Mouse.BackStroke);
  Mouse.SpTBXComboBoxGestureForward.ItemIndex := MapStrokeToIndex(Attribs.Mouse.ForwardStroke);
  Mouse.SpTBXComboBoxGestureNewTab.ItemIndex := MapStrokeToIndex(Attribs.Mouse.NewTabStroke);

  SpTBXCheckBoxShowPopupHints.Checked := Attribs.ShowPopupHints;
end;

procedure TTntFormOptionsFrame.TreeViewChange(Sender: TObject;
  Node: TTreeNode);
begin
  if not (LockTreeChange or (csLoading in ComponentState)) then
  begin
    SendMessage(Handle, WM_SETREDRAW, 0, 0);
    try
      if Assigned(CurrentFrame) then
      begin
        CurrentFrame.Visible := False;
        CurrentFrame.Parent := nil;
        CurrentFrame := nil;
      end;

      CurrentFrame := OptionFrames[ Integer( Node.Data)];

      if Assigned(CurrentFrame) then
      begin
        CurrentFrame.BorderStyle := bsNone;
        CurrentFrame.Align := alClient;
        CurrentFrame.Parent := PanelFrame;
        CurrentFrame.Visible := True;
      end
    finally
      SendMessage(Handle, WM_SETREDRAW, 1, 0);
      RedrawWindow(Handle, nil, 0, RDW_ERASE or RDW_INVALIDATE or RDW_ERASENOW or
          RDW_UPDATENOW or RDW_ALLCHILDREN or RDW_FRAME);
    end
  end
end;

procedure TTntFormOptionsFrame.TntFormShow(Sender: TObject);
var
  i: Integer;
begin
  FixFormFont(Font);
  if SpTBXButtonOk.Top + SpTBXButtonOk.Height > ClientHeight then
   ClientHeight := SpTBXButtonOk.Top + SpTBXButtonOk.Height + 8;
  for i := 0 to MAX_OPTIONFRAMES-1 do
  begin
    OptionFrames[i] := FRAME_FORMS[i].Create(Self);
    OptionFrames[i].Visible := False;
    OptionFrames[i].ColorDialog := ColorDialog;
    OptionFrames[i].FontDialog := FontDialog;
    OptionFrames[i].IsDirty := SetIsDirty;
    OptionFrames[i].LockAllColors := IsColorsLocked;
    OptionFrames[i].LockAllFonts:= IsFontsLocked;
    OptionFrames[i].QueryModifyColor := QueryModifyColor;
    OptionFrames[i].QueryModifyFont := QueryModifyFont;
    OptionFrames[i].SetAllWindowColors := SetAllWindowColors;
    OptionFrames[i].SetAllWindowFonts := SetAllWindowFonts;
    OptionFrames[i].TempColor := TempColor;
    OptionFrames[i].TempFont := TempFont;
    OptionFrames[i].TntOpenDialog := TntOpenDialog;
    OptionFrames[i].SpTBXDockHeader := SpTBXDockHeader;
    OptionFrames[i].SpTBXToolbar := SpTBXToolbar;
  end;

  BuildTreeview;

  if Assigned(Attribs.LoadCallback) then
    Attribs.LoadCallback(Attribs);
  SyncControlsWithAttribs;
  SelectFrame(InitialOptionsFrame);
  IsDirty := False;
end;

procedure TTntFormOptionsFrame.SpTBXButtonOkClick(Sender: TObject);
begin
  ApplyChanges;
  Close;
end;

procedure TTntFormOptionsFrame.SpTBXButtonCancelClick(Sender: TObject);
begin
  Close;
end;

procedure TTntFormOptionsFrame.SpTBXButtonApplyClick(Sender: TObject);
begin
  ApplyChanges
end;

procedure TTntFormOptionsFrame.SpTBXButtonHelpClick(Sender: TObject);
begin
  beep
end;

procedure TTntFormOptionsFrame.ApplyChanges;
var
  i: Integer;
  ExtColorItem: TExtColorCodeItem;
begin
  if IsDirty and Assigned(Attribs.SaveCallback) then
  begin
    Attribs.ShowPopupHints := SpTBXCheckBoxShowPopupHints.Checked;

    // General Frame
    // Important to have the LockWindowXXX attribs set before setting colors and fonts
    // in remaining controls.
    Attribs.General.SingleInstance := General.SpTBXCheckBoxSingleInstance.Checked;
    Attribs.General.PathInCaption := General.SpTBXCheckBoxPathInCaption.Checked;
    Attribs.General.ShowHints := General.SpTBXCheckBoxShowHints.Checked;
    Attribs.General.ShowFullPathInAddressbar := General.SpTBXCheckBoxFullPathInAddress.Checked;
    Attribs.General.TrackSelectedSize := General.SpTBXCheckBoxGeneralTrackFileSize.Checked;
    Attribs.General.UseOnByDefaultFlag := General.SpTBXCheckBoxGeneralUseOnByDefaultFlag.Checked;
    Attribs.General.AutoCheckForUpdates := General.SpTBXCheckBoxGeneralAutoCheckForUpdates.Checked;
    Attribs.General.LockWindowFonts := General.SpTBXCheckBoxGeneralLockFonts.Checked;
    Attribs.General.LockWindowColors := General.SpTBXCheckBoxGeneralLockColors.Checked;
    Attribs.General.TrayIcon := General.SpTBXCheckBoxGeneralTray.Checked;
    Attribs.General.SuppressNoDiskMessages := General.SpTBXCheckBoxSuppressNoDiskMessages.Checked;
    Attribs.Listview.FileObjects := [];
    if General.SpTBXCheckBoxFolders.Checked then
      Attribs.Listview.FileObjects := Attribs.Listview.FileObjects + [foFolders];
    if General.SpTBXCheckBoxNonFolders.Checked then
      Attribs.Listview.FileObjects := Attribs.Listview.FileObjects + [foNonFolders];
    if General.SpTBXCheckBoxHidden.Checked then
      Attribs.Listview.FileObjects := Attribs.Listview.FileObjects + [foHidden];
    Attribs.General.IncrementalDelayTime := General.SpTBXSpinEditIncrementalDelayTime.SpinOptions.ValueAsInteger;

    // DropStack Frame
    Attribs.DropStack.Color := DropStack.SpTBXEditDropStackSample.Color;
    Attribs.DropStack.Font.Assign(DropStack.SpTBXEditDropStackSample.Font);
    Attribs.DropStack.Depth := StrToInt(DropStack.SpTBXEditDropStackDepth.Text);
    Attribs.DropStack.HintCount := StrToInt(DropStack.SpTBXEditDropStackHintFiles.Text);
    Attribs.DropStack.AutoRemove := TVirtualDropStackAutoRemove(DropStack.SpTBXRadioGroupDropStackAutoRemove.ItemIndex);
    Attribs.DropStack.ContentsHint := DropStack.SpTBXCheckBoxDropStackContentsHint.Checked;

    // CmdLine Frame
    Attribs.CmdLine.Color := CmdLine.SpTBXEditCmdLineSample.Color;
    Attribs.CmdLine.Font.Assign(CmdLine.SpTBXEditCmdLineSample.Font);
    Attribs.CmdLine.MaxLineCount := StrToInt(CmdLine.SpTBXEditOptionsCmdLineDepth.Text);
    Attribs.CmdLine.SyncWithFolder := CmdLine.SpTBXCheckBoxCmdLineSync.Checked;

    // History Frame
    Attribs.History.Color := History.SpTBXEditHistorySample.Color;
    Attribs.History.Font.Assign(History.SpTBXEditHistorySample.Font);
    Attribs.History.SaveRemovableDrivePaths := History.SpTBXCheckBoxHistorySaveRemovableDrives.Checked;
    Attribs.History.SaveNetworkPaths := History.SpTBXCheckBoxHistorySaveNetworkPaths.Checked;
    Attribs.History.SaveMappedPaths := History.SpTBXCheckBoxHistorySaveMappedPaths.Checked;
    Attribs.History.WarnBeforeContextAction := History.SpTBXCheckBoxOptionsHistoryContextWarning.Checked;
    Attribs.History.WarnBeforeDropAction := History.SpTBXCheckBoxOptionsHistoryDropWarning.Checked;
    Attribs.History.Levels := StrToInt(History.SpTBXEditHistoryStackDepth.Text);
    Attribs.History.SavePerTab := History.SpTBXCheckBoxSavePerTab.Checked;

    // Favorites Frame
    Attribs.Favorites.Color := Favorites.SpTBXEditFavoritesSample.Color;
    Attribs.Favorites.Font.Assign(Favorites.SpTBXEditFavoritesSample.Font);
    Attribs.Favorites.ShowPathHint := Favorites.SpTBXCheckBoxFavoritesShowPathHint.Checked;
    Attribs.Favorites.WarnBeforeDropAction := Favorites.SpTBXCheckBoxOptionsFavoritesDropWarning.Checked;
    Attribs.Favorites.WarnBeforeContextAction := Favorites.SpTBXCheckBoxOptionsFavoritesContextWarning.Checked;
    Attribs.Favorites.FullRowSelect := Favorites.SpTBXCheckBoxFavoritesFullRowSelect.Checked;
    Attribs.Favorites.OpenFoldersInNewTab := Favorites.SpTBXCheckBoxFavoritesOpenInNewTab.Checked;

    // Tabs Filter
    Attribs.Filter.Color := Filter.SpTBXEditFilterSample.Color;
    Attribs.Filter.Font.Assign(Filter.SpTBXEditFilterSample.Font);

    Attribs.Filter.QuickFilterClearOnBrowse := Filter.SpTBXCheckBoxQuickFilterClearOnBrowse.Checked;
    Attribs.Filter.QuickFilterSavePerTab := Filter.SpTBXCheckBoxQuickFilterSavePerTab.Checked;

    PIDLMgr.FreePIDL(Attribs.General.DefaultTabPIDL);
    PIDLMgr.FreePIDL(Attribs.General.DefaultStartupPath);

    // Tabs Frame
    Attribs.General.DefaultTabPIDL := PIDLMgr.CopyPIDL(Tabs.CustomNewTabLocation.AbsolutePIDL);
    Attribs.General.DefaultStartupPath := PIDLMgr.CopyPIDL(Tabs.CustomStartupPath.AbsolutePIDL);
    Attribs.General.DefaultTab := TDefaultTabLocation( Tabs.SpTBXRadioGroupDefaultNewTab.ItemIndex);
    Attribs.General.DefaultPath := TDefaultStartupPath( Tabs.SpTBXRadioGroupGeneralStartup.ItemIndex);
    Attribs.Listview.HideTabsetWhenOneTab := Tabs.SpTBXCheckBoxHideTabsetWithOne.Checked;
    Attribs.General.SaveRemovableDriveTabs := Tabs.SpTBXCheckBoxRemoveableDrivesTab.Checked;
    Attribs.General.SaveNetworkPathTabs := Tabs.SpTBXCheckBoxNetworkDrivesTab.Checked;
    Attribs.General.ShowTabPathHints := Tabs.SpTBXCheckBoxGeneralTabsShowHints.Checked;
    Attribs.General.SaveMappedDriveTabs := Tabs.SpTBXCheckBoxGeneralSaveMappedDrives.Checked;
    Attribs.General.FullPathInTabMenus := Tabs.SpTBXCheckBoxTabsetShowFullPaths.Checked;
    Attribs.Tabs.Position := TUltraTabsetPosition( Tabs.SpTBXRadioGroupTabSetPosition.ItemIndex);
    if Tabs.SpTBXCheckBoxTabWidth.Checked then
      Attribs.Tabs.MaxWidth := Tabs.SpTBXSpinEditTabWidth.SpinOptions.ValueAsInteger
    else
      Attribs.Tabs.MaxWidth := -1;

    // Toolbars Frame
    Attribs.Toolbars.BreadCrumbWrap := Toolbar.SpTBXRadioGroupToolbarBreadCrumb.ItemIndex;
    Attribs.Toolbars.FavoriteCrumbWrap := Toolbar.SpTBXRadioGroupToolbarsFavoriteCrumb.ItemIndex;
    Attribs.General.ShowCloseButtons := Toolbar.SpTBXCheckBoxGeneralShowCloseButtons.Checked; // Moved to Toolbars Tab
    Attribs.Toolbars.Lock := Toolbar.SpTBXCheckBoxToolbarsLock.Checked;
    Attribs.Toolbars.AddressBarWidth := StrToInt( Toolbar.SpTBXEditAddressComboboxWidth.Text);
    Attribs.Toolbars.AutoOpenBreadcrumb := Toolbar.SpTBXCheckBoxAutoOpenBreadcrumb.Checked;
    Attribs.Toolbars.BreadCrumbHoverTime := StrToInt(Toolbar.SpTBXEditBreadCrumbHoverTime.Text);
    Attribs.Toolbars.ShowBrowseableInBreadcrumbMenus := Toolbar.SpTBXCheckBoxShowBrowseable.Checked;

    // Toolbars-Drives Frame
    Attribs.Toolbars.ButtonLayout := TUltraButtonLayout( ToolbarDrives.SpTBXRadioGroupToolbarsButtonLayout.ItemIndex);
    Attribs.Toolbars.DriveShowNetwork := ToolbarDrives.SpTBXCheckBoxToolbarsDrivesShowNetwork.Checked;
    Attribs.Toolbars.DriveShowControlPanel := ToolbarDrives.SpTBXCheckBoxToolbarsDrivesShowControlPanel.Checked;
    Attribs.Toolbars.DriveShowRecycleBin := ToolbarDrives.SpTBXCheckBoxToolbarsDrivesShowRecycleBin.Checked;
    Attribs.Toolbars.DriveShowMyComputer := ToolbarDrives.SpTBXCheckBoxToolbarsDrivesShowMyComputer.Checked;
    Attribs.Toolbars.DriveShowMyDocuments := ToolbarDrives.SpTBXCheckBoxToolbarsDrivesShowMyDocuments.Checked;
    Attribs.Toolbars.DriveShowHints := ToolbarDrives.SpTBXCheckBoxToolbarDrivesShowHints.Checked;
    Attribs.Toolbars.DriveShowDesktop := ToolbarDrives.SpTBXCheckBoxToolbarsDrivesShowDesktop.Checked;

    // Viewer Frame
    Attribs.Viewer.TextExtensions.Text := Viewer.SpTBXEditViewerTextExt.Text;
    Attribs.Viewer.ImageExtensions.Text := Viewer.SpTBXEditViewerImageExt.Text;
    Attribs.Viewer.MediaExtensions.Text := Viewer.SpTBXEditViewerMediaExt.Text;
    Attribs.Viewer.InternetExtensions.Text := Viewer.SpTBXEditViewerInternetExt.Text;
    Attribs.Viewer.RTFExtensions.Text := Viewer.SpTBXEditViewerRTFExt.Text;
    Attribs.Viewer.TextBkGndColor := Viewer.SpTBXPanelTextWindowBkgnd.Color;
    Attribs.Viewer.Hex1Color := Viewer.SpTBXPanelTextWindowHex1.Color;
    Attribs.Viewer.Hex2Color := Viewer.SpTBXPanelTextWindowHex2.Color;
    Attribs.Viewer.HexBkGndColor := Viewer.SpTBXPanelTextWindowHexBkGnd.Color;
    Attribs.Viewer.OEMFont.Assign(Viewer.OEMFont);
    Attribs.Viewer.Font.Assign(Viewer.AnsiFont);
    Attribs.Viewer.AutoPlayOnLoad := Viewer.SpTBXCheckBoxAutoPlayOnLoad.Checked;

    // Listview Frame
    Attribs.Listview.ShowTitleBar := Listview.SpTBXCheckBoxShowTitleBar.Checked;
    Attribs.Listview.SelectFirstItem := Listview.SpTBXCheckBoxSelectFirst.Checked;
    Attribs.Listview.CopyPrevView := Listview.SpTBXCheckBoxCopyPrevFolder.Checked;
    Attribs.Listview.SavePerFolder := Listview.SpTBXCheckBoxPerFolderSave.Checked;
    Attribs.Listview.QueryInfoHints := Listview.SpTBXCheckBoxQueryInfoHints.Checked;
    Attribs.Listview.DefaultView := Listview.SpTBXComboBoxView.ItemIndex;
    Attribs.Listview.Color := Listview.SpTBXEditListviewSample.Color;
    Attribs.Listview.Font.Assign(Listview.SpTBXEditListviewSample.Font);
    Attribs.Listview.FocusRect  := Listview.SpTBXCheckBoxSelectionFocusRect.Checked;
    Attribs.Listview.GradientSelection := Listview.SpTBXCheckBoxSelectionGradient.Checked;
    Attribs.Listview.FullRowSelect := Listview.SpTBXCheckBoxSelectionFullRow.Checked;
    Attribs.Listview.HilightCompressed := Listview.SpTBXCheckBoxListviewHilightCompressed.Checked;
    Attribs.Listview.HilightEncrypted := Listview.SpTBXCheckBoxHilightEncryptedFiles.Checked;
    Attribs.Listview.AlwaysShowHeader := Listview.SpTBXCheckBoxListviewAlwaysShowHeader.Checked;
    Attribs.Listview.GridLines := Listview.SpTBXCheckBoxListviewGridLines.Checked;
    Attribs.Listview.UpLevelOnDblClk := Listview.SpTBXCheckBoxDblClickBkBrowse.Checked;
    Attribs.Listview.AutoSizeCells := Listview.SpTBXCheckBoxAutoSizeItemCaptions.Checked;
    Attribs.Listview.QueryInfoHintTimout := Listview.SpTBXSpinEditPopupHintTimeout.SpinOptions.ValueAsInteger;
    Attribs.Listview.HotTrack := Listview.SpTBXCheckBoxListviewHotTrack.Checked;
    Attribs.Listview.FileSizeStyle := ListviewColumns.SpTBXRadioGroupFileSizeStyle.ItemIndex;

    // Listview Columns Frame
    Attribs.Listview.DefaultColumn := ListviewColumns.SpTBXComboBoxDefaultColumn.ItemIndex;
    Attribs.Listview.HilightColumn := ListviewColumns.SpTBXCheckBoxListviewHighlightColumn.Checked;
    Attribs.Listview.HilightColumnColor := ListviewColumns.SpTBXPanelColumnColor.Color;
    Attribs.Listview.ColumnsShownInHeaderMenu := StrToInt(ListviewColumns.SpTBXEditColumnInHeaderMenu.Text);
    Attribs.Listview.SmoothScroll := Round(ListviewColumns.SpTBXSpinEditListviewSmoothScroll.Value);
    if ListviewColumns.SpTBXRadioColumnSortDirection.ItemIndex = 0 then
      Attribs.Listview.SortDirection := esdAscending
    else
      Attribs.Listview.SortDirection := esdDescending;
    Attribs.Listview.DefaultColumnModeWidth := StrToInt(ListviewColumns.SpTBXEditDefaultColumnWidth.Text);
    Attribs.Listview.FoldersAlwaysFirst := ListviewColumns.SpTBXCheckBoxFoldersFirst.Checked;
    Attribs.Listview.ColumnHilightBanding  := ListviewColumns.SpTBXCheckBoxHilightBanding.Checked;
    Attribs.Listview.ColumnHilightBandColor := ListviewColumns.SpTBXPanelHilightBandColor.Color;

    // Listview Thumbnails Frame
    Attribs.Listview.ThumbsLoadAll := ListviewThumbnail.SpTBXCheckBoxListviewThumbsLoadAll.Checked;
    Attribs.Listview.ThumbsSingleFile := ListviewThumbnail.SpTBXCheckBoxCacheSingleFile.Checked;
    Attribs.Listview.ThumbsWidth := Round(ListviewThumbnail.SpTBXSpinEditListviewThumbsWidth.Value);
    Attribs.Listview.ThumbsHeight := Round(ListviewThumbnail.SpTBXSpinEditListviewThumbsHeight.Value);
    Attribs.Listview.ThumbsCacheFile := ListviewThumbnail.SpTBXButtonEditListviewThumbsCacheFile.Text;
    Attribs.Listview.ThumbsCache := ListviewThumbnail.SpTBXCheckBoxThumbsCache.Checked;
    Attribs.Listview.ThumbsCompress := ListviewThumbnail.SpTBXCheckBoxListviewThumbsCompress.Checked;
    Attribs.Listview.HideCaptions := ListviewThumbnail.SpTBXCheckBoxHideCaption.Checked;
    Attribs.Listview.AspectRatio := StrToFloat(ListviewThumbnail.SpTBXEditAspectRatio.Text);

    // Listview Grouping Frame
    Attribs.Listview.GroupedByDefault := ListviewGrouping.SpTBXCheckBoxGroupedByDefault.Checked;
    Attribs.Listview.DefaultGroup := ListviewGrouping.SpTBXComboBoxDefaultGrouping.ItemIndex;


    // Listview Coloring Frame
    Attribs.Listview.ExtensionColorCodeList.Clear;
    for i := 0 to ListviewColoring.EasyListviewExtColorGrouping.Items.Count - 1 do
    begin
      ExtColorItem := ListviewColoring.EasyListviewExtColorGrouping.Items[i] as TExtColorCodeItem;
      Attribs.Listview.ExtensionColorCodeList.Add(ExtColorItem.Caption, ExtColorItem.Color, ExtColorItem.Bold, ExtColorItem.Italic, ExtColorItem.UnderLine, ExtColorItem.Checked)
    end;
    Attribs.Listview.ExtensionColorCode := ListviewColoring.SpTBXCheckBoxColorGroupingEnable.Checked;

    // Folder Size Frame
    Attribs.Listview.FolderSizeDisplay := TBtyeSize( FolderSize.SpTBXRadioGroupFolderSizeDisplayLayout.ItemIndex);
    Attribs.Listview.FolderSizeShow := FolderSize.SpTBXCheckBoxFolderSizeShowColumn.Checked;

    // Treeview
    Attribs.Treeview.Color := Treeview.SpTBXEditTreeviewSample.Color;
    Attribs.Treeview.Font.Assign(Treeview.SpTBXEditTreeviewSample.Font);
    Attribs.Treeview.HoverExpand := Treeview.SpTBXCheckBoxTreeviewHoverAutoExpand.Checked;
    Attribs.Treeview.AutoHorzScroll := Treeview.SpTBXCheckBoxTreeviewAutoHorzScroll.Checked;
    Attribs.Treeview.ShowBranchLines := Treeview.SpTBXCheckBoxTreeviewBranchLines.Checked;
    Attribs.Treeview.HotTrack := Treeview.SpTBXCheckBoxTreeviewHotTrack.Checked;
    Attribs.Treeview.FullRowSelect := Treeview.SpTBXCheckBoxTreeviewFullRowSelect.Checked;
    Attribs.Treeview.SingleClickExpand := Treeview.SpTBXCheckBoxTreeviewSingleClickExpand.Checked;
    Attribs.Treeview.ShowToolTip := Treeview.SpTBXCheckBoxTreeviewShowToolTip.Checked;
    Attribs.Treeview.HilightCompressed := Treeview.SpTBXCheckBoxHilightCompressedFolders.Checked;
    Attribs.Treeview.HilightEncrypted := Treeview.SpTBXCheckBoxHilightEncryptedFiles.Checked;
    Attribs.Treeview.AlwaysCollapseFirst := Treeview.SpTBXCheckBoxAlwaysCollapseFirst.Checked;

    // Mouse
    if Mouse.SpTBXRadioButtonDragSelect.Checked then
      Attribs.Mouse.RightButtonAssignment := 0
    else
      Attribs.Mouse.RightButtonAssignment := 1;
    Attribs.Mouse.FolderUpStroke := MapIndexToStroke(Mouse.SpTBXComboBoxGestureUp.ItemIndex);
    Attribs.Mouse.BackStroke := MapIndexToStroke(Mouse.SpTBXComboBoxGestureBack.ItemIndex);
    Attribs.Mouse.ForwardStroke := MapIndexToStroke(Mouse.SpTBXComboBoxGestureForward.ItemIndex);
    Attribs.Mouse.NewTabStroke := MapIndexToStroke(Mouse.SpTBXComboBoxGestureNewTab.ItemIndex);

    if StrokesValidated then
    begin
      IsDirty := False;
      if Assigned(Attribs.SaveCallback) then
        Attribs.SaveCallback(Attribs);
      if Assigned(Attribs.ApplyCallback) then
        Attribs.ApplyCallback(Self);
    end;
    ShowHint := Attribs.General.ShowHints;
  end
end;

procedure TTntFormOptionsFrame.SetIsDirty(const Value: Boolean);
begin
  if not (csLoading in ComponentState) then
  begin
    FIsDirty := Value;
    ActionOptionsApply.Enabled := FIsDirty
  end
end;

procedure TTntFormOptionsFrame.TntFormCreate(Sender: TObject);
begin
  TempFont := TFont.Create;
  BorderStyle := bsSizeable;
end;

procedure TTntFormOptionsFrame.TntFormDestroy(Sender: TObject);
var
  i: Integer;
begin
  for i := 0 to MAX_OPTIONFRAMES-1 do
    OptionFrames[i] := nil;
  TempFont.Free;
end;

procedure TTntFormOptionsFrame.ActionOptionsOkExecute(Sender: TObject);
begin
  if StrokesValidated then
  begin
    ApplyChanges;
    Close;
  end
end;

procedure TTntFormOptionsFrame.ActionOptionsCancelExecute(Sender: TObject);
begin
  Close;
end;

procedure TTntFormOptionsFrame.ActionOptionsApplyExecute(Sender: TObject);
begin
  ApplyChanges
end;

procedure TTntFormOptionsFrame.ActionOptionsHelpExecute(Sender: TObject);
begin
  beep
end;

procedure TTntFormOptionsFrame.TntFormKeyDown(Sender: TObject;
  var Key: Word; Shift: TShiftState);
begin
  if Key = VK_ESCAPE then
    Close
end;

procedure TTntFormOptionsFrame.SpTBXCheckBoxShowPopupHintsClick(Sender: TObject);
begin
  ShowHint := SpTBXCheckBoxShowPopupHints.Checked;
  Attribs.ShowPopupHints := SpTBXCheckBoxShowPopupHints.Checked;
end;

procedure TTntFormOptionsFrame.DKLanguageController1LanguageChanged(
  Sender: TObject);
begin
  BuildTreeview
end;

procedure TTntFormOptionsFrame.TntTreeViewCollapsing(Sender: TObject;
  Node: TTreeNode; var AllowCollapse: Boolean);
begin
  AllowCollapse := False
end;

procedure TTntFormOptionsFrame.TntFormCloseQuery(Sender: TObject;
  var CanClose: Boolean);
begin
  LockTreeChange := True
end;

initialization



end.
