unit UnitOptionsListviewColoring;

// The contents of this file are subject to the Mozilla Public License
// Version 1.1 (the "License"); you maynot use this file except in compliance
// with the License. You may obtain a copy of the License at http://www.mozilla.org/MPL/
//
// Alternatively, you may redistribute this library, use and/or modify it under the terms of the
// GNU Lesser General Public License as published by the Free Software Foundation;
// either version 2.1 of the License, or (at your option) any later version.
// You may obtain a copy of the LGPL at http://www.gnu.org/copyleft/.
//
// Software distributed under the License is distributed on an "AS IS" basis,
// WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for the
// specific language governing rights and limitations under the License.

interface

uses
  Windows,
  Messages,
  SysUtils,
  Variants,
  Classes,
  Graphics,
  Controls,
  Forms,
  ActiveX,
  Dialogs,
  TntForms,
  SpTBXControls,
  StdCtrls,
  TntStdCtrls,
  SpTBXEditors,
  SpTBXDkPanels,
  DKLang,
  UltraOptions,
  UnitOptionsBase,
  MPCommonObjects,
  MPCommonUtilities,
  UltraUserStrings,
  EasyListview, TB2Item, SpTBXItem, Menus, ImgList;

const
  ID_EXAMPLETEXT_COLUMN = 5;

type
  TExtColorCodeItem = class(TEasyItemStored)
  private
    FBold: Boolean;
    FColor: TColor;
    FItalic: Boolean;
    FUnderLine: Boolean;
  public
    procedure LoadItems(AnExtension: WideString; AColor: TColor; IsBold: Boolean; IsItalic: Boolean; IsUnderLine: Boolean; IsEnabled: Boolean);
    property Bold: Boolean read FBold write FBold;
    property Color: TColor read FColor write FColor;
    property Italic: Boolean read FItalic write FItalic;
    property UnderLine: Boolean read FUnderLine write FUnderLine;
  end;

  TTntFormOptionsListviewColoring = class(TTntOptionsBaseForm)
    SpTBXPanelBkGnd: TSpTBXPanel;
    DKLanguageController1: TDKLanguageController;
    SpTBXGroupBoxExtColorGrouping: TSpTBXGroupBox;
    EasyListviewExtColorGrouping: TEasyListview;
    SpTBXButtonExtColorGroupingAdd: TSpTBXButton;
    SpTBXButtonExtColorGroupingDelete: TSpTBXButton;
    SpTBXPopupMenuColorGrouping: TSpTBXPopupMenu;
    SpTBXItemColorGroupingSort: TSpTBXItem;
    SpTBXItemColorGroupingAdd: TSpTBXItem;
    SpTBXItemColorGroupingDelete: TSpTBXItem;
    SpTBXSeparatorItemColorListviewMenu2: TSpTBXSeparatorItem;
    SpTBXSeparatorItemColorListviewMenu0: TSpTBXSeparatorItem;
    SpTBXItemColorGroupingEdit: TSpTBXItem;
    ImageListGrouping: TImageList;
    SpTBXItemColorGroupingUnCheckall: TSpTBXItem;
    SpTBXItemColorGroupingCheckall: TSpTBXItem;
    SpTBXSeparatorItemColorListviewMenu1: TSpTBXSeparatorItem;
    SpTBXCheckBoxColorGroupingEnable: TSpTBXCheckBox;
    SpTBXButtonExtColorGroupingEdit: TSpTBXButton;
    procedure IsDirtyClick(Sender: TObject);
    procedure SpTBXButtonEditGroupingCriteriaClick(Sender: TObject);
    procedure FormShow(Sender: TObject);
    procedure EasyListviewExtColorGroupingOLEDragStart(Sender: TCustomEasyListview; ADataObject: IDataObject; var AvailableEffects: TCommonDropEffects; var AllowDrag: Boolean);
    procedure SpTBXButtonExtColorGroupingAddClick(Sender: TObject);
    procedure SpTBXButtonExtColorGroupingDeleteClick(Sender: TObject);
    procedure EasyListviewExtColorGroupingItemDblClick(Sender: TCustomEasyListview; Button: TCommonMouseButton; MousePos: TPoint; HitInfo: TEasyHitInfoItem);
    procedure EasyListviewExtColorGroupingItemPaintText(Sender: TCustomEasyListview; Item: TEasyItem; Position: Integer; ACanvas: TCanvas);
    procedure EasyListviewExtColorGroupingItemCheckChange(Sender: TCustomEasyListview; Item: TEasyItem);
    procedure SpTBXItemColorGroupingAddClick(Sender: TObject);
    procedure SpTBXItemColorGroupingDeleteClick(Sender: TObject);
    procedure SpTBXItemColorGroupingSortClick(Sender: TObject);
    procedure SpTBXItemColorGroupingEditClick(Sender: TObject);
    procedure SpTBXItemColorGroupingCheckallClick(Sender: TObject);
    procedure SpTBXItemColorGroupingUnCheckallClick(Sender: TObject);
    procedure EasyListviewExtColorGroupingKeyAction(
      Sender: TCustomEasyListview; var CharCode: Word;
      var Shift: TShiftState; var DoDefault: Boolean);
    procedure SpTBXButtonExtColorGroupingEditClick(Sender: TObject);
    procedure EasyListviewExtColorGroupingItemSelectionsChanged(
      Sender: TCustomEasyListview);
  private
    procedure ColorGroupingAdd;
    procedure ColorGroupingDelete;
    procedure ColorGroupingEdit(Item: TExtColorCodeItem);
  public
  end;

implementation

uses
  UnitOptionsListviewGroupingColorEditor;

{$R *.DFM}

procedure TTntFormOptionsListviewColoring.ColorGroupingAdd;
var
  Item: TEasyItem;
begin

  if EasyListviewExtColorGrouping.Selection.Count = 0 then
    Item := EasyListviewExtColorGrouping.Items.AddCustom(TExtColorCodeItem)
  else
    Item := EasyListviewExtColorGrouping.Items.InsertCustom(EasyListviewExtColorGrouping.Selection.First.Index, TExtColorCodeItem);
  if Assigned(Item) then
  begin
    Item.Captions[0] := '*.sample';
    (Item as TExtColorCodeItem).Color := clWindowText;
    (Item as TExtColorCodeItem).Bold := False;
    (Item as TExtColorCodeItem).Italic := False;
    (Item as TExtColorCodeItem).UnderLine := False;
    Item.Captions[1] := ColorToString(clWindowText);
    Item.Captions[2] := STR_FALSE;
    Item.Captions[3] := STR_FALSE;
    Item.Captions[4] := STR_FALSE;
    Item.Captions[5] := STR_SAMPLETEXT;
    Item.Checked := True;
    Item.Focused := True;
    Item.Selected := True;
    ColorGroupingEdit(Item as TExtColorCodeItem);
  end;
  Dirty(True);
end;

procedure TTntFormOptionsListviewColoring.ColorGroupingDelete;
begin
  EasyListviewExtColorGrouping.Selection.DeleteSelected(True);
  Dirty(True);
end;

procedure TTntFormOptionsListviewColoring.ColorGroupingEdit(Item: TExtColorCodeItem);
var
  F: TTntFormExtensionColorEditor;
begin
  F := TTntFormExtensionColorEditor.CreateEx(nil);
  try
    F.Extension := Item.Caption;
    F.Color := Item.Color;
    F.Bold := Item.Bold;
    F.Italic := Item.Italic;
    F.UnderLine := Item.UnderLine;
    if F.ShowModal = MROK then
    begin
      Item.LoadItems(F.Extension, F.Color, F.Bold, F.Italic, F.UnderLine, Item.Checked);
      Dirty(True)
    end
  finally
    F.Release;
  end;
end;

procedure TTntFormOptionsListviewColoring.IsDirtyClick(Sender: TObject);
begin
  Dirty(True)
end;

procedure TTntFormOptionsListviewColoring.SpTBXButtonEditGroupingCriteriaClick(Sender: TObject);
begin
  if EditGroupingCriteria(Self, SpTBXDockHeader, SpTBXToolbar) then
    Dirty(True)
end;

procedure TTntFormOptionsListviewColoring.FormShow(Sender: TObject);
begin
  inherited;
  FixFormFont(Font);
  ClientWidth := SpTBXPanelBkGnd.Width;
  ClientHeight := SpTBXPanelBkGnd.Height;
  EasyListviewExtColorGrouping.Header.Columns[0].Caption := STR_EXTENSION;
  EasyListviewExtColorGrouping.Header.Columns[1].Caption := STR_COLOR;
  EasyListviewExtColorGrouping.Header.Columns[2].Caption := STR_BOLD;
  EasyListviewExtColorGrouping.Header.Columns[3].Caption := STR_ITALIC;
  EasyListviewExtColorGrouping.Header.Columns[4].Caption := STR_UNDERLINE;
  EasyListviewExtColorGrouping.Header.Columns[ID_EXAMPLETEXT_COLUMN].Caption := STR_SAMPLE;
  EasyListviewExtColorGrouping.Header.Columns[ID_EXAMPLETEXT_COLUMN].BkGndColor := Attribs.Listview.Color;
  EasyListviewExtColorGrouping.Selection.Gradient := Attribs.Listview.GradientSelection;
  if EasyListviewExtColorGrouping.Selection.Gradient then
    EasyListviewExtColorGrouping.Selection.TextColor := $00A00000
  else
    EasyListviewExtColorGrouping.Selection.TextColor := clHighlightText;
end;

procedure TTntFormOptionsListviewColoring.EasyListviewExtColorGroupingOLEDragStart(
  Sender: TCustomEasyListview; ADataObject: IDataObject;
  var AvailableEffects: TCommonDropEffects; var AllowDrag: Boolean);
begin
  AllowDrag := True;
  AvailableEffects := [cdeMove];
  Dirty(True)
end;

procedure TTntFormOptionsListviewColoring.SpTBXButtonExtColorGroupingAddClick(Sender: TObject);
begin
  ColorGroupingAdd
end;

procedure TTntFormOptionsListviewColoring.SpTBXButtonExtColorGroupingDeleteClick(Sender: TObject);
begin
  ColorGroupingDelete;
end;

procedure TTntFormOptionsListviewColoring.EasyListviewExtColorGroupingItemDblClick(
  Sender: TCustomEasyListview; Button: TCommonMouseButton;
  MousePos: TPoint; HitInfo: TEasyHitInfoItem);
begin
  ColorGroupingEdit(HitInfo.Item as TExtColorCodeItem);
end;

procedure TTntFormOptionsListviewColoring.EasyListviewExtColorGroupingItemPaintText(
  Sender: TCustomEasyListview; Item: TEasyItem; Position: Integer;
  ACanvas: TCanvas);
begin
  if Position = ID_EXAMPLETEXT_COLUMN then
  begin
    ACanvas.Font.Color := TExtColorCodeItem(Item).Color;
    if TExtColorCodeItem(Item).Bold then
      ACanvas.Font.Style := ACanvas.Font.Style + [fsBold];
    if TExtColorCodeItem(Item).Italic then
      ACanvas.Font.Style := ACanvas.Font.Style + [fsItalic];
    if TExtColorCodeItem(Item).UnderLine then
      ACanvas.Font.Style := ACanvas.Font.Style + [fsUnderline]
  end
end;

procedure TTntFormOptionsListviewColoring.EasyListviewExtColorGroupingItemCheckChange(
  Sender: TCustomEasyListview; Item: TEasyItem);
begin
  inherited;
  Dirty(True)
end;

{ TExtColorCodeItem}
procedure TExtColorCodeItem.LoadItems(AnExtension: WideString; AColor: TColor; IsBold: Boolean; IsItalic: Boolean; IsUnderLine: Boolean; IsEnabled: Boolean);
begin
  Captions[0] := AnExtension;
  Color := AColor;
  Bold := IsBold;
  Italic := IsItalic;
  UnderLine := IsUnderLine;
  try
    Captions[1] := ColorToString(Color);
  except
    Color := clWindowText;
    Captions[1] := ColorToString(Color);
  end;
  if Bold then
    Captions[2] := STR_TRUE
  else
    Captions[2] := STR_FALSE;
  if Italic then
    Captions[3] := STR_TRUE
  else
    Captions[3] := STR_FALSE;
  if UnderLine then
    Captions[4] := STR_TRUE
  else
    Captions[4] := STR_FALSE;
  Captions[ID_EXAMPLETEXT_COLUMN] := STR_SAMPLETEXT;
  Checked := IsEnabled;
end;

procedure TTntFormOptionsListviewColoring.SpTBXItemColorGroupingAddClick(
  Sender: TObject);
begin
  ColorGroupingAdd
end;

procedure TTntFormOptionsListviewColoring.SpTBXItemColorGroupingDeleteClick(
  Sender: TObject);
begin
  ColorGroupingDelete
end;

procedure TTntFormOptionsListviewColoring.SpTBXItemColorGroupingSortClick(Sender: TObject);
begin
  EasyListviewExtColorGrouping.Sort.SortAll(True);
end;

procedure TTntFormOptionsListviewColoring.SpTBXItemColorGroupingEditClick(Sender: TObject);
begin
  if EasyListviewExtColorGrouping.Selection.Count > 0 then
    ColorGroupingEdit(EasyListviewExtColorGrouping.Selection.First as TExtColorCodeItem)
end;

procedure TTntFormOptionsListviewColoring.SpTBXItemColorGroupingCheckallClick(
  Sender: TObject);
begin
  EasyListviewExtColorGrouping.CheckManager.CheckAll
end;

procedure TTntFormOptionsListviewColoring.SpTBXItemColorGroupingUnCheckallClick(
  Sender: TObject);
begin
  EasyListviewExtColorGrouping.CheckManager.UnCheckAll
end;

procedure TTntFormOptionsListviewColoring.EasyListviewExtColorGroupingKeyAction(
  Sender: TCustomEasyListview; var CharCode: Word; var Shift: TShiftState;
  var DoDefault: Boolean);
begin
  DoDefault := False;
  case CharCode of
    VK_DELETE: ColorGroupingDelete;
    VK_INSERT: ColorGroupingAdd
  else
    DoDefault := True
  end;
end;

procedure TTntFormOptionsListviewColoring.SpTBXButtonExtColorGroupingEditClick(
  Sender: TObject);
begin
  if EasyListviewExtColorGrouping.Selection.Count > 0 then
    ColorGroupingEdit(EasyListviewExtColorGrouping.Selection.First as TExtColorCodeItem)
end;

procedure TTntFormOptionsListviewColoring.EasyListviewExtColorGroupingItemSelectionsChanged(
  Sender: TCustomEasyListview);
begin
  inherited;
  SpTBXButtonExtColorGroupingEdit.Enabled := EasyListviewExtColorGrouping.Selection.Count = 1
end;

end.


