unit UnitOptionsListviewGroupingColorEditor;

interface

uses
  Windows,
  Messages,
  SysUtils,
  Variants,
  Classes,
  Graphics,
  Controls,
  Forms,
  Dialogs,
  TntForms,
  SpTBXControls,
  StdCtrls,
  TntStdCtrls,
  MPCommonUtilities,
  MPResources,
  SpTBXEditors,
  UltraUserStrings,
  UltraOptions,
  DKLang,
  ExtCtrls, SpTBXItem;

type
  TTntFormExtensionColorEditor = class(TTntForm)
    SpTBXPanelBkGnd: TSpTBXPanel;
    SpTBXLabelExtension: TSpTBXLabel;
    SpTBXEditExtension: TSpTBXEdit;
    SpTBXButtonOk: TSpTBXButton;
    SpTBXButtonCancel: TSpTBXButton;
    DKLanguageController1: TDKLanguageController;
    SpTBXGroupBoxColor: TSpTBXGroupBox;
    SpTBXComboBoxColor: TSpTBXComboBox;
    SpTBXButtonCustomColor: TSpTBXButton;
    SpTBXCheckBoxBoldText: TSpTBXCheckBox;
    SpTBXLabelSample: TSpTBXLabel;
    ColorDialogExtensionEditor: TColorDialog;
    SpTBXCheckBoxItalicText: TSpTBXCheckBox;
    SpTBXEditSampleText: TSpTBXEdit;
    SpTBXCheckBoxUnderlineText: TSpTBXCheckBox;
    procedure TntFormShow(Sender: TObject);
    procedure TntFormCloseQuery(Sender: TObject; var CanClose: Boolean);
    procedure SpTBXButtonCustomColorClick(Sender: TObject);
    procedure SpTBXComboBoxColorChange(Sender: TObject);
    procedure SpTBXCheckBoxBoldTextClick(Sender: TObject);
    procedure SpTBXCheckBoxItalicTextClick(Sender: TObject);
    procedure TntFormKeyDown(Sender: TObject; var Key: Word;
      Shift: TShiftState);
    procedure SpTBXCheckBoxUnderlineTextClick(Sender: TObject);
  private
    { Private declarations }
    FBold: Boolean;
    FColor: TColor;
    FExtension: WideString;
    FItalic: Boolean;
    FUnderLine: Boolean;
    FWndParent: HWnd;
    procedure ColorCallback(const S: string);
    procedure UpdateSampleText;
  protected
    procedure CreateParams(var Params: TCreateParams); override;
    property WndParent: HWnd read FWndParent write FWndParent;
  public
    { Public declarations }
    constructor CreateEx(AnOwner: TCustomForm);

    property Bold: Boolean read FBold write FBold;
    property Color: TColor read FColor write FColor;
    property Extension: WideString read FExtension write FExtension;
    property Italic: Boolean read FItalic write FItalic;
    property UnderLine: Boolean read FUnderLine write FUnderLine;
  end;

implementation

uses
  UnitOKDialog;

{$R *.DFM}

constructor TTntFormExtensionColorEditor.CreateEx(AnOwner: TCustomForm);
begin
  if Assigned(AnOwner) then
     WndParent := AnOwner.Handle
   else
     WndParent := 0;
   inherited Create(nil);
end;

procedure TTntFormExtensionColorEditor.CreateParams(var Params: TCreateParams);
begin
  inherited;
  if WndParent <> 0 then
    Params.WndParent := WndParent;
end;

procedure TTntFormExtensionColorEditor.ColorCallback(const S: string);
begin
  SpTBXComboBoxColor.AddItem(S, nil);
end;

procedure TTntFormExtensionColorEditor.TntFormShow(Sender: TObject);
begin
  inherited;
  FixFormFont(Font);
  // Fix size for large font mode of th OS
  ClientWidth := SpTBXPanelBkGnd.Width;
  ClientHeight := SpTBXPanelBkGnd.Height;

  SpTBXEditSampleText.Text := STR_SAMPLETEXT;
  SpTBXEditSampleText.Color := Attribs.Listview.Color;

  SpTBXComboBoxColor.Clear;
  // Fill with the standard colors
  GetColorValues(ColorCallback);

  // Setup attributes
  SpTBXCheckBoxBoldText.Checked := Bold;
  SpTBXCheckBoxItalicText.Checked := Italic;
  SpTBXCheckBoxUnderlineText.Checked := UnderLine;

  SpTBXComboBoxColor.ItemIndex := SpTBXComboBoxColor.Items.IndexOf(  ColorToString( Color));
  // Not a standard color
  if SpTBXComboBoxColor.ItemIndex < 0 then
    SpTBXComboBoxColor.Text := ColorToString( Color);

  SpTBXEditExtension.Text := Extension;

  UpdateSampleText;
end;

procedure TTntFormExtensionColorEditor.TntFormCloseQuery(Sender: TObject; var CanClose: Boolean);
var
  F: TTntFormOkDialog;
begin
  if ModalResult = mrOk then
  begin
    try
      Extension := SpTBXEditExtension.Text;
      Color := StringToColor(SpTBXComboBoxColor.Text);
      Bold := SpTBXCheckBoxBoldText.Checked;
      Italic := SpTBXCheckBoxItalicText.Checked;
      UnderLine := SpTBXCheckBoxUnderLineText.Checked;
    except
      F := TTntFormOkDialog.CreateEx(nil);
      try
        F.Caption := STR_ERROR;
        F.SpTBXLabel.Caption := STR_INVALID_COLOR;
        F.ShowModal;
      finally
        CanClose := False;
        F.Release
      end
    end
  end
end;

procedure TTntFormExtensionColorEditor.SpTBXButtonCustomColorClick(Sender: TObject);
begin
  ColorDialogExtensionEditor.Color := StringToColor(SpTBXComboBoxColor.Text);
  if ColorDialogExtensionEditor.Execute then
  begin
    SpTBXComboBoxColor.Text := ColorToString(ColorDialogExtensionEditor.Color);
    UpdateSampleText
  end;
end;

procedure TTntFormExtensionColorEditor.SpTBXComboBoxColorChange(Sender: TObject);
begin
  UpdateSampleText
end;

procedure TTntFormExtensionColorEditor.UpdateSampleText;
begin
  try
    SpTBXEditSampleText.Font.Color := StringToColor(SpTBXComboBoxColor.Text);
    if SpTBXCheckBoxBoldText.Checked then
      SpTBXEditSampleText.Font.Style := SpTBXEditSampleText.Font.Style + [fsBold]
    else
      SpTBXEditSampleText.Font.Style := SpTBXEditSampleText.Font.Style - [fsBold];
    if SpTBXCheckBoxItalicText.Checked then
      SpTBXEditSampleText.Font.Style := SpTBXEditSampleText.Font.Style + [fsItalic]
    else
      SpTBXEditSampleText.Font.Style := SpTBXEditSampleText.Font.Style - [fsItalic];
    if SpTBXCheckBoxUnderLineText.Checked then
      SpTBXEditSampleText.Font.Style := SpTBXEditSampleText.Font.Style + [fsUnderline]
    else
      SpTBXEditSampleText.Font.Style := SpTBXEditSampleText.Font.Style - [fsUnderline];
  except
  end
end;

procedure TTntFormExtensionColorEditor.SpTBXCheckBoxBoldTextClick(Sender: TObject);
begin
  UpdateSampleText
end;

procedure TTntFormExtensionColorEditor.SpTBXCheckBoxItalicTextClick(Sender: TObject);
begin
  UpdateSampleText
end;

procedure TTntFormExtensionColorEditor.SpTBXCheckBoxUnderlineTextClick(
  Sender: TObject);
begin
  UpdateSampleText
end;

procedure TTntFormExtensionColorEditor.TntFormKeyDown(Sender: TObject; var Key: Word; Shift: TShiftState);
begin
  if Key = VK_ESCAPE then
  begin
    Key := 0;    // No beep
    Close
  end
end;

end.
