unit UnitOptionsTabs;

// The contents of this file are subject to the Mozilla Public License
// Version 1.1 (the "License"); you maynot use this file except in compliance
// with the License. You may obtain a copy of the License at http://www.mozilla.org/MPL/
//
// Alternatively, you may redistribute this library, use and/or modify it under the terms of the
// GNU Lesser General Public License as published by the Free Software Foundation;
// either version 2.1 of the License, or (at your option) any later version.
// You may obtain a copy of the LGPL at http://www.gnu.org/copyleft/.
//
// Software distributed under the License is distributed on an "AS IS" basis,
// WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for the
// specific language governing rights and limitations under the License.

interface

uses
  Windows,
  Messages,
  SysUtils,
  Variants,
  Classes,
  Graphics,
  Controls,
  Forms,
  Dialogs,
  TntForms,
  SpTBXDkPanels,
  SpTBXControls,
  UnitOptionsBase,
  StdCtrls,
  ShlObj,
  MPShellUtilities,
  UltraUtilities,
  SpTBXEditors,
  DKLang, SpTBXItem;

type
  TTntFormOptionsTabs = class(TTntOptionsBaseForm)
    SpTBXPanel1: TSpTBXPanel;
    SpTBXGroupBoxTabSetOptions: TSpTBXGroupBox;
    SpTBXCheckBoxNetworkDrivesTab: TSpTBXCheckBox;
    SpTBXCheckBoxRemoveableDrivesTab: TSpTBXCheckBox;
    SpTBXCheckBoxHideTabsetWithOne: TSpTBXCheckBox;
    SpTBXCheckBoxGeneralTabsShowHints: TSpTBXCheckBox;
    SpTBXCheckBoxGeneralSaveMappedDrives: TSpTBXCheckBox;
    SpTBXCheckBoxTabsetShowFullPaths: TSpTBXCheckBox;
    SpTBXRadioGroupDefaultNewTab: TSpTBXRadioGroup;
    SpTBXRadioGroupGeneralStartup: TSpTBXRadioGroup;
    SpTBXButtonEditDefaultNewTabLocation: TSpTBXButtonEdit;
    SpTBXButtonEditStartupCustomFolder: TSpTBXButtonEdit;
    DKLanguageController1: TDKLanguageController;
    SpTBXRadioGroupTabSetPosition: TSpTBXRadioGroup;
    SpTBXCheckBoxTabWidth: TSpTBXCheckBox;
    SpTBXSpinEditTabWidth: TSpTBXSpinEdit;
    SpTBXLabelTabWidthPixels: TSpTBXLabel;
    procedure IsDirtyClick(Sender: TObject);
    procedure SpTBXRadioGroupDefaultNewTabClick(Sender: TObject);
    procedure SpTBXRadioGroupGeneralStartupClick(Sender: TObject);
    procedure SpTBXButtonEditStartupCustomFolderSubEditButton0Click(
      Sender: TObject);
    procedure SpTBXButtonEditDefaultNewTabLocationSubEditButton0Click(
      Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure FormShow(Sender: TObject);
    procedure SpTBXCheckBoxTabWidthClick(Sender: TObject);
  private
    FCustomStartupPath: TNamespace;
    FCustomNewTabLocation: TNamespace;
    { Private declarations }
    procedure SetCustomNewTabLocation(const Value: TNamespace);
    procedure SetCustomStartupPath(const Value: TNamespace);
  public
    { Public declarations }
    property CustomNewTabLocation: TNamespace read FCustomNewTabLocation write SetCustomNewTabLocation;
    property CustomStartupPath: TNamespace read FCustomStartupPath write SetCustomStartupPath;
  end;

implementation

{$R *.DFM}

procedure TTntFormOptionsTabs.IsDirtyClick(Sender: TObject);
begin
  Dirty(True)
end;

procedure TTntFormOptionsTabs.SetCustomNewTabLocation(const Value: TNamespace);
begin
  if Assigned(FCustomNewTabLocation) then
    FreeAndNil(FCustomNewTabLocation);
  FCustomNewTabLocation := Value;
end;

procedure TTntFormOptionsTabs.SetCustomStartupPath(const Value: TNamespace);
begin
  if Assigned(FCustomStartupPath) then
    FreeAndNil(FCustomStartupPath);
  FCustomStartupPath := Value;
end;

procedure TTntFormOptionsTabs.SpTBXRadioGroupDefaultNewTabClick(
  Sender: TObject);
begin
  SpTBXButtonEditDefaultNewTabLocation.Enabled := SpTBXRadioGroupDefaultNewTab.ItemIndex = 1;
  Dirty(True)
end;

procedure TTntFormOptionsTabs.SpTBXRadioGroupGeneralStartupClick(Sender: TObject);
begin
  SpTBXButtonEditStartupCustomFolder.Enabled := SpTBXRadioGroupGeneralStartup.ItemIndex = 1;
  Dirty(True)
end;

procedure TTntFormOptionsTabs.SpTBXButtonEditStartupCustomFolderSubEditButton0Click(
  Sender: TObject);
var
  OutPIDL: PItemIDList;
begin
  if UltraSelectDirectory(Handle, CustomStartupPath.AbsolutePIDL, OutPIDL) then
  begin
    FreeAndNil(FCustomStartupPath);
    CustomStartupPath := TNamespace.Create(OutPIDL, nil);
    SpTBXButtonEditStartupCustomFolder.Text := CustomStartupPath.NameAddressBar;
    Dirty(True)
  end;
end;

procedure TTntFormOptionsTabs.SpTBXButtonEditDefaultNewTabLocationSubEditButton0Click(
  Sender: TObject);
var
  OutPIDL: PItemIDList;
begin
  if UltraSelectDirectory(Handle, CustomNewTabLocation.AbsolutePIDL, OutPIDL) then
  begin
    FreeAndNil(FCustomNewTabLocation);
    CustomNewTabLocation := TNamespace.Create(OutPIDL, nil);
    SpTBXButtonEditDefaultNewTabLocation.Text := CustomNewTabLocation.NameAddressbar;
    Dirty(True)
  end;

end;

procedure TTntFormOptionsTabs.FormDestroy(Sender: TObject);
begin
  FreeAndNil(FCustomNewTabLocation);
  FreeAndNil(FCustomStartupPath);
  inherited;
end;

procedure TTntFormOptionsTabs.FormShow(Sender: TObject);
begin
  ClientWidth := SpTBXPanel1.Width;
  ClientHeight := SpTBXPanel1.Height;
  SpTBXSpinEditTabWidth.Enabled := SpTBXCheckBoxTabWidth.Checked;
  SpTBXLabelTabWidthPixels.Enabled := SpTBXCheckBoxTabWidth.Checked;
end;

procedure TTntFormOptionsTabs.SpTBXCheckBoxTabWidthClick(Sender: TObject);
begin
  SpTBXSpinEditTabWidth.Enabled := SpTBXCheckBoxTabWidth.Checked;
  SpTBXLabelTabWidthPixels.Enabled := SpTBXCheckBoxTabWidth.Checked;
  Dirty(True)
end;

end.
