unit UnitPlugInBaseFrame;

interface

uses
  Windows,
  Messages,
  SysUtils,
  Variants,
  Classes,
  Graphics,
  Controls,
  Forms,
  Dialogs,
  TntForms,
  ImgList,
  ActiveX,
  ShlObj,
  Menus,
  UltraUtilities,
  UltraOptions,
  UnitInPlaceWndInf,
  SpTBXItem,
  TB2Item,
  VirtualExplorerEasyListview,
  EasyListview,
  MPShellUtilities,
  MPCommonUtilities,
  VirtualExplorerTree,
  VirtualThumbnails,
  VirtualExplorerEasyListModeview,
  TntClasses,
  SpTBXTabs,
  MPThreadManager,
  VirtualShellNotifier;

const
  ID_FILTER_BKGND = 1001;          // Resource ID

type
  TTntFrameBasePlugIn = class(TTntFrame, IUnknown, IUltraInPlaceWindow)
  private
    { Private declarations }
    FBkGndMenu: TVirtualShellBackgroundContextMenu;
    FEventsConnected: Boolean;
    FNotifier: IUltraNotifier;
    FOwnerTabItem: TSpTBXTabItem;     // The UltraSpTBXTabItem that owns the plugin window
    FRefCount: Integer;
  protected
    function ActionCalculateFolderSizes(SelectedOnly, Force: Boolean): Boolean; virtual;
    function ActionClearFolderSizes: Boolean; virtual;
    function ActionClipboard(ActionType: TViewClipboardAction): Boolean;
    function ActionCopyPath(Info: PViewCopyPathAction): Boolean; virtual;
    function ActionCopyToClipboard: Boolean; virtual;
    function ActionCreateNewFolder: Boolean; virtual;
    function ActionCutToClipboard: Boolean; virtual;
    function ActionQuickFilter(Info: PViewQuickFilterRecAction): Boolean; virtual;
    function ActionRebuild: Boolean; virtual;
    function ActionSelect(Info: PViewSelectRecAction): Boolean;
    function ActionSelectAll(Info: PViewSelectRecAction): Boolean; virtual;
    function ActionSelectAllFiles(Info: PViewSelectRecAction): Boolean; virtual;
    function ActionSelectAllFolders(Info: PViewSelectRecAction): Boolean; virtual;
    function ActionSelectedDelete(Info: PViewSelectRecAction): Boolean; virtual;
    function ActionInvalidateAndRedraw: Boolean; virtual;
    function ActionPasteAsShortCutFromClipboard: Boolean; virtual;
    function ActionPasteFromClipboard: Boolean; virtual;
    function ActionSelectedInvert(Info: PViewSelectRecAction): Boolean; virtual;
    function ActionSelectMatchText(Info: PViewSelectRecAction): Boolean; virtual;
    function ActionUnSelectAllFiles(Info: PViewSelectRecAction): Boolean; virtual;
    function ActionUnSelectAllFolders(Info: PViewSelectRecAction): Boolean; virtual;
    function ActionUnSelectMatchText(Info: PViewSelectRecAction): Boolean; virtual;
    procedure DisconnectViewEventsAndBkGndMenu; virtual; abstract;  // nil all events here and disconnect the Background menu from the view
    procedure DoGlobalThreadCallback(Msg: TWMThreadRequest); virtual;
    procedure LoadDefaultListviewEventsAndConnectBkGndMenu(Listview: TCustomVirtualExplorerEasyListview; ConnectBkGndMenu, Clear: Boolean);
    procedure LoadDefaultListviewProperties(Listview: TCustomVirtualExplorerEasyListview);
    procedure LoadDefaultViewType(Listview: TCustomVirtualExplorerEasyListview); virtual;
    procedure LoadUserListviewProperties(Listview: TCustomVirtualExplorerEasyListview);
    // Common Listview Event Handlers
    procedure OnListviewClipboardChange(Sender: TObject);
    procedure OnListviewColumnContextMenu(Sender: TCustomEasyListview; HitInfo: TEasyHitInfoColumn; WindowPoint: TPoint; var Menu: TPopupMenu);
    procedure OnListviewContextMenu(Sender: TCustomEasyListview; MousePt: TPoint; var Handled: Boolean);
    procedure OnListviewContextMenu2Message(Sender: TCustomVirtualExplorerEasyListview; var Msg: TMessage);
    procedure OnListviewContextMenuAfterCmd(Sender: TCustomVirtualExplorerEasyListview; Namespace: TNamespace; Verb: WideString; MenuItemID: Integer; Successful: Boolean);
    procedure OnListviewContextMenuCmd(Sender: TCustomVirtualExplorerEasyListview; Namespace: TNamespace; Verb: WideString; MenuItemID: Integer; var Handled: Boolean);
    procedure OnListviewContextMenuItemChange(Sender: TCustomVirtualExplorerEasyListview; Namespace: TNamespace; MenuItemID: Integer; SubMenuID: hMenu; MouseSelect: Boolean);
    procedure OnListviewDblClick(Sender: TCustomEasyListview; Button: TCommonMouseButton; MousePos: TPoint; ShiftState: TShiftState);
    procedure OnListviewEnter(Sender: TObject);
    procedure OnListviewEnumFinished(Sender: TCustomVirtualExplorerEasyListview);
    procedure OnListviewEnumFolder(Sender: TCustomVirtualExplorerEasyListview; Namespace: TNamespace; var AllowAsChild: Boolean);
    procedure OnListviewItemCustomView(Sender: TCustomEasyListview; Item: TEasyItem; ViewStyle: TEasyListStyle; var View: TEasyViewItemClass);
    procedure OnListviewItemDblClick(Sender: TCustomEasyListview; Button: TCommonMouseButton; MousePos: TPoint; HitInfo: TEasyHitInfoItem);
    procedure OnListviewItemEdited(Sender: TCustomEasyListview; Item: TEasyItem; var NewValue: Variant; var Accept: Boolean);
    procedure OnListviewItemFocusChanged(Sender: TCustomEasyListview; Item: TEasyItem);
    procedure OnListviewItemMouseDown(Sender: TCustomEasyListview; Item: TEasyItem; Button: TCommonMouseButton; var DoDefault: Boolean);
    procedure OnListviewItemSelectionsChanged(Sender: TCustomEasyListview);
    procedure OnListviewKeyAction(Sender: TCustomEasyListview; var CharCode: Word; var Shift: TShiftState; var DoDefault: Boolean);
    procedure OnListviewLoadStorageFromRoot(Sender: TCustomVirtualExplorerEasyListview; NodeStorage: TNodeStorage);
    procedure OnListviewMouseDown(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
    procedure OnListviewMouseGesture(Sender: TCustomEasyListview; Gesture: WideString; var DoDefaultMouseAction: Boolean);
    procedure OnListviewMouseUp(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
    procedure OnListviewScroll(Sender: TCustomEasyListview; DeltaX, DeltaY: Integer);
    procedure OnListviewRootChange(Sender: TCustomVirtualExplorerEasyListview); virtual;
    procedure OnListviewRootChanging(Sender: TCustomVirtualExplorerEasyListview; const NewValue: TRootFolder; const CurrentNamespace, Namespace: TNamespace; var Allow: Boolean);
    procedure OnListviewSaveRootToStorage(Sender: TCustomVirtualExplorerEasyListview; NodeStorage: TNodeStorage);
    procedure OnListviewSetView(Sender: TCustomEasyListview);
    procedure OnListviewShellExecute(Sender: TCustomVirtualExplorerEasyListview; Namespace: TNamespace; var WorkingDir: WideString; var CmdLineArgument: WideString; var Allow: Boolean);
    procedure OnListviewShellNotify(Sender: TCustomVirtualExplorerEasyListview; ShellEvent: TVirtualShellEvent);
    procedure RefreshOwnerTabUI;
    procedure WMEasyThreadCallback(var Msg: TWMThreadRequest); message WM_COMMONTHREADCALLBACK;
    property BkGndMenu: TVirtualShellBackgroundContextMenu read FBkGndMenu write FBkGndMenu;
    property RefCount: Integer read FRefCount write FRefCount;
  public
    { Public declarations }

    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    // IUnknown
    function QueryInterface(const IID: TGUID; out Obj): HResult; override; stdcall;
    function _AddRef: Integer; reintroduce; stdcall;
    function _Release: Integer; reintroduce; stdcall;

    // IUltraInPlaceWindow
    procedure ConnectEventsAndBkGndMenu(DoConnect, DoConnectBkGndMenu: Boolean); virtual;
    function ActiveViewUI(DoActiveate: Boolean): Boolean; virtual; abstract;
    function DestroyWindow: Boolean; virtual;
    function GetHandle(var WindowHandle: hWnd): Boolean; virtual;
    function GetObject(var APlugin: TTntFrame): Boolean; virtual;
    function HandleMessage(var Msg: TMsg; var Handled: Boolean): Boolean; virtual;
    function HintMsg(var AHint: WideString): Boolean; virtual;
    function LoadConfig(StreamVer: Integer; ConfigStream: TTntFileStream): Boolean; virtual; abstract;
    function Persistent: Boolean; virtual;
    function RegisterNotifier(ANotifier: IUltraNotifier): Boolean; virtual;
    function SaveConfig(StreamVer: Integer; ConfigStream: TTntFileStream): Boolean; virtual; abstract;
    function SetParentWnd(AParent: TWinControl): Boolean; virtual;
    function SetTheme(ThemeName: PChar): Boolean;  virtual;
    function SetWindowBounds(AWidth, AHeight: Integer): Boolean; virtual;
    function ShowWindow(Show: Boolean): Boolean; virtual;
    procedure SetOwnerItem(ATabItem: TSpTBXTabItem);
    function TabCaption(var ACaption: WideString): Boolean; virtual;
    function TabImage(var AnImageList: TCustomImageList; var AnImageIndex: Integer): Boolean; virtual;
    procedure ViewBeginUpdate; virtual;
    procedure ViewEndUpdate; virtual;
    function ViewExecuteAction(Action: TViewExecuteAction; Data: Integer): Boolean;
    function ViewItemCount: Integer; virtual;
    function ViewItemFirst: TExplorerItem; virtual;
    function ViewItemNext(Item: TExplorerItem): TExplorerItem; virtual;
    function ViewRootPIDL: PItemIDList; virtual;
    function ViewSelectionCount: Integer; virtual;
    function ViewSelectionFirst: TExplorerItem; virtual;
    procedure ViewSelectionChangeLock(DoLock: Boolean); virtual;
    function ViewSelectionNext(Item: TExplorerItem): TExplorerItem; virtual;
    procedure ViewSetRootPIDL(PIDL: PItemIDList); virtual;
    function ViewShellFolder: IShellFolder; virtual;
    function ViewTopItem: TExplorerItem; virtual;

    property EventsConnected: Boolean read FEventsConnected;
    property Notifier: IUltraNotifier read FNotifier write FNotifier;
    property OwnerTabItem: TSpTBXTabItem read FOwnerTabItem write FOwnerTabItem;
  end;

  TTntFrameBasePlugInClass = class of TTntFrameBasePlugIn;

implementation

{$R *.DFM}


type
  TCustomVirtualExplorerEasyListviewHack = class(TCustomVirtualExplorerEasyListview);

constructor TTntFrameBasePlugIn.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  BkGndMenu := TVirtualShellBackgroundContextMenu.Create(Self);
end;

destructor TTntFrameBasePlugIn.Destroy;
begin
  BkGndMenu.Free;     
  inherited Destroy;
end;

function TTntFrameBasePlugIn.ActionCalculateFolderSizes(SelectedOnly, Force: Boolean): Boolean;
begin
   Result := False
end;

function TTntFrameBasePlugIn.ActionClearFolderSizes: Boolean;
begin
  Result := False
end;

function TTntFrameBasePlugIn.ActionClipboard(ActionType: TViewClipboardAction): Boolean;
begin
  case ActionType of
    vcaCopy: Result := ActionCopyToClipboard;
    vcaCut: Result := ActionCutToClipboard;
    vcaPaste: Result := ActionPasteFromClipboard;
    vcaPasteAsShortcut: Result := ActionPasteAsShortCutFromClipboard
  end
end;

function TTntFrameBasePlugIn.ActionCopyPath(Info: PViewCopyPathAction): Boolean;
begin
  Result := False
end;

function TTntFrameBasePlugIn.ActionCreateNewFolder: Boolean;
begin
  Result := False
end;

function TTntFrameBasePlugIn.ActionQuickFilter(Info: PViewQuickFilterRecAction): Boolean;
begin
  Result := False
end;

function TTntFrameBasePlugIn.ActionRebuild: Boolean;
begin
  Result := False
end;

function TTntFrameBasePlugIn.ActionSelect(Info: PViewSelectRecAction): Boolean;
begin
  case Info.Action of
    vsaSelectedDelete:      Result := ActionSelectedDelete(Info);
    vsaSelectedInvert:      Result := ActionSelectedInvert(Info);
    vsaSelectAll:           Result := ActionSelectAll(Info);
    vsaSelectAllFiles:      Result := ActionSelectAllFiles(Info);
    vsaSelectAllFolders:    Result := ActionSelectAllFolders(Info);
    vsaSelectMatchText:     Result := ActionSelectMatchText(Info);
    vsaUnSelectFiles:       Result := ActionUnSelectAllFiles(Info);
    vsaUnSelectFolders:     Result := ActionUnSelectAllFolders(Info);
    vsaUnSelectMatchText:   Result := ActionUnSelectMatchText(Info);
  end;
end;

function TTntFrameBasePlugIn.ActionSelectAll(Info: PViewSelectRecAction): Boolean;
begin
  Result := False
end;

function TTntFrameBasePlugIn.ActionSelectAllFiles(Info: PViewSelectRecAction): Boolean;
begin

end;

function TTntFrameBasePlugIn.ActionSelectAllFolders(Info: PViewSelectRecAction): Boolean;
begin

end;

function TTntFrameBasePlugIn.ActionSelectedDelete(Info: PViewSelectRecAction): Boolean;
begin
  Result := False
end;

function TTntFrameBasePlugIn.ActionInvalidateAndRedraw: Boolean;
begin
  Result := False
end;

function TTntFrameBasePlugIn.ActionPasteAsShortCutFromClipboard: Boolean;
begin
  Result := False;
end;

function TTntFrameBasePlugIn.ActionSelectedInvert(Info: PViewSelectRecAction): Boolean;
begin
  Result := False
end;

function TTntFrameBasePlugIn.ActionSelectMatchText(Info: PViewSelectRecAction): Boolean;
begin
  Result := False
end;

function TTntFrameBasePlugIn.ActionUnSelectAllFiles(Info: PViewSelectRecAction): Boolean;
begin
  Result := False
end;

function TTntFrameBasePlugIn.ActionUnSelectAllFolders(Info: PViewSelectRecAction): Boolean;
begin
  Result := False
end;

function TTntFrameBasePlugIn.ActionUnSelectMatchText(Info: PViewSelectRecAction): Boolean;
begin
  Result := False
end;

function TTntFrameBasePlugIn.DestroyWindow: Boolean;
begin
  DisconnectViewEventsAndBkGndMenu;
  DestroyHandle;
  Result := True
end;

function TTntFrameBasePlugIn.GetHandle(var WindowHandle: hWnd): Boolean;
begin
  WindowHandle := 0;
  if HandleAllocated then
  begin
    WindowHandle := Handle;
    Result := True
  end else
    Result := False
end;

function TTntFrameBasePlugIn.GetObject(var APlugin: TTntFrame): Boolean;
begin
  APlugin := Self;
  Result := True
end;

function TTntFrameBasePlugIn.HandleMessage(var Msg: TMsg; var Handled: Boolean): Boolean;
begin
  Handled := False;
  Result := True;
end;

function TTntFrameBasePlugIn.HintMsg(var AHint: WideString): Boolean;
begin
  Result := True;
  AHint := Hint
end;

function TTntFrameBasePlugIn.Persistent: Boolean;
begin
  Result := False;
end;

function TTntFrameBasePlugIn.RegisterNotifier(ANotifier: IUltraNotifier): Boolean;
begin
  // Sends an valid IUltraNotifier when the plugin is created and a nil when
  // the Notifier Sink is being destroyed or the plugin is being destroyed
  Notifier := ANotifier;
  Result := True
end;

function TTntFrameBasePlugIn.SetParentWnd(AParent: TWinControl): Boolean;
begin
  Parent := AParent;
  Align := alClient;
  Result := Parent = AParent
end;

function TTntFrameBasePlugIn.SetTheme(ThemeName: PChar): Boolean;
begin
  // Use if Form is in DLL where a new TBXSwitcher is needed
  Result := True
end;

function TTntFrameBasePlugIn.SetWindowBounds(AWidth, AHeight: Integer): Boolean;
begin
  Width := AWidth;
  Height := AHeight;
  Result := (AWidth = Width) and (AHeight = Height)
end;

function TTntFrameBasePlugIn.ShowWindow(Show: Boolean): Boolean;
begin
  Visible := Show;
  Result := Visible = Show
end;

function TTntFrameBasePlugIn.TabCaption(var ACaption: WideString): Boolean;
begin
  Result := True;
  ACaption := Caption
end;

function TTntFrameBasePlugIn.ViewExecuteAction(Action: TViewExecuteAction; Data: Integer): Boolean;
begin
  Result := False;
  case Action of
    veaCreateNewFolder: Result := ActionCreateNewFolder;
    veaClipboard: Result := ActionClipboard(TViewClipboardAction( Data));
    veaCalcualteFolderSizes: Result := ActionCalculateFolderSizes(Boolean( LoWord( Data)), Boolean( HiWord( Data)));
    veaFolderSizesClear: Result := ActionClearFolderSizes;
    veaInvalidateAndRedraw: Result := ActionInvalidateAndRedraw;
    veaSelect: Result := ActionSelect( PViewSelectRecAction( Data));
    veaQuickFilter: Result := ActionQuickFilter( PViewQuickFilterRecAction( Data));
    vaeRebuild: Result := ActionRebuild;
    vaeCopyPath: Result := ActionCopyPath( PViewCopyPathAction( Data));
  end;
end;

function TTntFrameBasePlugIn.ViewItemCount: Integer;
begin
  Result := 0
end;

function TTntFrameBasePlugIn.ViewItemFirst: TExplorerItem;
begin
  Result := nil
end;

function TTntFrameBasePlugIn.ViewItemNext(Item: TExplorerItem): TExplorerItem;
begin
  Result := nil
end;

function TTntFrameBasePlugIn.ViewRootPIDL: PItemIDList;
begin
  // Caution this is expected to return the actual PIDL, NOT a COPY!!!!
  Result := nil
end;

function TTntFrameBasePlugIn.ViewSelectionCount: Integer;
begin
  Result := 0
end;

function TTntFrameBasePlugIn.ViewSelectionFirst: TExplorerItem;
begin
  Result := nil
end;

function TTntFrameBasePlugIn.ViewSelectionNext(Item: TExplorerItem): TExplorerItem;
begin
  Result := nil
end;

function TTntFrameBasePlugIn.ViewShellFolder: IShellFolder;
begin

end;

function TTntFrameBasePlugIn.ViewTopItem: TExplorerItem;
begin
  Result := nil
end;

function TTntFrameBasePlugIn._AddRef: Integer;
begin
  InterlockedIncrement(FRefCount);
  Result := RefCount;
end;

function TTntFrameBasePlugIn._Release: Integer;
begin
  InterlockedDecrement(FRefCount);
  Result := RefCount;
  if FRefCount = 0 then
    Free
end;

function TTntFrameBasePlugIn.ActionCopyToClipboard: Boolean;
begin
  Result := False
end;

function TTntFrameBasePlugIn.ActionCutToClipboard: Boolean;
begin
  Result := False
end;

function TTntFrameBasePlugIn.ActionPasteFromClipboard: Boolean;
begin
  Result := False
end;

procedure TTntFrameBasePlugIn.ConnectEventsAndBkGndMenu(DoConnect, DoConnectBkGndMenu: Boolean);
begin
  FEventsConnected :=  DoConnect
end;

procedure TTntFrameBasePlugIn.DoGlobalThreadCallback(Msg: TWMThreadRequest);
begin

end;

{ TTntFrame1}
procedure TTntFrameBasePlugIn.LoadDefaultListviewEventsAndConnectBkGndMenu(Listview: TCustomVirtualExplorerEasyListview; ConnectBkGndMenu, Clear: Boolean);
var
  LV: TCustomVirtualExplorerEasyListviewHack;
begin
  LV := TCustomVirtualExplorerEasyListviewHack( Listview);
  if Clear then
  begin
    LV.OnClipboardChange := nil;
    LV.OnColumnContextMenu := nil;
    LV.OnContextMenu := nil;
    LV.OnContextMenu2Message := nil;
    LV.OnContextMenuAfterCmd := nil;
    LV.OnContextMenuCmd := nil;
    LV.OnContextMenuItemChange := nil;
    LV.OnDblClick := nil;
    LV.OnEnter := nil;
    LV.OnEnumFinished := nil;
    LV.OnEnumFolder := nil;
    LV.OnItemCustomView := nil;
    LV.OnItemDblClick := nil;
    LV.OnItemEdited := nil;
    LV.OnItemFocusChanged := nil;
    LV.OnItemMouseDown := nil;
    LV.OnItemSelectionsChanged := nil;
    LV.OnKeyAction := nil;
    LV.OnLoadStorageFromRoot := nil;
    LV.OnMouseDown := nil;
    LV.OnMouseGesture := nil;
    LV.OnMouseUp := nil;
    LV.OnSaveRootToStorage := nil;
    LV.OnViewChange := nil;
    LV.OnScroll := nil;
    LV.OnShellExecute := nil;
    LV.BackGndMenu := nil;
    LV.OnShellNotify := nil;
  end else
  begin
    LV.OnClipboardChange := OnListviewClipboardChange;
    LV.OnColumnContextMenu := OnListviewColumnContextMenu;
    LV.OnContextMenu := OnListviewContextMenu;
    LV.OnContextMenu2Message := OnListviewContextMenu2Message;
    LV.OnContextMenuAfterCmd := OnListviewContextMenuAfterCmd;
    LV.OnContextMenuCmd := OnListviewContextMenuCmd;
    LV.OnContextMenuItemChange := OnListviewContextMenuItemChange;
    LV.OnDblClick := OnListviewDblClick;
    LV.OnEnter := OnListviewEnter;
    LV.OnEnumFinished := OnListviewEnumFinished;
    LV.OnEnumFolder := OnListviewEnumFolder;
    LV.OnItemCustomView := OnListviewItemCustomView;
    LV.OnItemDblClick := OnListviewItemDblClick;
    LV.OnItemEdited := OnListviewItemEdited;
    LV.OnItemFocusChanged := OnListviewItemFocusChanged;
    LV.OnItemMouseDown := OnListviewItemMouseDown;
    LV.OnItemSelectionsChanged := OnListviewItemSelectionsChanged;
    LV.OnKeyAction := OnListviewKeyAction;
    LV.OnLoadStorageFromRoot := OnListviewLoadStorageFromRoot;
    LV.OnMouseDown := OnListviewMouseDown;
    LV.OnMouseGesture := OnListviewMouseGesture;
    LV.OnMouseUp := OnListviewMouseUp;
    LV.OnRootChanging := OnListviewRootChanging;
    LV.OnSaveRootToStorage := OnListviewSaveRootToStorage;
    LV.OnViewChange := OnListviewSetView;
    LV.OnScroll := OnListviewScroll;
    LV.OnShellExecute := OnListviewShellExecute;
    if ConnectBkGndMenu then
      LV.BackGndMenu := BkGndMenu
  end;
end;

procedure TTntFrameBasePlugIn.LoadDefaultListviewProperties(Listview: TCustomVirtualExplorerEasyListview);
var
  LV: TCustomVirtualExplorerEasyListviewHack;
begin
  LV := TCustomVirtualExplorerEasyListviewHack( Listview);
  LV.IncrementalSearch.StartType := eissFocusedNode;
  LV.ShowHint := True; // Always Show the hint/Tooltip
  LV.IncrementalSearch.ResetTime := 500;
//  LV.OpenViewer := OpenViewer;
//  LV.ViewerOpenFile := ViewerOpenFile;
//  LV.CmdLineSetPath := CmdLineSetPath;
//  LV.CmdLineCurrentPath := @CmdLineCurrentPath;
  LV.Active := False;
  LV.Align := alClient;
  LV.BevelInner := bvNone;
  LV.BevelOuter := bvNone;
  LV.BorderWidth := 0;
  LV.ShowThemedBorder := False;
  LV.Themed := True;
  LV.Ctl3D := False;
  LV.BevelWidth := 1;
  LV.Color := clWindow;
  LV.TabStop := True;
  LV.DragManager.Enabled := True;
  LV.Font.Color := clWindowText;
  LV.Grouped := False;
  LV.GroupingColumn := 0;
  LV.GroupFont.Color := clWindowText;
  LV.Header.Font.Color := clWindowText;
  LV.Header.Visible := True;
  LV.IncrementalSearch.Enabled := True;
  
  LV.HotTrack.Color := LV.Font.Color;
  LV.BackGround.Image.LoadFromResourceID(hInstance, ID_FILTER_BKGND);
  LV.BackGround.Transparent := True;
  LV.BackGround.OffsetTrack := True;
  LV.BackGround.AlphaBlend := True;
  LV.BackGround.Tile := False;
  LV.BackGround.BlendAlpha := 192;
  LV.PaintInfoGroup.MarginTop.Visible := False;
  LV.ShowInactive := False;
  LV.GroupFont.Style := LV.GroupFont.Style + [fsBold];
  LV.ShowThemedBorder := False;
  LV.Selection.AlphaBlendSelRect := True;
  LV.Selection.EnableDragSelect := True;
  LV.Selection.MouseButton := [cmbLeft, cmbRight];
  LV.Selection.MouseButtonSelRect := [cmbLeft, cmbRight];
  LV.Selection.MultiSelect := True;
  LV.Selection.RectSelect := False;
  LV.Selection.RoundRect := True;
  LV.DragManager.MouseButton := [cmbLeft, cmbRight];
  LV.TabOrder := 0;
  LV.ThumbsManager.StorageCompressed := False;
  LV.ThumbsManager.StorageFilename := 'Thumbnails.album';
  LV.EditManager.Enabled := True;
  LV.GroupingModifiedArray := GroupingModifiedArray;
  LV.GroupingFileSizeArray := GroupingFileSizeArray;
  LV.EditManager.AutoEditDelayTime  := 750;
  LV.Options := [
    eloChangeNotifierThread,
    eloBrowseExecuteFolder,
    eloFullFlushItemsOnChangeNotify,
    eloBrowseExecuteFolderShortcut,
    eloBrowseExecuteZipFolder,
    eloExecuteOnDblClick,
    eloThreadedImages,
    eloQueryInfoHints,
    eloShellContextMenus,
    eloTrackChangesInMappedDrives,
    {$IFDEF THREADEDENUM}eloThreadedEnumeration, {$ENDIF}
    eloThreadedDetails,
    eloGhostHiddenFiles
  ];
end;

procedure TTntFrameBasePlugIn.LoadDefaultViewType(Listview: TCustomVirtualExplorerEasyListview);
var
  LV: TCustomVirtualExplorerEasyListviewHack;
begin
  LV := TCustomVirtualExplorerEasyListviewHack( Listview);
  case Attribs.Listview.DefaultView of
    0: LV.View := elsIcon;
    1: LV.View := elsSmallIcon;
    2: LV.View := elsList;
    3: LV.View := elsReport;
    4: LV.View := elsThumbnail;
    5: LV.View := elsTile;
    6: begin // TBD
       end;
    end;
end;

procedure TTntFrameBasePlugIn.LoadUserListviewProperties(Listview: TCustomVirtualExplorerEasyListview);
//
// Called from SaveAttribs which is called in two instances
//  1) After the Options file is read from the disk to allow to copy the
//     saved options to the controls
//  2) After the "Apply" or "Ok" button is pressed in the options dialog.
//     It is called after the options from the dialog are written to the
//     passed Attribs class.
//
var
  LV: TCustomVirtualExplorerEasyListviewHack;

  procedure ApplyProperties;
  //
  // Based on user properties some other props may need to be reset
  //
  begin
    if LV.Selection.Gradient then
    begin
      LV.Selection.AlphaBlend := True;
      LV.Selection.TextColor := Font.Color;
      LV.Selection.FullCellPaint := True;
    end else
    begin
      LV.Selection.AlphaBlend := False;
      LV.Selection.TextColor := clHighlightText;
      LV.Selection.FullCellPaint := False;
    end;
  end;

begin
  LV := TCustomVirtualExplorerEasyListviewHack( Listview);
  LV.HotTrack.Enabled := Attribs.Listview.HotTrack;
//  Listview.CellSizes.List.AutoSize := Attribs.Listview.AutoSizeCells;
//  Listview.CellSizes.Report.AutoSize := Attribs.Listview.AutoSizeCells;
//  Listview.CellSizes.SmallIcon.AutoSize := Attribs.Listview.AutoSizeCells;
  LV.CellSizes.Thumbnail.Height := Attribs.Listview.ThumbsHeight;
  LV.CellSizes.Thumbnail.Width := Attribs.Listview.ThumbsWidth;
  LV.Color := Attribs.Listview.Color;
  LV.CompressedFile.Hilight := Attribs.Listview.HilightCompressed;
  LV.DefaultSortColumn := Attribs.Listview.DefaultColumn;
  LV.DefaultSortDir := Attribs.Listview.SortDirection;
  LV.EditManager.Font.Assign(LV.Font);
  LV.EncryptedFile.Hilight := Attribs.Listview.HilightEncrypted;
  LV.ExtensionColorCode := Attribs.Listview.ExtensionColorCode;
  LV.ExtensionColorCodeList.Assign(Attribs.Listview.ExtensionColorCodeList);
  LV.FileSizeFormat := TVirtualFileSizeFormat( Attribs.Listview.FileSizeStyle);
  LV.Font.Assign(Attribs.Listview.Font);
  LV.Header.Font.Assign(Attribs.Listview.Font);
  LV.Header.ShowInAllViews := Attribs.Listview.AlwaysShowHeader;
  LV.HotTrack.Color := LV.Font.Color;
  LV.HotTrack.Enabled := Attribs.Listview.HotTrack;
  LV.PaintInfoColumn.HilightFocusedColor := Attribs.Listview.HilightColumnColor;
  LV.PaintInfoItem.GridLines := Attribs.Listview.GridLines;
  LV.QueryInfoHintTimeout := Attribs.Listview.QueryInfoHintTimout;
  LV.Selection.FullRowSelect := Attribs.Listview.FullRowSelect;
  LV.Selection.Gradient := Attribs.Listview.GradientSelection;
  LV.Selection.UseFocusRect := Attribs.Listview.FocusRect;
  LV.SortFolderFirstAlways := Attribs.Listview.FoldersAlwaysFirst;
  LV.ThumbsManager.AutoLoad := Attribs.Listview.ThumbsCache;
  LV.ThumbsManager.AutoSave := Attribs.Listview.ThumbsCache;
  LV.ThumbsManager.HideCaption := Attribs.Listview.HideCaptions;
  LV.ThumbsManager.LoadAllAtOnce := Attribs.Listview.ThumbsLoadAll;
  LV.ThumbsManager.StorageCompressed := Attribs.Listview.ThumbsCompress;
  LV.ThumbsManager.StorageFilename := WideExtractFileName(Attribs.Listview.ThumbsCacheFile);
  LV.ThumbsManager.StorageRepositoryFolder := WideExtractFilePath(Attribs.Listview.ThumbsCacheFile);
  LV.ThumbsManager.UseExifOrientation := Attribs.Listview.AutoOrientation;

 { if Listview is TUltraExplorerEasyListview then
  begin
    UEListview := TUltraExplorerEasyListview( Listview);
    if Assigned(UEListview.ColumnModeView) then
    begin
      UEListview.ColumnModeView.DefaultSortColumn := Attribs.Listview.DefaultColumn;
      UEListview.ColumnModeView.DefaultSortDir := Attribs.Listview.SortDirection;
      UEListview.ColumnModeView.SmoothScrollDelta := Attribs.Listview.SmoothScroll;
      UEListview.ColumnModeView.HilightActiveColumn := Attribs.Listview.HilightColumn;
      UEListview.ColumnModeView.HilightActiveColumn := Attribs.Listview.HilightColumn;
      UEListview.ColumnModeView.HilightColumnColor := Attribs.Listview.HilightColumnColor;
      if Attribs.Listview.QueryInfoHints then
        UEListview.ColumnModeView.Options := UEListview.ColumnModeView.Options + [eloQueryInfoHints]
      else
        UEListview.ColumnModeView.Options := UEListview.ColumnModeView.Options - [eloQueryInfoHints];
      UEListview.ColumnModeView.Font.Assign(Attribs.Listview.Font);
      UEListview.ColumnModeView.DefaultColumnWidth := Attribs.Listview.DefaultColumnModeWidth;
      UEListview.ColumnModeView.SortFolderFirstAlways := Attribs.Listview.FoldersAlwaysFirst;
      UEListview.ColumnModeView.Color := Attribs.Listview.Color;
      UEListview.ColumnModeView.BandHilight := Attribs.Listview.ColumnHilightBanding;
      UEListview.ColumnModeView.BandHilightColor := Attribs.Listview.ColumnHilightBandColor;
    end;
    if Attribs.Mouse.RightButtonAssignment = 0 then
      LV.EnableGesture := False
    else
      UEListview.EnableGesture := True;
  end;     }

  if Attribs.Listview.ThumbsSingleFile then
    LV.ThumbsManager.StorageType := tasRepository
  else
    LV.ThumbsManager.StorageType := tasPerFolder;

  if not Attribs.Listview.SavePerFolder then
  begin
    LV.Options := LV.Options - [eloPerFolderStorage];
    LV.Storage.Clear;
  end else
    LV.Options := LV.Options + [eloPerFolderStorage];

  if Attribs.General.UseOnByDefaultFlag then
    LV.Options := LV.Options + [eloUseColumnOnByDefaultFlag]
  else
    LV.Options := LV.Options + [eloUseColumnOnByDefaultFlag];

  if Attribs.Listview.QueryInfoHints then
    LV.Options := LV.Options + [eloQueryInfoHints]
  else
    LV.Options := LV.Options - [eloQueryInfoHints];

  ApplyProperties;
end;

procedure TTntFrameBasePlugIn.OnListviewColumnContextMenu(Sender: TCustomEasyListview; HitInfo: TEasyHitInfoColumn; WindowPoint: TPoint; var Menu: TPopupMenu);
begin
  (Sender as TCustomVirtualExplorerEasyListviewHack).ColumnMenuItemCount := Attribs.Listview.ColumnsShownInHeaderMenu;
  (Sender as TCustomVirtualExplorerEasyListviewHack).ItemClass := TSpTBXItem;
  (Sender as TCustomVirtualExplorerEasyListviewHack).MenuClass := TSpTBXPopupMenu;
end;

procedure TTntFrameBasePlugIn.OnListviewClipboardChange(Sender: TObject);
begin
  if Assigned(Notifier) then
    Notifier.ViewClipboardChanged
end;

procedure TTntFrameBasePlugIn.OnListviewContextMenu(Sender: TCustomEasyListview; MousePt: TPoint; var Handled: Boolean);
//
// Called on the background Listview Menu
//
//var
//  NewMenuCommand: Boolean;
begin
    // Using the Virtual BkGnd Menu now............
{
  if (MousePt.X = -1) and (MousePt.X = -1) then
  begin
    MousePt.X := 0;
    MousePt.Y := 0;
  end;
  SyncListviewBkGndMenu;
  ViewBkGndMenu.Show(ActiveView, MousePt, Sender.Handle, ActiveRoot, NewMenuCommand);

  if NewMenuCommand then
    HandleShellNewMenuSelect;


  Handled := True;      }
end;

procedure TTntFrameBasePlugIn.OnListviewContextMenu2Message(Sender: TCustomVirtualExplorerEasyListview; var Msg: TMessage);
begin

end;

procedure TTntFrameBasePlugIn.OnListviewContextMenuAfterCmd(Sender: TCustomVirtualExplorerEasyListview; Namespace: TNamespace; Verb: WideString; MenuItemID: Integer; Successful: Boolean);
begin

end;

procedure TTntFrameBasePlugIn.OnListviewContextMenuItemChange(Sender: TCustomVirtualExplorerEasyListview; Namespace: TNamespace; MenuItemID: Integer; SubMenuID: hMenu; MouseSelect: Boolean);
begin

end;

procedure TTntFrameBasePlugIn.OnListviewDblClick(Sender: TCustomEasyListview; Button: TCommonMouseButton; MousePos: TPoint; ShiftState: TShiftState);
begin

end;

procedure TTntFrameBasePlugIn.OnListviewEnter(Sender: TObject);
begin

end;

procedure TTntFrameBasePlugIn.OnListviewEnumFinished(Sender: TCustomVirtualExplorerEasyListview);
begin

end;

procedure TTntFrameBasePlugIn.OnListviewEnumFolder(Sender: TCustomVirtualExplorerEasyListview; Namespace: TNamespace; var AllowAsChild: Boolean);
begin

end;

procedure TTntFrameBasePlugIn.OnListviewItemCustomView(Sender: TCustomEasyListview; Item: TEasyItem; ViewStyle: TEasyListStyle; var View: TEasyViewItemClass);
begin

end;

procedure TTntFrameBasePlugIn.OnListviewItemDblClick(Sender: TCustomEasyListview; Button: TCommonMouseButton; MousePos: TPoint; HitInfo: TEasyHitInfoItem);
begin

end;

procedure TTntFrameBasePlugIn.OnListviewItemEdited(Sender: TCustomEasyListview; Item: TEasyItem; var NewValue: Variant; var Accept: Boolean);
begin

end;

procedure TTntFrameBasePlugIn.OnListviewItemFocusChanged(Sender: TCustomEasyListview; Item: TEasyItem);
begin

end;

procedure TTntFrameBasePlugIn.OnListviewItemMouseDown(Sender: TCustomEasyListview; Item: TEasyItem; Button: TCommonMouseButton; var DoDefault: Boolean);
begin

end;

procedure TTntFrameBasePlugIn.OnListviewItemSelectionsChanged(Sender: TCustomEasyListview);
begin

end;

procedure TTntFrameBasePlugIn.OnListviewKeyAction(Sender: TCustomEasyListview; var CharCode: Word; var Shift: TShiftState; var DoDefault: Boolean);
begin

end;

procedure TTntFrameBasePlugIn.OnListviewLoadStorageFromRoot(Sender: TCustomVirtualExplorerEasyListview; NodeStorage: TNodeStorage);
begin

end;

procedure TTntFrameBasePlugIn.OnListviewMouseDown(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
begin

end;

procedure TTntFrameBasePlugIn.OnListviewMouseGesture(Sender: TCustomEasyListview; Gesture: WideString; var DoDefaultMouseAction: Boolean);
begin

end;

procedure TTntFrameBasePlugIn.OnListviewMouseUp(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
begin

end;

procedure TTntFrameBasePlugIn.OnListviewRootChange(Sender: TCustomVirtualExplorerEasyListview);
begin
  // Always enabled to keep associated tab up to date use the ConnectedEvents property in this event to determine actions
  RefreshOwnerTabUI
end;

procedure TTntFrameBasePlugIn.OnListviewRootChanging(Sender: TCustomVirtualExplorerEasyListview; const NewValue: TRootFolder; const CurrentNamespace, Namespace: TNamespace; var Allow: Boolean);
begin

end;

procedure TTntFrameBasePlugIn.OnListviewSaveRootToStorage(Sender: TCustomVirtualExplorerEasyListview; NodeStorage: TNodeStorage);
begin

end;

procedure TTntFrameBasePlugIn.OnListviewScroll(Sender: TCustomEasyListview; DeltaX, DeltaY: Integer);
begin

end;

procedure TTntFrameBasePlugIn.OnListviewSetView(Sender: TCustomEasyListview);
begin

end;

procedure TTntFrameBasePlugIn.OnListviewShellExecute(Sender: TCustomVirtualExplorerEasyListview; Namespace: TNamespace; var WorkingDir: WideString; var CmdLineArgument: WideString; var Allow: Boolean);
begin

end;

procedure TTntFrameBasePlugIn.OnListviewContextMenuCmd(Sender: TCustomVirtualExplorerEasyListview; Namespace: TNamespace; Verb: WideString; MenuItemID: Integer; var Handled: Boolean);
begin

end;

function TTntFrameBasePlugIn.TabImage(var AnImageList: TCustomImageList;
  var AnImageIndex: Integer): boolean;
begin
  Result := False;
end;

function TTntFrameBasePlugIn.QueryInterface(const IID: TGUID;
  out Obj): HResult;
begin
  if GetInterface(IID, Obj) then Result := S_OK
    else Result := E_NOINTERFACE
end;

procedure TTntFrameBasePlugIn.OnListviewShellNotify(Sender: TCustomVirtualExplorerEasyListview; ShellEvent: TVirtualShellEvent);
begin
  // Always enabled to keep associated tab up to date use the ConnectedEvents property in this event to determine actions
  RefreshOwnerTabUI
end;

procedure TTntFrameBasePlugIn.RefreshOwnerTabUI;
begin
  if Assigned(Notifier) and Assigned(OwnerTabItem) then
    Notifier.UpdateUI(nuiTabItem, Integer(OwnerTabItem))
end;

procedure TTntFrameBasePlugIn.SetOwnerItem(ATabItem: TSpTBXTabItem);
begin
  OwnerTabItem := ATabItem
end;

procedure TTntFrameBasePlugIn.ViewBeginUpdate;
begin

end;

procedure TTntFrameBasePlugIn.ViewEndUpdate;
begin

end;

procedure TTntFrameBasePlugIn.ViewSelectionChangeLock(DoLock: Boolean);
begin

end;

procedure TTntFrameBasePlugIn.ViewSetRootPIDL(PIDL: PItemIDList);
begin
  // Expects the caller to free the PIDL, make a copy if needed for later
end;

procedure TTntFrameBasePlugIn.WMEasyThreadCallback(var Msg: TWMThreadRequest);
begin
  DoGlobalThreadCallback(Msg);
end;

end.
