unit UnitPlugInListviewFrame;

interface

uses
  Windows,
  Messages,
  SysUtils,
  Variants,
  Classes,
  Graphics,
  Controls,
  Forms,
  TntForms,
  ShlObj,
  Dialogs,
  ImgList,
  StdCtrls,
  ClipboardProc,
  EasyListview,
  MPCommonObjects,
  MPCommonUtilities,
  MPShellUtilities,
  MPThreadManager,
  SpTBXControls,
  SpTBXDkPanels,
  SpTBXEditors,
  SpTBXItem,
  TntClasses,
  TntStdCtrls,
  UltraFavTree,
  UltraUserStrings,
  UltraUtilities,
  UltraOptions,
  UnitPlugInBaseFrame,
  UnitInPlaceWndInf,
  VirtualShellNotifier,
  VirtualResources,
  VirtualExplorerEasyListview;

type
  TTntFrameBasePlugInListview = class(TTntFrameBasePlugIn)
    VirtualExplorerEasyListview: TVirtualExplorerEasyListview;
    function VirtualExplorerEasyListviewItemCompare(Sender: TCustomEasyListview; Column: TEasyColumn; Group: TEasyGroup; Item1, Item2: TEasyItem; var DoDefault: Boolean): Integer;
    procedure VirtualExplorerEasyListviewEnumFinished(Sender: TCustomVirtualExplorerEasyListview);
    procedure VirtualExplorerEasyListviewItemGetCaption(Sender: TCustomEasyListview; Item: TEasyItem; Column: Integer; var Caption: WideString);
    procedure VirtualExplorerEasyListviewItemCustomView(Sender: TCustomEasyListview; Item: TEasyItem; ViewStyle: TEasyListStyle; var View: TEasyViewItemClass);
    procedure VirtualExplorerEasyListviewExplorerItemClass(
      Sender: TCustomVirtualExplorerEasyListview;
      var ExplorerClass: TExplorerItemClass);
  private
   { Private declarations }
    FFolderSizeMax: Int64;
    procedure SetFolderSizeMax(const Value: Int64);
  protected
   { Protected declarations }
    function ActionCalculateFolderSizes(SelectedOnly: Boolean; Force: Boolean): Boolean; override;
    function ActionClearFolderSizes: Boolean; override;
    function ActionCopyPath(Info: PViewCopyPathAction): Boolean; override;
    function ActionCopyToClipboard: Boolean; override;
    function ActionCreateNewFolder: Boolean; override;
    function ActionCutToClipboard: Boolean; override;
    function ActionInvalidateAndRedraw: Boolean; override;
    function ActionPasteAsShortCutFromClipboard: Boolean; override;
    function ActionPasteFromClipboard: Boolean; override;
    function ActionQuickFilter(Info: PViewQuickFilterRecAction): Boolean; override;
    function ActionRebuild: Boolean; override;
    function ActionSelectAll(Info: PViewSelectRecAction): Boolean; override;
    function ActionSelectAllFiles(Info: PViewSelectRecAction): Boolean; override;
    function ActionSelectAllFolders(Info: PViewSelectRecAction): Boolean; override;
    function ActionSelectedDelete(Info: PViewSelectRecAction): Boolean; override;
    function ActionSelectedInvert(Info: PViewSelectRecAction): Boolean; override;
    function ActionSelectMatchText(Info: PViewSelectRecAction): Boolean; override;
    function ActionUnSelectAllFiles(Info: PViewSelectRecAction): Boolean; override;
    function ActionUnSelectAllFolders(Info: PViewSelectRecAction): Boolean; override;
    function ActionUnSelectMatchText(Info: PViewSelectRecAction): Boolean; override;
    procedure DisconnectViewEventsAndBkGndMenu; override;
    procedure DoGlobalThreadCallback(Msg: TWMThreadRequest); override;
    procedure OnListviewExplorerClass(Sender: TCustomVirtualExplorerEasyListview; var ExplorerClass: TExplorerItemClass);
    procedure OnListviewRootChange(Sender: TCustomVirtualExplorerEasyListview); override;
  public
    { Public declarations }
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    // IUltraInPlaceWindow
    function ActiveViewUI(DoActivate: Boolean): Boolean; override;
    function GetObject(var APlugin: TTntFrame): Boolean; override;
    function HintMsg(var AHint: WideString): Boolean; override;
    function LoadConfig(StreamVer: Integer; ConfigStream: TTntFileStream): Boolean; override;
    function Persistent: Boolean; override;
    function SaveConfig(StreamVer: Integer; ConfigStream: TTntFileStream): Boolean; override;
    function TabCaption(var ACaption: WideString): Boolean; override;
    function TabImage(var AnImageList: TCustomImageList; var AnImageIndex: Integer): Boolean; override;
    function ViewRootPIDL: PItemIDList; override;
    procedure ConnectEventsAndBkGndMenu(DoConnect, DoConnectBkGndMenu: Boolean); override;
    procedure ViewBeginUpdate; override;
    procedure ViewEndUpdate; override;
    function ViewItemCount: Integer; override;
    function ViewItemFirst: TExplorerItem; override;
    function ViewItemNext(Item: TExplorerItem): TExplorerItem; override;
    function ViewSelectionCount: Integer; override;
    function ViewSelectionFirst: TExplorerItem; override;
    procedure ViewSelectionChangeLock(DoLock: Boolean); override;
    function ViewSelectionNext(Item: TExplorerItem): TExplorerItem; override;
    function ViewShellFolder: IShellFolder; override;
    function ViewTopItem: TExplorerItem; override;
    procedure ViewSetRootPIDL(PIDL: PItemIDList); override;

    property FolderSizeMax: Int64 read FFolderSizeMax write SetFolderSizeMax;
  end;

implementation

{$R *.dfm}

{ TTntFrameBasePlugInListview } 
constructor TTntFrameBasePlugInListview.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);

  // Notice here we can set all these properties and not fire events since they are not set yet
  LoadDefaultListviewProperties(VirtualExplorerEasyListview);
  LoadUserListviewProperties(VirtualExplorerEasyListview);
  LoadDefaultViewType(VirtualExplorerEasyListview);

  // Always enabled to keep associated tab up to date use the ConnectedEvents property in this event to determine actions
  VirtualExplorerEasyListview.OnRootChange := OnListviewRootChange;
  VirtualExplorerEasyListview.OnShellNotify := OnListviewShellNotify;
  ChangeNotifier.RegisterKernelChangeNotify(Self, AllKernelNotifiers);
  ChangeNotifier.RegisterKernelSpecialFolderWatch(CSIDL_BITBUCKET);
  ChangeNotifier.RegisterKernelSpecialFolderWatch(CSIDL_DESKTOP);
  ChangeNotifier.RegisterKernelSpecialFolderWatch(CSIDL_DRIVES);
end;

destructor TTntFrameBasePlugInListview.Destroy;
begin
  ChangeNotifier.UnRegisterKernelChangeNotify(Self);
  inherited Destroy;
end;

function TTntFrameBasePlugInListview.ActionCalculateFolderSizes(SelectedOnly: Boolean; Force: Boolean): Boolean;
var
  ThreadRequest: TUltraFolderSizeThreadRequest;
  UltraItem: TUltraExplorerItem;
  Allow: Boolean;
begin
  if Attribs.Listview.FolderSizeShow or Force then
  begin
    UltraItem := TUltraExplorerItem(VirtualExplorerEasyListview.Groups.FirstItem);
    while Assigned(UltraItem) do
    begin
      if UltraItem.Namespace.Folder and not UltraItem.Namespace.Browsable and UltraItem.Namespace.Parent.FileSystem then
      begin
        Allow := True;
        if SelectedOnly then
          Allow := UltraItem.Selected;
        if Allow then
        begin
          ThreadRequest := TUltraFolderSizeThreadRequest.Create;
          ThreadRequest.PIDL := PIDLMgr.CopyPIDL(UltraItem.Namespace.AbsolutePIDL);
          ThreadRequest.Window := Self;
          ThreadRequest.Item := UltraItem;
          ThreadRequest.Priority := 100;
          ThreadRequest.ID := TID_FOLDERSIZE;
          TUltraExplorerItem(UltraItem).UltraStates := UltraItem.UltraStates - [uisThreadLoaded] + [uisThreadLoading];
          GlobalThreadManager.AddRequest(ThreadRequest, True);
        end
      end;
      UltraItem := TUltraExplorerItem(VirtualExplorerEasyListview.Groups.NextItem(UltraItem));
    end;
    FolderSizeMax := 0;
  end;
  Result := True;
end;

function TTntFrameBasePlugInListview.ActionClearFolderSizes: Boolean;
var
  Item: TUltraExplorerItem;
begin
  Item := TUltraExplorerItem(VirtualExplorerEasyListview.Groups.FirstItem);
  while Assigned(Item) do
  begin
    Item.FolderSize := -1;
    Item := TUltraExplorerItem(VirtualExplorerEasyListview.Groups.NextItem(Item))
  end;
  FFolderSizeMax := -1;
  Result := True
end;

function TTntFrameBasePlugInListview.ActionCopyPath(Info: PViewCopyPathAction): Boolean;
var
  Files: TTntStringList;
  Item: TExplorerItem;
begin
  Files := TTntStringList.Create;
  try
    Item := TExplorerItem( VirtualExplorerEasyListview.Selection.First);
    while Assigned(Item) do
    begin
      if Info.DOSPath then
      begin
        if Info.FileNamesOnly then
          Files.Add(WideExtractFileName(ShortFileName( Item.Namespace.NameForParsing)))
        else
          Files.Add( ShortFileName( Item.Namespace.NameForParsing));
      end else
      begin
        if Info.FileNamesOnly then
          Files.Add(WideExtractFileName( Item.Namespace.NameForParsing))
        else
          Files.Add(Item.Namespace.NameForParsing);
      end;
      Item := TExplorerItem(VirtualExplorerEasyListview.Selection.Next(Item));
    end;
    SCopyToClipboardW(Files.Text);
  finally
    Files.Free
  end;
  Result := True;
end;

function TTntFrameBasePlugInListview.ActionInvalidateAndRedraw: Boolean;
begin
  VirtualExplorerEasyListview.Invalidate;
  VirtualExplorerEasyListview.Update;
  Result := True;
end;

function TTntFrameBasePlugInListview.ActionPasteAsShortCutFromClipboard: Boolean;
begin
  VirtualExplorerEasyListview.PasteFromClipboardAsShortcut;
  Result := True;
end;

function TTntFrameBasePlugInListview.ActionQuickFilter(Info: PViewQuickFilterRecAction): Boolean;
begin
  if Info.Enable then
  begin
    VirtualExplorerEasyListview.QuickFilterMask := Info.Mask;
    VirtualExplorerEasyListview.QuickFiltered := True;
    VirtualExplorerEasyListview.BackGround.Enabled := Info.ShowImage;
    VirtualExplorerEasyListview.BackGround.OffsetX := VirtualExplorerEasyListview.ClientWidth - 120;
    VirtualExplorerEasyListview.BackGround.OffsetX := VirtualExplorerEasyListview.ClientWidth - 120;
  end else
  begin
    VirtualExplorerEasyListview.QuickFiltered := Info.Enable;
    VirtualExplorerEasyListview.BackGround.Enabled := False
  end;
  Result := True;
end;

function TTntFrameBasePlugInListview.ActionRebuild: Boolean;
begin
  VirtualExplorerEasyListview.Rebuild(True);
  Result := True;
end;

function TTntFrameBasePlugInListview.ActionSelectAll(Info: PViewSelectRecAction): Boolean;
begin
  VirtualExplorerEasyListview.Selection.SelectAll;
  if VirtualExplorerEasyListview.CanFocus and Info.SetFocusToView then
    VirtualExplorerEasyListview.SetFocus;
  Result := True;
end;

function TTntFrameBasePlugInListview.ActionSelectAllFiles(Info: PViewSelectRecAction): Boolean;
var
  Item: TExplorerItem;
begin
  VirtualExplorerEasyListview.BeginUpdate;
  VirtualExplorerEasyListview.Selection.IncMultiChangeCount;
  try
    if not Info.Combine then
      VirtualExplorerEasyListview.Selection.ClearAll;

    Item := TExplorerItem( VirtualExplorerEasyListview.Groups.FirstItem);
    while Assigned(Item) do
    begin
      if not Item.Namespace.Folder then
        Item.Selected := True;
      Item := TExplorerItem( VirtualExplorerEasyListview.Groups.NextItem(Item))
    end
  finally
    VirtualExplorerEasyListview.Selection.DecMultiChangeCount;
    VirtualExplorerEasyListview.EndUpdate(True)
  end;
  if VirtualExplorerEasyListview.CanFocus and Info.SetFocusToView then
    VirtualExplorerEasyListview.SetFocus;
  Result := True;
end;

function TTntFrameBasePlugInListview.ActionSelectAllFolders(Info: PViewSelectRecAction): Boolean;
var
  Item: TExplorerItem;
begin
  VirtualExplorerEasyListview.BeginUpdate;
  VirtualExplorerEasyListview.Selection.IncMultiChangeCount;
  try
    if not Info.Combine then
      VirtualExplorerEasyListview.Selection.ClearAll;

    Item := TExplorerItem( VirtualExplorerEasyListview.Groups.FirstItem);
    while Assigned(Item) do
    begin
      if Item.Namespace.Folder then
        Item.Selected := True;
      Item := TExplorerItem( VirtualExplorerEasyListview.Groups.NextItem(Item))
    end
  finally
    VirtualExplorerEasyListview.Selection.DecMultiChangeCount;
    VirtualExplorerEasyListview.EndUpdate(True)
  end;
  if VirtualExplorerEasyListview.CanFocus and Info.SetFocusToView then
    VirtualExplorerEasyListview.SetFocus;
  Result := True;
end;

function TTntFrameBasePlugInListview.ActionSelectedDelete(Info: PViewSelectRecAction): Boolean;
begin
  VirtualExplorerEasyListview.SelectedFilesDelete;
  if VirtualExplorerEasyListview.CanFocus and Info.SetFocusToView then
    VirtualExplorerEasyListview.SetFocus;
  Result := True;
end;

function TTntFrameBasePlugInListview.ActionSelectedInvert(Info: PViewSelectRecAction): Boolean;
begin
  VirtualExplorerEasyListview.Selection.Invert;
  if VirtualExplorerEasyListview.CanFocus and Info.SetFocusToView then
    VirtualExplorerEasyListview.SetFocus;
  Result := True;
end;

function TTntFrameBasePlugInListview.ActionSelectMatchText(Info: PViewSelectRecAction): Boolean;
var
  Item: TExplorerItem;
begin
   VirtualExplorerEasyListview.BeginUpdate;
   VirtualExplorerEasyListview.Selection.IncMultiChangeCount;
  try
    Item := TExplorerItem(  VirtualExplorerEasyListview.Groups.FirstItem);
    while Assigned(Item) do
    begin
      // implement some text matching here
      Item := TExplorerItem(  VirtualExplorerEasyListview.Groups.NextItem(Item))
    end
  finally
     VirtualExplorerEasyListview.Selection.DecMultiChangeCount;
     VirtualExplorerEasyListview.EndUpdate(True)
  end;
  if VirtualExplorerEasyListview.CanFocus and Info.SetFocusToView then
    VirtualExplorerEasyListview.SetFocus;
  Result := True
end;

function TTntFrameBasePlugInListview.ActionUnSelectAllFiles(Info: PViewSelectRecAction): Boolean;
var
  Item: TExplorerItem;
begin
  VirtualExplorerEasyListview.BeginUpdate;
  VirtualExplorerEasyListview.Selection.IncMultiChangeCount;
  try
    Item := TExplorerItem( VirtualExplorerEasyListview.Groups.FirstItem);
    while Assigned(Item) do
    begin
      if not Item.Namespace.Folder then
        Item.Selected := False;
      Item := TExplorerItem( VirtualExplorerEasyListview.Groups.NextItem(Item))
    end
  finally
    VirtualExplorerEasyListview.Selection.DecMultiChangeCount;
    VirtualExplorerEasyListview.EndUpdate(True)
  end;
  if VirtualExplorerEasyListview.CanFocus and Info.SetFocusToView then
    VirtualExplorerEasyListview.SetFocus;
  Result := True;
end;

function TTntFrameBasePlugInListview.ActionUnSelectAllFolders(Info: PViewSelectRecAction): Boolean;
var
  Item: TExplorerItem;
begin
  VirtualExplorerEasyListview.BeginUpdate;
  VirtualExplorerEasyListview.Selection.IncMultiChangeCount;
  try
    Item := TExplorerItem( VirtualExplorerEasyListview.Groups.FirstItem);
    while Assigned(Item) do
    begin
      if Item.Namespace.Folder then
        Item.Selected := False;
      Item := TExplorerItem( VirtualExplorerEasyListview.Groups.NextItem(Item))
    end
  finally
    VirtualExplorerEasyListview.Selection.DecMultiChangeCount;
    VirtualExplorerEasyListview.EndUpdate(True)
  end;
  if VirtualExplorerEasyListview.CanFocus and Info.SetFocusToView then
    VirtualExplorerEasyListview.SetFocus;
  Result := True;
end;

function TTntFrameBasePlugInListview.ActionUnSelectMatchText(Info: PViewSelectRecAction): Boolean;
var
  Item: TExplorerItem;
begin
   VirtualExplorerEasyListview.BeginUpdate;
   VirtualExplorerEasyListview.Selection.IncMultiChangeCount;
  try
    Item := TExplorerItem(  VirtualExplorerEasyListview.Groups.FirstItem);
    while Assigned(Item) do
    begin
      // implement some text matching here
      Item := TExplorerItem(  VirtualExplorerEasyListview.Groups.NextItem(Item))
    end
  finally
     VirtualExplorerEasyListview.Selection.DecMultiChangeCount;
     VirtualExplorerEasyListview.EndUpdate(True)
  end;
  if VirtualExplorerEasyListview.CanFocus and Info.SetFocusToView then
    VirtualExplorerEasyListview.SetFocus;
  Result := True
end;

function TTntFrameBasePlugInListview.ActiveViewUI(DoActivate: Boolean): Boolean;
begin
  Result := True;
  VirtualExplorerEasyListview.Active := DoActivate
end;

function TTntFrameBasePlugInListview.GetObject(var APlugin: TTntFrame): Boolean;
begin
  APlugin := Self;
  Result := True
end;

function TTntFrameBasePlugInListview.HintMsg(var AHint: WideString): Boolean;
begin
  Result := True;
  AHint := VirtualExplorerEasyListview.RootFolderNamespace.NameAddressbar;
end;

function TTntFrameBasePlugInListview.LoadConfig(StreamVer: Integer; ConfigStream: TTntFileStream): Boolean;
var
  PIDL: PItemIDList;
begin
  Result := True;
  PIDL := PIDLMgr.LoadFromStream(ConfigStream);
  VirtualExplorerEasyListview.RootFolderCustomPIDL := PIDL;
  PIDLMgr.FreePIDL(PIDL);
end;

function TTntFrameBasePlugInListview.Persistent: Boolean;
begin
  Result := True
end;

function TTntFrameBasePlugInListview.SaveConfig(StreamVer: Integer; ConfigStream: TTntFileStream): Boolean;
begin
  Result := True;
  PIDLMgr.SaveToStream(ConfigStream, VirtualExplorerEasyListview.RootFolderNamespace.AbsolutePIDL);
end;

function TTntFrameBasePlugInListview.TabCaption(var ACaption: WideString): Boolean;
begin
  Result := True;
  ACaption := VirtualExplorerEasyListview.RootFolderNamespace.NameInFolder;
end;

function TTntFrameBasePlugInListview.TabImage(var AnImageList: TCustomImageList; var AnImageIndex: Integer): Boolean;
begin
  Result := True;
  AnImageIndex := VirtualExplorerEasyListview.RootFolderNamespace.GetIconIndex(False, icSmall);
  AnImageList := SmallSysImages;
end;

function TTntFrameBasePlugInListview.ViewItemCount: Integer;
begin
  Result := VirtualExplorerEasyListview.Groups.ItemCount
end;

function TTntFrameBasePlugInListview.ViewItemFirst: TExplorerItem;
begin
  Result := TExplorerItem( VirtualExplorerEasyListview.Groups.FirstItem)
end;

function TTntFrameBasePlugInListview.ViewItemNext(Item: TExplorerItem): TExplorerItem;
begin
  Result := TExplorerItem( VirtualExplorerEasyListview.Groups.NextItem(Item))
end;

function TTntFrameBasePlugInListview.ViewRootPIDL: PItemIDList;
begin
  Result := VirtualExplorerEasyListview.RootFolderNamespace.AbsolutePIDL
end;

function TTntFrameBasePlugInListview.ViewSelectionCount: Integer;
begin
  Result := VirtualExplorerEasyListview.Selection.Count
end;

function TTntFrameBasePlugInListview.ViewSelectionFirst: TExplorerItem;
begin
  Result := TExplorerItem( VirtualExplorerEasyListview.Selection.First)
end;

function TTntFrameBasePlugInListview.ViewSelectionNext(Item: TExplorerItem): TExplorerItem;
begin
  Result := TExplorerItem( VirtualExplorerEasyListview.Selection.Next(Item))
end;

function TTntFrameBasePlugInListview.ViewShellFolder: IShellFolder;
begin
  Result := VirtualExplorerEasyListview.RootFolderNamespace.ShellFolder
end;

function TTntFrameBasePlugInListview.ViewTopItem: TExplorerItem;
begin
  Result := TExplorerItem( VirtualExplorerEasyListview.TopItem)
end;

function TTntFrameBasePlugInListview.ActionCopyToClipboard: Boolean;
begin
  VirtualExplorerEasyListview.CopyToClipboard;
  Result := True;
end;

function TTntFrameBasePlugInListview.ActionCreateNewFolder: Boolean;
var
  NewPath: WideString;
  NS: TNamespace;
  Item: TExplorerItem;
begin
  Result := False;
  NewPath := UniqueDirName(VirtualExplorerEasyListview.RootFolderNamespace.NameForParsing + '\' + S_NEW + ' ' + S_FOLDER);
  WideCreateDir(NewPath);
  if WideDirectoryExists(NewPath) then
  begin
    Item := VirtualExplorerEasyListview.FindItemByPath(NewPath) as TExplorerItem;
    // Make sure we can get to the desired folder so we can select the new item
    if not Assigned(Item) then
    begin
      NS := TNamespace.Create(PathToPIDL(NewPath), nil);
      Item := VirtualExplorerEasyListview.AddCustomItem(nil, NS, True);
    end;
    if Assigned(Item) then
    begin
      VirtualExplorerEasyListview.Selection.ClearAll;
      Item.MakeVisible(emvAuto);
      Item.Focused := True;
      Item.Selected := True;
      Item.Edit;
      Result := True
    end
  end;

end;

function TTntFrameBasePlugInListview.ActionCutToClipboard: Boolean;
begin
  VirtualExplorerEasyListview.CutToClipboard;
  Result := True;
end;

function TTntFrameBasePlugInListview.ActionPasteFromClipboard: Boolean;
begin
  VirtualExplorerEasyListview.PasteFromClipboard;
  Result := True;
end;

procedure TTntFrameBasePlugInListview.ConnectEventsAndBkGndMenu(DoConnect, DoConnectBkGndMenu: Boolean);
begin
  inherited;
  LoadDefaultListviewEventsAndConnectBkGndMenu(VirtualExplorerEasyListview, DoConnectBkGndMenu, DoConnect)
end;

procedure TTntFrameBasePlugInListview.DisconnectViewEventsAndBkGndMenu;
begin
  // Don't allow any event to be fired while we are begin destroyed
  LoadDefaultListviewEventsAndConnectBkGndMenu(VirtualExplorerEasyListview, False, True);
end;

procedure TTntFrameBasePlugInListview.DoGlobalThreadCallback(Msg: TWMThreadRequest);
var
  NS: TNamespace;
  AnItem: TUltraExplorerItem;
  FolderSizeRequest: TUltraFolderSizeThreadRequest;
begin
  AnItem := TUltraExplorerItem( Msg.Request.Item);
  case Msg.RequestID of
    TID_FOLDERSIZE:
      begin
        if VirtualExplorerEasyListview.ValidateNamespace(AnItem, NS) then
        begin
          FolderSizeRequest := Msg.Request as TUltraFolderSizeThreadRequest;
          if FolderSizeRequest.FolderSize > FolderSizeMax then
            FolderSizeMax := FolderSizeRequest.FolderSize;  
          AnItem.FolderSize := FolderSizeRequest.FolderSize;
          AnItem.UltraStates := AnItem.UltraStates + [uisThreadLoaded] - [uisThreadLoading];
          VirtualExplorerEasyListview.Groups.InvalidateItem(AnItem, False);
        end
      end
  end;
  Msg.Request.Release;
end;

procedure TTntFrameBasePlugInListview.OnListviewExplorerClass(Sender: TCustomVirtualExplorerEasyListview; var ExplorerClass: TExplorerItemClass);
begin
  ExplorerClass := TUltraExplorerItem
end;

procedure TTntFrameBasePlugInListview.OnListviewRootChange(Sender: TCustomVirtualExplorerEasyListview);
begin
  inherited OnListviewRootChange(Sender);
  ChangeNotifier.NotifyWatchFolder(Self, VirtualExplorerEasyListview.RootFolderNamespace.NameForParsing);
end;

procedure TTntFrameBasePlugInListview.SetFolderSizeMax(const Value: Int64);
begin
  if Value <> FFolderSizeMax then
  begin
    FFolderSizeMax := Value;
    if HandleAllocated then
      InvalidateRect(Handle, nil, False)
  end
end;

procedure TTntFrameBasePlugInListview.ViewBeginUpdate;
begin
  VirtualExplorerEasyListview.BeginUpdate
end;

procedure TTntFrameBasePlugInListview.ViewEndUpdate;
begin
  VirtualExplorerEasyListview.EndUpdate
end;

procedure TTntFrameBasePlugInListview.ViewSelectionChangeLock(DoLock: Boolean);
begin
  if DoLock then
    VirtualExplorerEasyListview.Selection.IncMultiChangeCount
  else
   VirtualExplorerEasyListview.Selection.DecMultiChangeCount
end;

procedure TTntFrameBasePlugInListview.ViewSetRootPIDL(PIDL: PItemIDList);
begin
  VirtualExplorerEasyListview.RootFolderCustomPIDL := PIDL
end;

function TTntFrameBasePlugInListview.VirtualExplorerEasyListviewItemCompare(
  Sender: TCustomEasyListview; Column: TEasyColumn; Group: TEasyGroup;
  Item1, Item2: TEasyItem; var DoDefault: Boolean): Integer;
var
  ColumnIndex: Integer;
  IsFolder1, IsFolder2: Boolean;
begin
  Result := 0;
  DoDefault := True;
  ColumnIndex := Column.Index;
  if (ColumnIndex = ID_SIZE_COLUMN) then
  begin
    DoDefault := False;
    IsFolder1 := TExplorerItem(Item1).Namespace.Folder and not TExplorerItem(Item1).Namespace.Browsable;
    IsFolder2 := TExplorerItem(Item2).Namespace.Folder and not TExplorerItem(Item2).Namespace.Browsable;

    Result := 0;
    if VirtualExplorerEasyListview.SortFolderFirstAlways then
    begin
      if IsFolder1 xor IsFolder2 then
      begin
        if IsFolder1 then
          Result := -1
        else
          Result := 1;
      end
    end;

    if Result = 0 then
    begin
      if IsFolder1 and IsFolder2 then
      begin
        if TUltraExplorerItem( Item2).FolderSize = TUltraExplorerItem( Item1).FolderSize then
          Result := 0
        else
        if TUltraExplorerItem( Item2).FolderSize > TUltraExplorerItem( Item1).FolderSize then
          Result := -1
        else
          Result := 1
      end else
      begin
        if TUltraExplorerItem( Item2).Namespace.SizeOfFileInt64 = TUltraExplorerItem( Item1).Namespace.SizeOfFileInt64 then
          Result := 0
        else
        if TUltraExplorerItem( Item2).Namespace.SizeOfFileInt64 >  TUltraExplorerItem( Item1).Namespace.SizeOfFileInt64 then
          Result := -1
        else
          Result := 1
      end;
      // Secondary level of sorting is on the Name
      if (Result = 0) and (ColumnIndex > 0) then
        Result := TExplorerItem(Item2).Namespace.ComparePIDL(TExplorerItem(Item1).Namespace.RelativePIDL, False, 0);
      if (ColumnIndex > -1) and (Column.SortDirection = esdDescending) then
        Result := -Result
    end
  end;
end;

procedure TTntFrameBasePlugInListview.VirtualExplorerEasyListviewEnumFinished(
  Sender: TCustomVirtualExplorerEasyListview);
begin
  // Reload the Folder Sizes if needed
  ActionCalculateFolderSizes(False, False);
end;

procedure TTntFrameBasePlugInListview.VirtualExplorerEasyListviewItemGetCaption(
  Sender: TCustomEasyListview; Item: TEasyItem; Column: Integer;
  var Caption: WideString);
var
  UltraItem: TUltraExplorerItem;
begin
  UltraItem := TUltraExplorerItem( Item);
  if (Column = ID_SIZE_COLUMN) and UltraItem.Namespace.Folder and not UltraItem.Namespace.Browsable and UltraItem.Namespace.Parent.FileSystem then
  begin
    if UltraItem.Namespace.FileSystem and UltraItem.Namespace.Folder and not UltraItem.Namespace.Browsable then
    begin
      if (uisThreadLoaded in TUltraExplorerItem( Item).UltraStates) then
      begin
        case Attribs.Listview.FolderSizeDisplay of
          bsKiloBytes:
            begin
              Caption := Format('%0.0n '+ 'KB', [UltraItem.FolderSize / 1024]);
              if (Caption = '0 KB') and (UltraItem.FolderSize > 0) then
                Caption := '1 KB';
              Caption := Caption + '  '
            end;
          bsMegaBytes:
            begin
              Caption := Format('%0.0n '+ 'MB', [UltraItem.FolderSize / 1048576]);
              if (Caption = '0 MB') and (UltraItem.FolderSize > 0) then
                Caption := '1 MB';
              Caption := Caption + '  '
            end;
          bsGigiBytes:
            begin
              Caption := Format('%0.1n '+ 'GB', [UltraItem.FolderSize / 1073741824]);
              if (Caption = '0 GB') and (UltraItem.FolderSize > 0) then
                Caption := '1 GB';
              Caption := Caption + '  '
            end;
          bsTereBytes:
            begin
              Caption := Format('%0.1n '+ 'TB', [UltraItem.FolderSize / 1099511627776]);
              if (Caption = '0 TB') and (UltraItem.FolderSize > 0) then
                Caption := '1 TB';
              Caption := Caption + '  '
            end
        end;
      end else
      begin
        if (uisThreadLoading in TUltraExplorerItem( Item).UltraStates) then
          Caption := STR_LOADING + '  '
      end
    end
  end
end;

procedure TTntFrameBasePlugInListview.VirtualExplorerEasyListviewItemCustomView(
  Sender: TCustomEasyListview; Item: TEasyItem; ViewStyle: TEasyListStyle;
  var View: TEasyViewItemClass);
begin
  if (ViewStyle = elsReport) then
    View :=  TUltraViewReportItem
end;

procedure TTntFrameBasePlugInListview.VirtualExplorerEasyListviewExplorerItemClass(
  Sender: TCustomVirtualExplorerEasyListview;
  var ExplorerClass: TExplorerItemClass);
begin
  ExplorerClass := TUltraExplorerItem    
end;

initialization
  RegisterClass(TTntFrameBasePlugInListview);

finalization

end.

