unit UnitSearchPluginForm;

// The contents of this file are subject to the Mozilla Public License
// Version 1.1 (the "License"); you maynot use this file except in compliance
// with the License. You may obtain a copy of the License at http://www.mozilla.org/MPL/
//
// Alternatively, you may redistribute this library, use and/or modify it under the terms of the
// GNU Lesser General Public License as published by the Free Software Foundation;
// either version 2.1 of the License, or (at your option) any later version.
// You may obtain a copy of the LGPL at http://www.gnu.org/copyleft/.
//
// Software distributed under the License is distributed on an "AS IS" basis,
// WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for the
// specific language governing rights and limitations under the License.

interface

uses
//  ATStreamSearch,
  Classes,
  Controls,
  Dialogs,
  DKLang,
  EasyListview,
  EasyTaskPanelForm,
  ExtCtrls,
  Forms,
  Graphics,
  ImgList,
  Messages,
  Menus,
  UnitBasePlugInForm,
  MPCommonObjects,
  MPCommonUtilities,
  MPShellUtilities,
  MPThreadManager,
  ShlObj,
  SpTBXControls,
  SpTBXEditors,
  StdCtrls,
  SysUtils,
  SpTBXDkPanels,
  TntClasses,
  TntForms,
  TntSysUtils,
  UltraUserStrings,
  UltraOptions,
  UltraUtilities,
  UnitInPlaceWndInf,
  UnitSearchAdvOptions,
  UnitSearchBrowseTaskPanel,
  UnitSearchMainTaskPanel,
  UnitSearchModifedTaskPanel,
  UnitSearchSizeTaskPanel,
  Variants,
  VirtualExplorerEasyListview,
  VirtualExplorerTree,
  VirtualFileSearch,
  VirtualResources,
  VirtualTrees,
  Windows, SpTBXItem;



type
  TTntFormUltraSearch = class(TBasePluginForm)
    DKLanguageController: TDKLanguageController;
    EasyTaskPanelBandSearch: TEasyTaskPanelBand;
    ImageList: TImageList;
    SpTBXButtonSearch: TSpTBXButton;
    SpTBXPanelBkGnd: TSpTBXPanel;
    SpTBXSplitterMain: TSpTBXSplitter;

    procedure EasyTaskPanelBandSearchGetTaskPanel(Sender: TCustomEasyListview; Group: TEasyGroup; var TaskPanel: TEasyTaskPanelFormClass);
    procedure EasyTaskPanelBandSearchResize(Sender: TObject);
    procedure SpTBXButtonSearchClick(Sender: TObject);
    procedure SpTBXEditContentEnter(Sender: TObject);
    procedure SpTBXEditFileNameEnter(Sender: TObject);
    procedure TntFormClose(Sender: TObject; var Action: TCloseAction);
    procedure TntFormCreate(Sender: TObject);
    procedure TntFormDestroy(Sender: TObject);
    procedure TntFormShow(Sender: TObject);
    procedure UltraFileSearchCompare(Sender: TObject; const FilePath: WideString; FindFileData: _WIN32_FIND_DATAW; var UseFile: Boolean);
    procedure UltraFileSearchEnd(Sender: TObject; Results: TCommonPIDLList);
    procedure UltraFileSearchProgress(Sender: TObject; Results: TCommonPIDLList; var Handled: Boolean);
    procedure DKLanguageControllerLanguageChanged(Sender: TObject);

  private
    FRootFolder: PItemIDList;
    FRunning: Boolean;
    FSearchThread: TUltraFileSearch;
    FData: TUltraSearchAttribs;
    FVEELVSearch: TVirtualExplorerEasyListview;

    function GetSearchAdvOptionsForm: TEasyTaskPanelFormAdvOptions;
    function GetSearchBrowseForm: TEasyTaskPanelFormBrowse;
    function GetSearchContentForm: TEasyTaskPanelFormMainSearch;
    function GetSearchDateForm: TEasyTaskPanelFormSearchDate;
    function GetSearchSizeForm: TEasyTaskPanelFormSearchSize;
  protected
    procedure VEELVItemContextMenu(Sender: TCustomEasyListview; HitInfo: TEasyHitInfoItem; WindowPoint: TPoint; var Menu: TPopupMenu; var Handled: Boolean);
    procedure VEELVSearchEnter(Sender: TObject);
    procedure VEELVSearchEnumFolder(Sender: TCustomVirtualExplorerEasyListview; Namespace: TNamespace; var AllowAsChild: Boolean);
    procedure VEELVSearchCustomColumnAdd(Sender: TCustomVirtualExplorerEasyListview; AddColumnProc: TELVAddColumnProc);
    procedure VEELVSearchCustomColumnCompare(Sender: TCustomVirtualExplorerEasyListview; Column: TExplorerColumn; Group: TEasyGroup; Item1, Item2: TExplorerItem; var CompareResult: Integer);
    procedure VEELVSearchCustomColumnGetCaption(Sender: TCustomVirtualExplorerEasyListview; Column: TExplorerColumn; Item: TExplorerItem; var ACaption: WideString);
  public
    destructor Destroy; override;

    function SetTheme(ThemeName: PChar): Boolean; override;
    function TabImage(var AnImageList: TCustomImageList; var AnImageIndex: Integer): Boolean; override;

    // IUltraInplaceNotifications
    procedure RootFolderChanged(Root: PItemIDList); override;

    property Data: TUltraSearchAttribs read FData write FData;
    property RootFolder: PItemIDList read FRootFolder write FRootFolder;
    property Running: Boolean read FRunning write FRunning;
    property SearchAdvOptionsForm: TEasyTaskPanelFormAdvOptions read GetSearchAdvOptionsForm;
    property SearchBrowseForm: TEasyTaskPanelFormBrowse read GetSearchBrowseForm;
    property SearchContentForm: TEasyTaskPanelFormMainSearch read GetSearchContentForm;
    property SearchDateForm: TEasyTaskPanelFormSearchDate read GetSearchDateForm;
    property SearchSizeForm: TEasyTaskPanelFormSearchSize read GetSearchSizeForm;
    property SearchThread: TUltraFileSearch read FSearchThread write FSearchThread;
    property VEELVSearch: TVirtualExplorerEasyListview read FVEELVSearch write FVEELVSearch;
  published
  end;

implementation

{$R *.DFM}

{ TTntFormUltraSearch }
destructor TTntFormUltraSearch.Destroy;
begin
  inherited Destroy;
end;


function TTntFormUltraSearch.GetSearchAdvOptionsForm: TEasyTaskPanelFormAdvOptions;
begin
  Result := (EasyTaskPanelBandSearch.Groups[3].TaskPanel as TEasyTaskPanelFormAdvOptions)
end;

function TTntFormUltraSearch.GetSearchBrowseForm: TEasyTaskPanelFormBrowse;
begin
  Result := (EasyTaskPanelBandSearch.Groups[4].TaskPanel as TEasyTaskPanelFormBrowse)
end;

function TTntFormUltraSearch.GetSearchContentForm: TEasyTaskPanelFormMainSearch;
begin
  Result := (EasyTaskPanelBandSearch.Groups[0].TaskPanel as TEasyTaskPanelFormMainSearch)
end;

function TTntFormUltraSearch.GetSearchDateForm: TEasyTaskPanelFormSearchDate;
begin
  Result := (EasyTaskPanelBandSearch.Groups[1].TaskPanel as TEasyTaskPanelFormSearchDate)
end;

function TTntFormUltraSearch.GetSearchSizeForm: TEasyTaskPanelFormSearchSize;
begin
  Result := (EasyTaskPanelBandSearch.Groups[2].TaskPanel as TEasyTaskPanelFormSearchSize)
end;

function TTntFormUltraSearch.SetTheme(ThemeName: PChar): Boolean;
begin
  // Use if Form is in DLL where a new TBXSwitcher is needed
  VEELVSearch.Invalidate;
  if EasyTaskPanelBandSearch.Groups.Count > 4 then
  begin
    EasyTaskPanelBandSearch.Groups[0].Caption := STR_SEARCH_SEARCHFOR;;
    EasyTaskPanelBandSearch.Groups[1].Caption := STR_SEARCH_SEARCHMODIFIED;
    EasyTaskPanelBandSearch.Groups[2].Caption := STR_SEARCH_SEARCHSIZE;
    EasyTaskPanelBandSearch.Groups[3].Caption := STR_SEARCH_SEARCHADVANCED;
    EasyTaskPanelBandSearch.Groups[4].Caption := STR_SEARCH_SEARCHFOLDERS;
  end;
  EasyTaskPanelBandSearch.Invalidate;
  Invalidate;
  Result := True
end;

function TTntFormUltraSearch.TabImage(var AnImageList: TCustomImageList;
  var AnImageIndex: Integer): Boolean;
begin
  Result := True;
  AnImageList := ImageList;
  AnImageIndex := 0;
end;

procedure TTntFormUltraSearch.TntFormClose(Sender: TObject;
  var Action: TCloseAction);
begin 
  Action := caFree
end;

procedure TTntFormUltraSearch.VEELVItemContextMenu(Sender: TCustomEasyListview;
  HitInfo: TEasyHitInfoItem; WindowPoint: TPoint; var Menu: TPopupMenu;
  var Handled: Boolean);
var
  MultiMenu: TCommonShellMultiParentContextMenu;
begin
  MultiMenu := TCommonShellMultiParentContextMenu.Create(nil);
  try
    MultiMenu.ShowContextMenu(Self, TExplorerItem( HitInfo.Item).Namespace, VEELVSearch.SelectedToNamespaceArray, @WindowPoint);
  finally
    Handled := True;
    MultiMenu.Free;
  end
end;

procedure TTntFormUltraSearch.VEELVSearchEnter(Sender: TObject);
begin
  if Assigned(Notifier) then
    Notifier.FocusChanged(VEELVSearch);
end;

procedure TTntFormUltraSearch.SpTBXEditFileNameEnter(Sender: TObject);
begin
  if Assigned(Notifier) then
 //   Notifier.FocusChanged(SpTBXEditFileName);
end;

procedure TTntFormUltraSearch.SpTBXEditContentEnter(Sender: TObject);
begin
  if Assigned(Notifier) then
 //   Notifier.FocusChanged(SpTBXEditContent);
end;

procedure TTntFormUltraSearch.VEELVSearchEnumFolder(
  Sender: TCustomVirtualExplorerEasyListview; Namespace: TNamespace;
  var AllowAsChild: Boolean);
begin
  AllowAsChild := False;
end;

procedure TTntFormUltraSearch.RootFolderChanged(Root: PItemIDList);
begin
  inherited;
  RootFolder := PIDLMgr.CopyPIDL(Root)
end;

procedure TTntFormUltraSearch.EasyTaskPanelBandSearchGetTaskPanel(
  Sender: TCustomEasyListview; Group: TEasyGroup;
  var TaskPanel: TEasyTaskPanelFormClass);
begin
  case Group.Index of
    0: TaskPanel := TEasyTaskPanelFormMainSearch;
    1: TaskPanel := TEasyTaskPanelFormSearchDate;
    2: TaskPanel := TEasyTaskPanelFormSearchSize;
    3: TaskPanel := TEasyTaskPanelFormAdvOptions;
    4: TaskPanel := TEasyTaskPanelFormBrowse
  end
end;

procedure TTntFormUltraSearch.TntFormShow(Sender: TObject);
var
  Node: PVirtualNode;
begin
  if Assigned(Parent) then
  begin
    FixFormFont(Font);
    EasyTaskPanelBandSearch.Visible := True;

    EasyTaskPanelBandSearch.Groups[0].Height := SearchContentForm.SpTBXEditContent.Top + SearchContentForm.SpTBXEditContent.Height + 8;
    EasyTaskPanelBandSearch.Groups[1].Height := SearchDateForm.DateTimePickerTo.Top + SearchDateForm.DateTimePickerTo.Height + 8;
    EasyTaskPanelBandSearch.Groups[2].Height := SearchSizeForm.SpTBXComboBoxSize.Top + SearchSizeForm.SpTBXComboBoxSize.Height + 8;
    EasyTaskPanelBandSearch.Groups[3].Height := SearchAdvOptionsForm.SpTBXGroupBoxFileAttributes.Top + SearchAdvOptionsForm.SpTBXCheckBoxTemporary.Top + SearchAdvOptionsForm.SpTBXCheckBoxTemporary.Height + 16;
    EasyTaskPanelBandSearch.Groups[4].Height := SearchBrowseForm.SpTBXPanelBrowse.Top + SearchBrowseForm.SpTBXPanelBrowse.Height + 8;
    Node := SearchBrowseForm.VETSearch.ForceNodeByPIDL(RootFolder, True);
    if Assigned(Node) then
      SearchBrowseForm.VETSearch.CheckState[Node] := csCheckedNormal;
    PIDLMgr.FreeAndNilPIDL(FRootFolder);

    EasyTaskPanelBandSearch.Groups[0].Caption := STR_SEARCH_SEARCHFOR;;
    EasyTaskPanelBandSearch.Groups[1].Caption := STR_SEARCH_SEARCHMODIFIED;
    EasyTaskPanelBandSearch.Groups[2].Caption := STR_SEARCH_SEARCHSIZE;
    EasyTaskPanelBandSearch.Groups[3].Caption := STR_SEARCH_SEARCHADVANCED;
    EasyTaskPanelBandSearch.Groups[4].Caption := STR_SEARCH_SEARCHFOLDERS;
    EasyTaskPanelBandSearch.Groups[1].Expanded := False;
    EasyTaskPanelBandSearch.Groups[2].Expanded := False;
    EasyTaskPanelBandSearch.Groups[3].Expanded := False;
    EasyTaskPanelBandSearch.Groups[4].Expanded := False;
    VEELVSearch.RebuildShellHeader;
  end
end;

procedure TTntFormUltraSearch.EasyTaskPanelBandSearchResize(
  Sender: TObject);
begin
  VEELVSearch.Update
end;

procedure TTntFormUltraSearch.SpTBXButtonSearchClick(Sender: TObject);
begin
  if not Running then
  begin
    VEELVSearch.BeginUpdate;
    VEELVSearch.Clear;
    VEELVSearch.RebuildShellHeader;
    VEELVSearch.EndUpdate;
    SearchAdvOptionsForm.LoadSearchData(FData);
    SearchBrowseForm.LoadSearchData(FData);
    SearchContentForm.LoadSearchData(FData);
    SearchDateForm.LoadSearchData(FData);
    SearchSizeForm.LoadSearchData(FData);
    SearchThread.UpdateRate := 300;
    Running := SearchThread.Run;
    if Running then
    begin
      SpTBXButtonSearch.Caption := STR_COLUMNDLG_BUTTONCANCEL;
      SpTBXButtonSearch.Invalidate;
    end
  end else
  begin
    SearchThread.Stop;
    while not SearchThread.Finished do
      Sleep(100);
    SpTBXButtonSearch.Caption := STR_SEARCH;
    SpTBXButtonSearch.Invalidate;
  end
end;

procedure TTntFormUltraSearch.UltraFileSearchEnd(Sender: TObject;
  Results: TCommonPIDLList);
begin
  SpTBXButtonSearch.Caption := STR_SEARCH;
  SpTBXButtonSearch.Invalidate;
  Running := False;
end;

procedure TTntFormUltraSearch.UltraFileSearchCompare(Sender: TObject;
  const FilePath: WideString; FindFileData: _WIN32_FIND_DATAW;
  var UseFile: Boolean);
var
  Size: Int64;
begin
  // In the context of the THREAD!!!!

  if samModifyMask in Data.Masks then
  begin
    // Look at the time stamps for filtering
    case Data.DateSearch of
      dstModified:
        begin
          UseFile := (CompareFileTime(FindFileData.ftLastWriteTime, Data.DateMax) <= 0) and
                     (CompareFileTime(FindFileData.ftLastWriteTime, Data.DateMin) >= 0)
        end;
      dstCreated:
        begin
          UseFile := (CompareFileTime(FindFileData.ftCreationTime, Data.DateMax) <= 0) and
                     (CompareFileTime(FindFileData.ftCreationTime, Data.DateMin) >= 0)
        end;
      dstAccessed:
        begin
          UseFile := (CompareFileTime(FindFileData.ftLastAccessTime, Data.DateMax) <= 0) and
                     (CompareFileTime(FindFileData.ftLastAccessTime, Data.DateMin) >= 0)
        end;
    end;
  end;

  if samSizeMask in Data.Masks then
  begin
    // Look at the size for filtering
    Size := FindFileData.nFileSizeLow + (FindFileData.nFileSizeHigh shl 32);

    case Data.SizeCompare of 
      fscAtLeast:
        begin
          UseFile := Size >= Data.Size
        end;
      fcsNoMore:
        begin
          UseFile := Size <= Data.Size
        end;
    end;

  end;

  if samFileMask in Data.Masks then
  begin
    // Look filename for matching
  end;

  if samContentMask in Data.Masks then
  begin
    // Look in the file for matching content
  end;
end;

procedure TTntFormUltraSearch.UltraFileSearchProgress(Sender: TObject;
  Results: TCommonPIDLList; var Handled: Boolean);
var
  i: Integer;
begin
  VEELVSearch.BeginUpdate;
  try
    Results.SharePIDLs := True;
    for i := 0 to Results.Count - 1 do
    begin
      if Assigned(Results[i]) then
        VEELVSearch.AddCustomItem(nil, TNamespace.Create(Results[i], nil), True)
    end;
  finally
    Handled := True;
    VEELVSearch.EndUpdate
  end;
end;

procedure TTntFormUltraSearch.TntFormCreate(Sender: TObject);
begin
  SearchThread := TUltraFileSearch.Create(nil);
  SearchThread.OnProgress := UltraFileSearchProgress;
  SearchThread.OnSearchCompare := UltraFileSearchCompare;
  SearchThread.OnSearchEnd := UltraFileSearchEnd;

  VEELVSearch := TUltraExplorerEasyListview.Create(Self);
  VEELVSearch.OnEnter := VEELVSearchEnter;
  VEELVSearch.OnItemContextMenu := VEELVItemContextMenu;
  VEELVSearch.OnEnumFolder := VEELVSearchEnumFolder;
  VEELVSearch.OnCustomColumnAdd := VEELVSearchCustomColumnAdd;
  VEELVSearch.OnCustomColumnCompare := VEELVSearchCustomColumnCompare;
  VEELVSearch.OnCustomColumnGetCaption := VEELVSearchCustomColumnGetCaption;
  VEELVSearch.View := elsReport;
  VEELVSearch.Align := alClient;
  VEELVSearch.Parent := Self;
  VEELVSearch.Selection.MultiSelect := True;
  VEELVSearch.Options := [eloExecuteOnDblClick, eloThreadedImages, eloThreadedDetails,
    eloQueryInfoHints, eloShellContextMenus, eloGhostHiddenFiles];  
  VEELVSearch.CellSizes.Thumbnail.Height := Attribs.Listview.ThumbsHeight;
  VEELVSearch.CellSizes.Thumbnail.Width := Attribs.Listview.ThumbsWidth;
  VEELVSearch.ThumbsManager.AutoLoad := Attribs.Listview.ThumbsCache;
  VEELVSearch.ThumbsManager.AutoSave := Attribs.Listview.ThumbsCache;
  VEELVSearch.ThumbsManager.StorageCompressed := Attribs.Listview.ThumbsCompress;
  VEELVSearch.PaintInfoColumn.HilightFocused := Attribs.Listview.HilightColumn;
  VEELVSearch.PaintInfoColumn.HilightFocusedColor := Attribs.Listview.HilightColumnColor;
  if Attribs.Listview.QueryInfoHints then
    VEELVSearch.Options := VEELVSearch.Options + [eloQueryInfoHints]
  else
    VEELVSearch.Options := VEELVSearch.Options - [eloQueryInfoHints];
 // VEELVSearch.CellSizes.List.AutoSize := Attribs.Listview.AutoSizeCells;
 // VEELVSearch.CellSizes.Report.AutoSize := Attribs.Listview.AutoSizeCells;
//  VEELVSearch.CellSizes.SmallIcon.AutoSize := Attribs.Listview.AutoSizeCells;
  VEELVSearch.DefaultSortColumn := Attribs.Listview.DefaultColumn;
  VEELVSearch.DefaultSortDir := Attribs.Listview.SortDirection;
  VEELVSearch.ThumbsManager.HideCaption := Attribs.Listview.HideCaptions;
  VEELVSearch.Font.Assign(Attribs.Listview.Font);
  VEELVSearch.Color := Attribs.Listview.Color;
  VEELVSearch.Selection.UseFocusRect := Attribs.Listview.FocusRect;
  VEELVSearch.Selection.Gradient := Attribs.Listview.GradientSelection;
  VEELVSearch.Selection.FullRowSelect := Attribs.Listview.FullRowSelect;
  if VEELVSearch.Selection.Gradient then
  begin
    VEELVSearch.Selection.AlphaBlend := True;
    VEELVSearch.Selection.TextColor := Font.Color;
    VEELVSearch.Selection.FullCellPaint := True;
  end else
  begin
    VEELVSearch.Selection.AlphaBlend := False;
    VEELVSearch.Selection.TextColor := clHighlightText;
    VEELVSearch.Selection.FullCellPaint := False;
  end;
  VEELVSearch.CompressedFile.Hilight := Attribs.Listview.HilightCompressed;
  VEELVSearch.EncryptedFile.Hilight := Attribs.Listview.HilightEncrypted;
  VEELVSearch.EditManager.Font.Assign(VEELVSearch.Font);
  VEELVSearch.Header.ShowInAllViews := Attribs.Listview.AlwaysShowHeader;
  VEELVSearch.SortFolderFirstAlways := Attribs.Listview.FoldersAlwaysFirst;
  VEELVSearch.PaintInfoItem.GridLines := Attribs.Listview.GridLines;
  VEELVSearch.DragManager.Enabled := True;
  VEELVSearch.ShowThemedBorder := False;
  VEELVSearch.Selection.AlphaBlendSelRect := True;
  VEELVSearch.Selection.EnableDragSelect := True;
  VEELVSearch.Selection.MouseButton := [cmbLeft, cmbRight];
  VEELVSearch.Selection.MouseButtonSelRect := [cmbLeft, cmbRight];
  VEELVSearch.Selection.MultiSelect := True;
  VEELVSearch.Selection.RectSelect := False;
  VEELVSearch.Selection.RoundRect := True;
  VEELVSearch.DragManager.MouseButton := [cmbLeft, cmbRight];
  VEELVSearch.EditManager.Enabled := True;
  VEELVSearch.EditManager.AutoEditDelayTime  := 750;
  VEELVSearch.IncrementalSearch.Enabled := True;
  VEELVSearch.IncrementalSearch.StartType := eissFocusedNode;
  VEELVSearch.ShowHint := True; // Always Show the hint/Tooltip
  VEELVSearch.IncrementalSearch.ResetTime := 500;
  VEELVSearch.Visible := True;

  FData.SearchFolders := TTntStringList.Create;
  SearchThread.SearchData := @Data;
end;

procedure TTntFormUltraSearch.TntFormDestroy(Sender: TObject);
begin
  FreeAndNil(FSearchThread);
  FreeAndNil(FData.SearchFolders);
end;

procedure TTntFormUltraSearch.DKLanguageControllerLanguageChanged(
  Sender: TObject);
begin
  EasyTaskPanelBandSearch.Groups[0].Caption := STR_SEARCH_SEARCHFOR;;
  EasyTaskPanelBandSearch.Groups[1].Caption := STR_SEARCH_SEARCHMODIFIED;
  EasyTaskPanelBandSearch.Groups[2].Caption := STR_SEARCH_SEARCHSIZE;
  EasyTaskPanelBandSearch.Groups[3].Caption := STR_SEARCH_SEARCHADVANCED;
  EasyTaskPanelBandSearch.Groups[4].Caption := STR_SEARCH_SEARCHFOLDERS;
end;

procedure TTntFormUltraSearch.VEELVSearchCustomColumnAdd(
  Sender: TCustomVirtualExplorerEasyListview;
  AddColumnProc: TELVAddColumnProc);
var
  Column: TExplorerColumn;
begin
  Column := AddColumnProc;
  Column.Caption := STR_PATH;
  Column.Width := 500;
end;

procedure TTntFormUltraSearch.VEELVSearchCustomColumnCompare(
  Sender: TCustomVirtualExplorerEasyListview; Column: TExplorerColumn;
  Group: TEasyGroup; Item1, Item2: TExplorerItem;
  var CompareResult: Integer);
begin
  CompareResult := WideStrIComp(PWideChar( Item1.Namespace.NameForParsing), PWideChar( Item2.Namespace.NameForParsing));
  if Column.SortDirection = esdDescending then
    CompareResult := -CompareResult   
end;

procedure TTntFormUltraSearch.VEELVSearchCustomColumnGetCaption(
  Sender: TCustomVirtualExplorerEasyListview; Column: TExplorerColumn;
  Item: TExplorerItem; var ACaption: WideString);
begin
  ACaption := WideExtractFilePath( Item.Namespace.NameForParsing)
end;

end.
