unit UnitUltraLaunchDesignerForm;

// The contents of this file are subject to the Mozilla Public License
// Version 1.1 (the "License"); you maynot use this file except in compliance
// with the License. You may obtain a copy of the License at http://www.mozilla.org/MPL/
//
// Alternatively, you may redistribute this library, use and/or modify it under the terms of the
// GNU Lesser General Public License as published by the Free Software Foundation;
// either version 2.1 of the License, or (at your option) any later version.
// You may obtain a copy of the LGPL at http://www.gnu.org/copyleft/.
//
// Software distributed under the License is distributed on an "AS IS" basis,
// WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for the
// specific language governing rights and limitations under the License.

interface

uses
  Windows,
  Messages,
  SysUtils,
  Classes,
  Graphics,
  Controls,
  Forms,
  Dialogs,
  TntForms,
  SpTBXControls,
  VirtualTrees,
  StdCtrls,
  SpTBXEditors,
  TntStdCtrls,
  SpTBXDkPanels,
  ActnList,
  TntActnList,
  TntDialogs,
  DKLang,
  TntSysUtils,
  TntClasses,
  MPShellUtilities,
  UltraUserStrings,
  MPCommonUtilities,
  ImgList,
  MPCommonObjects,
  UltraOptions,
  ActiveX,
  UltraUtilities,
  UltraFavTree,
  MPDataObject, SpTBXItem;

const
  STR_LAUNCH_FILE_FILTER = 'EXE files (*.exe)|*.EXE|Batch Files (*.bat)|*.BAT|Command Files (*.com)|*.COM|All files (*.*)|*.*';

type
  TUltraShowWindow = (
    uswNormal,
    uswMaximized,
    uswMinimized
  );

  TUltraActionEnableState = (
    aesRename,
    aesDelete,
    aesNewApp,
    aesNewFolder
  );
  TUltraActionEnableStates = set of TUltraActionEnableState;

  TUltraLauncherData = class;

  TUltraLauncherDataList = class
  private
    FList: TList;
    function GetCount: Integer;
    function GetItems(Index: Integer): TUltraLauncherData;
    procedure SetItems(Index: Integer; Value: TUltraLauncherData);
  protected
    property List: TList read FList write FList;
  public
    constructor Create;
    destructor Destroy; override;
    procedure Add(LauncherData: TUltraLauncherData);
    procedure Clear;
    procedure Delete(Index: Integer);
    procedure LoadFromStream(S: TStream; var StreamVersion: Integer);
    procedure SaveToStream(S: TStream; StreamVersion: Integer);
    property Count: Integer read GetCount;
    property Items[Index: Integer]: TUltraLauncherData read GetItems write SetItems; default;
  end;

  TUltraLauncherData = class(TPersistent)
  private
    FApplicationPath: WideString;
    FInitialPath: WideString;
    FIsFolder: Boolean;
    FIsRoot: Boolean;
    FName: WideString;
    FNS: TNamespace;
    FOperation: WideString;
    FParameters: WideString;
    FShortCut: TShortCut;
    FShowWindow: TUltraShowWindow;
    FSubItems: TUltraLauncherDataList;
    function GetName: WideString;
    procedure SetNS(const Value: TNamespace);
  protected

  public
    constructor Create; virtual;
    destructor Destroy; override;
    function HasSubChildren: Boolean;
    procedure Assign(Source: TPersistent); override;
    
    procedure FreeSubItems;
    procedure LoadFromFile(FileName: WideString; var StreamVersion: Integer);
    procedure LoadFromStream(S: TStream; var StreamVersion: Integer);
    procedure SaveToFile(FileName: WideString; StreamVersion: Integer);
    procedure SaveToStream(S: TStream; StreamVersion: Integer);
    property ApplicationPath: WideString read FApplicationPath write FApplicationPath;
    property InitialPath: WideString read FInitialPath write FInitialPath;
    property IsFolder: Boolean read FIsFolder write FIsFolder;
    property IsRoot: Boolean read FIsRoot;
    property Name: WideString read GetName write FName;
    property NS: TNamespace read FNS write SetNS;
    property Operation: WideString read FOperation write FOperation;
    property Parameters: WideString read FParameters write FParameters;
    property ShortCut: TShortCut read FShortCut write FShortCut;
    property ShowWindow: TUltraShowWindow read FShowWindow write FShowWindow;
    property SubItems: TUltraLauncherDataList read FSubItems write FSubItems;
  end;

  TUltraLauncherDataRoot = class(TUltraLauncherData)
  public
    constructor Create; override;
  end;

  TTntFormUltraLaunchDesigner = class(TTntForm)
    VirtualStringTreeLauncher: TVirtualStringTree;
    SpTBXSplitter1: TSpTBXSplitter;
    SpTBXPanel1: TSpTBXPanel;
    SpTBXButtonNewApp: TSpTBXButton;
    SpTBXButtonNewMenu: TSpTBXButton;
    SpTBXButtonDelete: TSpTBXButton;
    SpTBXComboBoxOperation: TSpTBXComboBox;
    SpTBXLabelOperation: TSpTBXLabel;
    SpTBXButtonEditAppFile: TSpTBXButtonEdit;
    SpTBXLabelAppFile: TSpTBXLabel;
    SpTBXButtonEditInitialDir: TSpTBXButtonEdit;
    SpTBXLabelInitialDir: TSpTBXLabel;
    SpTBXEditCommandLine: TSpTBXEdit;
    SpTBXLabelCmdLineParams: TSpTBXLabel;
    SpTBXComboBoxShowWindow: TSpTBXComboBox;
    SpTBXLabelShowWindow: TSpTBXLabel;
    SpTBXButtonOk: TSpTBXButton;
    SpTBXButtonCancel: TSpTBXButton;
    TntActionList: TTntActionList;
    TntOpenDialog: TTntOpenDialog;
    ActionNewApplication: TTntAction;
    ActionNewSubMenu: TTntAction;
    ActionDelete: TTntAction;
    ActionQueryFileName: TTntAction;
    ActionQueryInitialDir: TTntAction;
    ActionOk: TTntAction;
    SpTBXButtonRename: TSpTBXButton;
    ActionRename: TTntAction;
    ImageList: TImageList;
    SpTBXLabelMacros: TSpTBXLabel;
    SpTBXListBoxMacros: TSpTBXListBox;
    DKLanguageController1: TDKLanguageController;
    procedure ActionNewApplicationExecute(Sender: TObject);
    procedure ActionNewSubMenuExecute(Sender: TObject);
    procedure ActionDeleteExecute(Sender: TObject);
    procedure ActionQueryFileNameExecute(Sender: TObject);
    procedure ActionQueryInitialDirExecute(Sender: TObject);
    procedure ActionOkExecute(Sender: TObject);
    procedure VirtualStringTreeLauncherFreeNode(Sender: TBaseVirtualTree; Node: PVirtualNode);
    procedure VirtualStringTreeLauncherNewText(Sender: TBaseVirtualTree; Node: PVirtualNode; Column: TColumnIndex; NewText: WideString);
    procedure VirtualStringTreeLauncherGetText(Sender: TBaseVirtualTree; Node: PVirtualNode; Column: TColumnIndex; TextType: TVSTTextType; var CellText: WideString);
    procedure VirtualStringTreeLauncherFocusChanged(Sender: TBaseVirtualTree; Node: PVirtualNode; Column: TColumnIndex);
    procedure TntFormShow(Sender: TObject);
    procedure VirtualStringTreeLauncherDragOver(Sender: TBaseVirtualTree; Source: TObject; Shift: TShiftState; State: TDragState; Pt: TPoint; Mode: TDropMode; var Effect: Integer; var Accept: Boolean);
    procedure ActionRenameExecute(Sender: TObject);
    procedure VirtualStringTreeLauncherGetImageIndexEx(Sender: TBaseVirtualTree; Node: PVirtualNode; Kind: TVTImageKind; Column: TColumnIndex; var Ghosted: Boolean; var ImageIndex: Integer; var ImageList: TCustomImageList);
    procedure TntFormKeyDown(Sender: TObject; var Key: Word; Shift: TShiftState);
    procedure VirtualStringTreeLauncherDragDrop(Sender: TBaseVirtualTree; Source: TObject; DataObject: IDataObject; Formats: TFormatArray; Shift: TShiftState; Pt: TPoint; var Effect: Integer; Mode: TDropMode);
    procedure VirtualStringTreeLauncherFocusChanging(Sender: TBaseVirtualTree; OldNode, NewNode: PVirtualNode; OldColumn, NewColumn: TColumnIndex; var Allowed: Boolean);
    procedure TntFormCloseQuery(Sender: TObject; var CanClose: Boolean);
    procedure SpTBXListBoxMacrosDblClick(Sender: TObject);
    procedure TntFormCreate(Sender: TObject);
  private
    FWndParent: HWnd;
    { Private declarations }
  protected
    procedure CreateParams(var Params: TCreateParams); override;
    property WndParent: HWnd read FWndParent write FWndParent;
    procedure EnableControls(Enable: Boolean);
    procedure EnableActions(Actions: TUltraActionEnableStates);
    procedure LoadControls(Data: TUltraLauncherData);
    procedure SelectMacro;
    procedure StoreControls(Data: TUltraLauncherData);
  public
    { Public declarations }
    constructor CreateEx(AnOwner: TCustomForm);
    function GetLaunchData(Node: PVirtualNode): TUltraLauncherData;
    procedure LoadLaunchData(Root: TUltraLauncherData);
    procedure SaveLaunchData(Root: TUltraLauncherData);
  end;

implementation

uses Math,
     UnitOkDialog;

const
  STREAM_VERSION = 0;

{$R *.DFM}

{ TUltraLauncherDataList }
constructor TUltraLauncherDataList.Create;
begin
  inherited Create;
  List := TList.Create;
end;

destructor TUltraLauncherDataList.Destroy;
begin
  FreeAndNil(FList);
  inherited Destroy;
end;

function TUltraLauncherDataList.GetCount: Integer;
begin
  Result := List.Count
end;

function TUltraLauncherDataList.GetItems(Index: Integer): TUltraLauncherData;
begin
  Result := TUltraLauncherData( List[Index])
end;

procedure TUltraLauncherDataList.Add(LauncherData: TUltraLauncherData);
begin
  List.Add(LauncherData)
end;

procedure TUltraLauncherDataList.Delete(Index: Integer);
begin
  TObject( List.Extract( List[Index])).Free
end;

procedure TUltraLauncherDataList.LoadFromStream(S: TStream; var StreamVersion: Integer);
var
  Len, i: Integer;
  Data: TUltraLauncherData;
begin
  StreamVersion := StreamHelper.ReadInteger(S);
  Len := StreamHelper.ReadInteger(S);
  for i := 0 to Len - 1 do
  begin
    Data := TUltraLauncherData.Create;
    Data.LoadFromStream(S, StreamVersion);
    Add(Data)
  end
end;

procedure TUltraLauncherDataList.SaveToStream(S: TStream; StreamVersion: Integer);
var
  Len, i: Integer;
begin
  StreamHelper.WriteInteger(S, StreamVersion);
  Len := List.Count;
  StreamHelper.WriteInteger(S, Len);
  for i := 0 to Len - 1 do
    Items[i].SaveToStream(S, StreamVersion)
end;

procedure TUltraLauncherDataList.SetItems(Index: Integer; Value: TUltraLauncherData);
begin
  List[Index] := Value
end;

procedure TUltraLauncherDataList.Clear;
var
  i: Integer;
begin
  for i := 0 to List.Count - 1 do
    TObject( List[i]).Free;
  List.Count := 0;
end;

{ TUltraLauncherData }
constructor TUltraLauncherData.Create;
begin
  inherited Create;
  SubItems := TUltraLauncherDataList.Create;
end;

destructor TUltraLauncherData.Destroy;
begin
  FreeSubItems;
  FreeAndNil(FNS);
  FreeAndNil(FSubItems);
  inherited Destroy;
end;

function TUltraLauncherData.GetName: WideString;
begin
  if (FName = '') and Assigned(NS) then
    Result := WideStripExt( NS.NameAddressbarInFolder)
  else
    Result := FName
end;

function TUltraLauncherData.HasSubChildren: Boolean;
begin
  Result := (SubItems.Count > 0) and IsFolder
end;

procedure TUltraLauncherData.Assign(Source: TPersistent);
var
  Data: TUltraLauncherData;
begin
  if Source is TUltraLauncherData then
  begin
    Data := TUltraLauncherData(Source);
    ApplicationPath := Data.ApplicationPath;
    InitialPath := Data.InitialPath;
    IsFolder := Data.IsFolder;
    Operation := Data.Operation;
    Parameters := Data.Parameters;
    ShowWindow := Data.ShowWindow;
    Name := Data.Name;
    if Assigned(Data.NS) and not Data.IsFolder then
      NS := TNamespace.Create(PIDLMgr.CopyPIDL(Data.NS.AbsolutePIDL), nil);
  end
end;

procedure TUltraLauncherData.FreeSubItems;
var
  i: Integer;
begin
  for i := 0 to SubItems.Count - 1 do
    SubItems[i].Free;
  SubItems.List.Count := 0
end;

procedure TUltraLauncherData.LoadFromFile(FileName: WideString; var StreamVersion: Integer);
var
  F: TTntFileStream;
begin
  F := TTntFileStream.Create(FileName, fmOpenRead or fmShareExclusive);
  try
    LoadFromStream(F, StreamVersion)
  finally
    F.Free
  end
end;

procedure TUltraLauncherData.LoadFromStream(S: TStream; var StreamVersion: Integer);
begin
  if not IsRoot then
  begin
    StreamVersion := StreamHelper.ReadInteger(S);
    ApplicationPath := StreamHelper.ReadWideString(S);
    InitialPath := StreamHelper.ReadWideString(S);
    Name := StreamHelper.ReadWideString(S);
    Operation := StreamHelper.ReadWideString(S);
    Parameters := StreamHelper.ReadWideString(S);
    IsFolder := StreamHelper.ReadBoolean(S);
    ShowWindow := TUltraShowWindow( StreamHelper.ReadInteger(S));
    if not IsFolder then
      NS := TNamespace.Create(PIDLMgr.LoadFromStream(S), nil);
    if StreamVersion > 4 then
      S.Read(FShortCut, SizeOf(ShortCut));
  end;
  SubItems.LoadFromStream(S, StreamVersion)
end;

procedure TUltraLauncherData.SaveToFile(FileName: WideString; StreamVersion: Integer);
var
  F: TTntFileStream;
begin
  F := TTntFileStream.Create(FileName, fmCreate	or fmShareExclusive);
  try
    SaveToStream(F, StreamVersion)
  finally
    F.Free
  end
end;

procedure TUltraLauncherData.SaveToStream(S: TStream; StreamVersion: Integer);
begin
  if not IsRoot then
  begin
    StreamHelper.WriteInteger(S, StreamVersion);
    StreamHelper.WriteWideString(S, ApplicationPath);
    StreamHelper.WriteWideString(S, InitialPath);
    StreamHelper.WriteWideString(S, Name);
    StreamHelper.WriteWideString(S, Operation);
    StreamHelper.WriteWideString(S, Parameters);
    StreamHelper.WriteBoolean(S, IsFolder);
    StreamHelper.WriteInteger(S, Integer( ShowWindow));
    if not IsFolder then
      PIDLMgr.SaveToStream(S, NS.AbsolutePIDL);
    S.Write(ShortCut, SizeOf(ShortCut));
  end;
  SubItems.SaveToStream(S, StreamVersion)
end;

procedure TUltraLauncherData.SetNS(const Value: TNamespace);
begin
  if Value <> NS then
  begin
    FreeAndNil(FNS);
    FNS := Value;
  end
end;

{TFormUltraLaunchDesigner}

constructor TTntFormUltraLaunchDesigner.CreateEx(AnOwner: TCustomForm);
begin
  if Assigned(AnOwner) then
     WndParent := AnOwner.Handle
   else
     WndParent := 0;
   inherited Create(nil);
end;

procedure TTntFormUltraLaunchDesigner.CreateParams(var Params: TCreateParams);
begin
  inherited;
  if WndParent <> 0 then
    Params.WndParent := WndParent;
end;

function TTntFormUltraLaunchDesigner.GetLaunchData(Node: PVirtualNode): TUltraLauncherData;
begin
  Result := TUltraLauncherData( VirtualStringTreeLauncher.GetNodeData(Node)^)
end;

procedure TTntFormUltraLaunchDesigner.ActionNewApplicationExecute(
  Sender: TObject);
var
  Data: TUltraLauncherData;
  Node: PVirtualNode;
begin
  Data := TUltraLauncherData.Create;
  Data.IsFolder := False;
  Data.Name := 'New Application';
  Data.Operation := 'Open';
  Node := VirtualStringTreeLauncher.GetFirstSelected;
  if Assigned(Node) then
  begin
    if not GetLaunchData(Node).IsFolder then
    begin
      if Node.Parent <> VirtualStringTreeLauncher.RootNode then
        Node := Node.Parent
      else
        Node := nil
    end
  end;
  Node := VirtualStringTreeLauncher.AddChild(Node, Data);
  VirtualStringTreeLauncher.Selected[Node] := True;
  VirtualStringTreeLauncher.FocusedNode := Node;
  ActionQueryFileNameExecute(SpTBXButtonEditAppFile);
  if not Assigned(GetLaunchData(Node).NS) then
    VirtualStringTreeLauncher.DeleteNode(Node)
end;

procedure TTntFormUltraLaunchDesigner.ActionNewSubMenuExecute(
  Sender: TObject);
var
  Data: TUltraLauncherData;
  Node: PVirtualNode;
begin
  Data := TUltraLauncherData.Create;
  Data.IsFolder := True;
  Data.Name := 'New Folder';
  Node := VirtualStringTreeLauncher.GetFirstSelected;
  if Assigned(Node) then
  begin
    if not GetLaunchData(Node).IsFolder then
    begin
      if Node.Parent <> VirtualStringTreeLauncher.RootNode then
        Node := Node.Parent
      else
        Node := nil
    end
  end;
  Node := VirtualStringTreeLauncher.AddChild(Node, Data);
  VirtualStringTreeLauncher.Selected[Node] := True;
  VirtualStringTreeLauncher.FocusedNode := Node;
  VirtualStringTreeLauncher.EditNode(Node, -1);
end;

procedure TTntFormUltraLaunchDesigner.ActionDeleteExecute(Sender: TObject);
var
  Node, NextNode: PVirtualNode;
begin
  Node := VirtualStringTreeLauncher.GetFirstSelected;
  if Assigned(Node) then
  begin
    NextNode := nil;
    if Assigned(Node.NextSibling) then
      NextNode := Node.NextSibling
    else
    if Assigned(Node.PrevSibling) then
      NextNode := Node.PrevSibling
    else
    if Assigned(Node.Parent) then
      NextNode := Node.Parent;
    VirtualStringTreeLauncher.DeleteNode(Node);
    VirtualStringTreeLauncher.FocusedNode := NextNode;
    VirtualStringTreeLauncher.Selected[NextNode] := True
  end
end;

procedure TTntFormUltraLaunchDesigner.ActionQueryFileNameExecute(Sender: TObject);
var
  Node: PVirtualNode;
  WarningForm: TTntFormOkDialog;
begin
  Node := VirtualStringTreeLauncher.GetFirstSelected;
  if Assigned(Node) then
  begin
    TntOpenDialog.Filter := STR_LAUNCH_FILE_FILTER;
    if TntOpenDialog.Execute then
    begin
      if WideFileExists(TntOpenDialog.FileName) then
      begin
        SpTBXButtonEditAppFile.Text := TntOpenDialog.FileName;
        GetLaunchData(Node).NS := TNamespace.CreateFromFileName(TntOpenDialog.FileName);
        if SpTBXButtonEditInitialDir.Text = '' then
          SpTBXButtonEditInitialDir.Text := WideStripTrailingBackslash( WideExtractFilePath(TntOpenDialog.FileName));
        GetLaunchData(Node).Name := WideStripExt( WideExtractFileName(TntOpenDialog.FileName))
      end else
      begin
        WarningForm := TTntFormOkDialog.CreateEx(nil);
        try
          WarningForm.Caption := STR_WARNING;
          WarningForm.SpTBXLabel.Caption := STR_CANNOTFINDFILE + ': ' + TntOpenDialog.FileName;
          WarningForm.ShowModal;
        finally
          WarningForm.Free;
        end;
      end
    end;
    VirtualStringTreeLauncher.InvalidateNode(Node)
  end
end;

procedure TTntFormUltraLaunchDesigner.ActionQueryInitialDirExecute(Sender: TObject);
var
  InitalPath, Path: WideString;
  Node: PVirtualNode;
begin
  Node := VirtualStringTreeLauncher.GetFirstSelected;
  if Assigned(Node) then
  begin
    InitalPath := WideStripTrailingBackslash(  WideExtractFilePath(SpTBXButtonEditAppFile.Text));
    if MPBrowseForFolder(STR_CHOOSEWORKINGDIR, '', InitalPath, [mpbfNewStyleDialog, mpbfEditBox, mpbfMustExist, mpbfFileSysFolder], Path) then
      SpTBXButtonEditInitialDir.Text := Path;
    VirtualStringTreeLauncher.InvalidateNode(Node);
  end
end;

procedure TTntFormUltraLaunchDesigner.ActionOkExecute(Sender: TObject);
begin
  if Assigned(VirtualStringTreeLauncher.FocusedNode) then
    StoreControls(GetLaunchData(VirtualStringTreeLauncher.FocusedNode)); 
end;

procedure TTntFormUltraLaunchDesigner.EnableControls(Enable: Boolean);
begin
  SpTBXButtonEditAppFile.Enabled := Enable;
  SpTBXComboBoxOperation.Enabled := Enable;
  SpTBXComboBoxShowWindow.Enabled := Enable;
  SpTBXEditCommandLine.Enabled := Enable;
  SpTBXButtonEditInitialDir.Enabled := Enable;
  SpTBXListBoxMacros.Enabled := Enable;
end;

procedure TTntFormUltraLaunchDesigner.EnableActions(Actions: TUltraActionEnableStates);
begin
  ActionRename.Enabled := aesRename in Actions;
  ActionDelete.Enabled := aesDelete in Actions;
  ActionNewApplication.Enabled := aesNewApp in Actions;
  ActionNewSubMenu.Enabled := aesNewFolder in Actions;
end;

procedure TTntFormUltraLaunchDesigner.LoadControls(Data: TUltraLauncherData);
begin
  if Assigned(Data) then
  begin
    SpTBXButtonEditAppFile.Text := Data.ApplicationPath;
    SpTBXComboBoxOperation.Text := Data.Operation;
    SpTBXComboBoxShowWindow.ItemIndex := Integer( Data.ShowWindow);
    SpTBXEditCommandLine.Text := Data.Parameters;
    SpTBXButtonEditInitialDir.Text := Data.InitialPath;
  end else
  begin
    SpTBXButtonEditAppFile.Text := '';
    SpTBXComboBoxOperation.Text := '';
    SpTBXEditCommandLine.Text := '';
    SpTBXButtonEditInitialDir.Text := '';
  end
end;

procedure TTntFormUltraLaunchDesigner.LoadLaunchData(Root: TUltraLauncherData);

  procedure RunData(Parent: PVirtualNode; Data: TUltraLauncherData);
  var
    i: Integer;
    Node: PVirtualNode;
    NewData: TUltraLauncherData;
  begin
    for i := 0 to Data.SubItems.Count - 1 do
    begin
      NewData := TUltraLauncherData.Create;
      NewData.Assign(Data.SubItems[i]);
      Node := VirtualStringTreeLauncher.AddChild(Parent, NewData);
      if Data.SubItems[i].IsFolder then
        RunData(Node, Data.SubItems[i])
    end;
  end;

begin
  VirtualStringTreeLauncher.Clear;
  RunData(VirtualStringTreeLauncher.RootNode, Root)
end;

procedure TTntFormUltraLaunchDesigner.SaveLaunchData(Root: TUltraLauncherData);


  procedure RunData(Parent: PVirtualNode; Data: TUltraLauncherData);
  var
    Node: PVirtualNode;
    NewData: TUltraLauncherData;
  begin
    Node := Parent.FirstChild;
    while Assigned(Node) do
    begin
      NewData := TUltraLauncherData.Create;
      NewData.Assign( GetLaunchData(Node));
      Data.SubItems.Add(NewData);
      if NewData.IsFolder then
        RunData(Node, NewData);
      Node := Node.NextSibling
    end;
  end;

begin
  RunData(VirtualStringTreeLauncher.RootNode, Root)
end;

procedure TTntFormUltraLaunchDesigner.SelectMacro;
begin
  SpTBXEditCommandLine.Text := SpTBXEditCommandLine.Text + SpTBXListBoxMacros.Items[SpTBXListBoxMacros.ItemIndex]
end;

procedure TTntFormUltraLaunchDesigner.StoreControls(Data: TUltraLauncherData);
begin
  if Assigned(Data) then
  begin
    Data.ApplicationPath := SpTBXButtonEditAppFile.Text;
    Data.Operation := SpTBXComboBoxOperation.Text;
    Data.ShowWindow := TUltraShowWindow( SpTBXComboBoxShowWindow.ItemIndex);
    Data.Parameters := SpTBXEditCommandLine.Text;
    Data.InitialPath := SpTBXButtonEditInitialDir.Text
  end
end;

procedure TTntFormUltraLaunchDesigner.VirtualStringTreeLauncherFreeNode(
  Sender: TBaseVirtualTree; Node: PVirtualNode);
begin
  TObject( Sender.GetNodeData(Node)^).Free;
end;

procedure TTntFormUltraLaunchDesigner.VirtualStringTreeLauncherNewText(
  Sender: TBaseVirtualTree; Node: PVirtualNode; Column: TColumnIndex;
  NewText: WideString);
begin
  GetLaunchData(Node).Name := NewText
end;

procedure TTntFormUltraLaunchDesigner.VirtualStringTreeLauncherGetText(
  Sender: TBaseVirtualTree; Node: PVirtualNode; Column: TColumnIndex;
  TextType: TVSTTextType; var CellText: WideString);
begin
  CellText := GetLaunchData(Node).Name
end;

procedure TTntFormUltraLaunchDesigner.VirtualStringTreeLauncherFocusChanged(
  Sender: TBaseVirtualTree; Node: PVirtualNode; Column: TColumnIndex);
begin
  if Assigned(Node) then
  begin
    if GetLaunchData(Node).IsFolder then
    begin
      EnableControls(False);
      EnableActions([aesRename, aesDelete, aesNewApp, aesNewFolder]);
    end else
    begin
      EnableControls(True);
      LoadControls(GetLaunchData(Node));
      EnableActions([aesRename, aesDelete, aesNewApp, aesNewFolder]);
    end
  end else
  begin
    EnableControls(False);
    EnableActions([aesNewApp, aesNewFolder]);
  end
end;

procedure TTntFormUltraLaunchDesigner.TntFormShow(Sender: TObject);
var
  FileName: WideString;
begin
  FixFormFont(Font);
  EnableControls(False);
  VirtualStringTreeLauncher.FullExpand;
  FileName := AppFilesPath + '\' + STR_LAUNCH_VERBS_FILE_OLD;
  if WideFileExists(FileName) then
  begin
    SpTBXComboBoxOperation.Items.LoadFromFile(FileName);
    WideDeleteFile(FileName)
  end else
  begin
    FileName := AppFilesPath + '\' + STR_LAUNCH_VERBS_FILE;
    if WideFileExists(FileName) then
      SpTBXComboBoxOperation.Items.LoadFromFile(FileName);
  end;
  if VirtualStringTreeLauncher.GetFirst <> nil then
  begin
    VirtualStringTreeLauncher.Selected[VirtualStringTreeLauncher.GetFirst] := True;
    VirtualStringTreeLauncher.FocusedNode := VirtualStringTreeLauncher.GetFirst
  end;
  ClientWidth := SpTBXPanel1.Width + VirtualStringTreeLauncher.Width + SpTBXSplitter1.Width + 8;
  ClientHeight := SpTBXPanel1.Height + 8;
  if SpTBXButtonOk.Top + SpTBXButtonOk.Height > ClientHeight then
   ClientHeight := SpTBXButtonOk.Top + SpTBXButtonOk.Height + 8
end;

procedure TTntFormUltraLaunchDesigner.VirtualStringTreeLauncherDragOver(
  Sender: TBaseVirtualTree; Source: TObject; Shift: TShiftState;
  State: TDragState; Pt: TPoint; Mode: TDropMode; var Effect: Integer;
  var Accept: Boolean);
begin
  if DataObjectSupportsShell(Sender.DragManager.DataObject) then
    Effect := DROPEFFECT_LINK;
  Accept := True
end;

procedure TTntFormUltraLaunchDesigner.ActionRenameExecute(Sender: TObject);
begin
  VirtualStringTreeLauncher.EditNode(VirtualStringTreeLauncher.GetFirstSelected, -1);
end;

procedure TTntFormUltraLaunchDesigner.VirtualStringTreeLauncherGetImageIndexEx(
  Sender: TBaseVirtualTree; Node: PVirtualNode; Kind: TVTImageKind;
  Column: TColumnIndex; var Ghosted: Boolean; var ImageIndex: Integer;
  var ImageList: TCustomImageList);
begin
  if Kind in [ikNormal, ikSelected] then
  begin
    if GetLaunchData(Node).IsFolder then
      ImageIndex := 0
    else begin
      ImageList := SmallSysImages;
      if Assigned(GetLaunchData(Node).NS) then
        ImageIndex := GetLaunchData(Node).NS.GetIconIndex(False, icSmall, True)
      else
        ImageIndex := DefaultSystemImageIndex(diUnknownFile);
    end
  end
end;

procedure TTntFormUltraLaunchDesigner.TntFormKeyDown(Sender: TObject;
  var Key: Word; Shift: TShiftState);
begin
  case Key of
    VK_DELETE:
      if ActiveControl = VirtualStringTreeLauncher then
        ActionDelete.Execute;
    VK_INSERT:
      begin
        if GetKeyState(VK_CONTROL) and $8000 > 0 then
          ActionNewSubMenu.Execute
        else
          ActionNewApplication.Execute;
      end;
    VK_ESCAPE: SpTBXButtonCancel.Click;
  end
end;

procedure TTntFormUltraLaunchDesigner.VirtualStringTreeLauncherDragDrop(
  Sender: TBaseVirtualTree; Source: TObject; DataObject: IDataObject;
  Formats: TFormatArray; Shift: TShiftState; Pt: TPoint;
  var Effect: Integer; Mode: TDropMode);
var
  HDrop: TCommonHDrop;
  i: Integer;
  FileName, Ext: WideString;
  Data: TUltraLauncherData;
  Node: PVirtualNode;
  Link: TVirtualShellLink;
begin
  Node := nil;
  if DataObjectSupportsShell(DataObject) then
  begin
    HDrop := TCommonHDrop.Create;
    try
      HDrop.LoadFromDataObject(DataObject);
      for i := 0 to HDrop.FileCount - 1 do
      begin
        FileName := HDrop.FileName(i);
        Ext := WideExtractFileExt(FileName);
        if (WideLowerCase(Ext) = '.exe') or (WideLowerCase(Ext) = '.bat') or (WideLowerCase(Ext) = '.com') then
        begin
          Data := TUltraLauncherData.Create;
          Data.IsFolder := False;
          Data.Name := WideStripExt(WideExtractFileName(FileName));
          Data.Operation := 'Open';
          Data.ApplicationPath := FileName;
          Data.InitialPath := WideExtractFilePath(FileName);
          Data.NS := TNamespace.CreateFromFileName(FileName);
          Node := Sender.AddChild(nil, Data);
          Sender.Selected[Node] := True;
          if not GetLaunchData(Sender.DropTargetNode).IsFolder and (Mode = dmOnNode) then
            Mode := dmBelow;
        end else
        if (WideLowerCase(Ext) = '.lnk') then
        begin
          Link := TVirtualShellLink.Create(nil);
          try
            Link.ReadLink(FileName);
            if WideFileExists(Link.TargetPath) then
            begin
              Data := TUltraLauncherData.Create;
              Data.IsFolder := False;
              Data.Name := WideStripExt(WideExtractFileName(Link.TargetPath));
              Data.Operation := 'Open';
              Data.ApplicationPath := Link.TargetPath;
              Data.InitialPath := WideExtractFilePath(Link.WorkingDirectory);
              Data.Parameters := Link.Arguments;
              Data.NS := TNamespace.CreateFromFileName(FileName);
              Node := Sender.AddChild(nil, Data);
              Sender.Selected[Node] := True;
              if not GetLaunchData(Sender.DropTargetNode).IsFolder and (Mode = dmOnNode) then
                Mode := dmBelow;
            end
          finally
            Link.Free
          end
        end
      end
    finally
      HDrop.Free
    end;
  end;  
  case Mode of
    dmAbove: Sender.MoveTo(Sender.GetFirstSelected, Sender.DropTargetNode, amInsertBefore, False);
    dmOnNode: Sender.MoveTo(Sender.GetFirstSelected, Sender.DropTargetNode, amAddChildLast, False);
    dmBelow: Sender.MoveTo(Sender.GetFirstSelected, Sender.DropTargetNode, amInsertAfter, False);
  end;
  Sender.Expanded[Sender.DropTargetNode] := True;
  
  if Assigned(Node) then
    Sender.FocusedNode := Node;
end;

{ TUltraLauncherDataRoot }
constructor TUltraLauncherDataRoot.Create;
begin
  inherited Create;
  FIsRoot := True
end;

procedure TTntFormUltraLaunchDesigner.VirtualStringTreeLauncherFocusChanging(
  Sender: TBaseVirtualTree; OldNode, NewNode: PVirtualNode; OldColumn,
  NewColumn: TColumnIndex; var Allowed: Boolean);
begin
  if Assigned(OldNode) then
    StoreControls(GetLaunchData(OldNode));
end;

procedure TTntFormUltraLaunchDesigner.TntFormCloseQuery(Sender: TObject;
  var CanClose: Boolean);
var
  i: Integer;
  Found: Boolean;
begin
  Found := False;
  SpTBXComboBoxOperation.Text := WideLowerCase(SpTBXComboBoxOperation.Text);
  for i := 0 to SpTBXComboBoxOperation.Items.Count - 1 do
    if WideCompareText(SpTBXComboBoxOperation.Items[i], SpTBXComboBoxOperation.Text) = 0 then
      Found := True;
  if not Found then
    SpTBXComboBoxOperation.Items.Add(SpTBXComboBoxOperation.Text);

  SpTBXComboBoxOperation.Items.SaveToFile(AppFilesPath + '\' + STR_LAUNCH_VERBS_FILE)
end;

procedure TTntFormUltraLaunchDesigner.SpTBXListBoxMacrosDblClick(
  Sender: TObject);
begin
  SelectMacro
end;

procedure TTntFormUltraLaunchDesigner.TntFormCreate(Sender: TObject);
begin
  BorderStyle := bsSizeable;
end;

end.
